/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package actions;

import java.awt.event.ActionEvent;
import java.io.IOException;
import dialogs.NouvelleFigureAvecRepereDlg;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CCalcul;
import objets.CConstante;
import objets.CElementGenerique;
import objets.CImplementationProto;
import objets.CListeObjets;
import objets.CMathGrafDoc;
import objets.CPrototype;
import types.UniteAngle;

@SuppressWarnings("serial")
public class ActionFichierNouvelleFigureAvecRepere extends MtgAction {
	public ActionFichierNouvelleFigureAvecRepere(MtgFrame frameInit) {
		super(frameInit, null, getString("chaineMenuNouvelleFigureAvecRepere"),
				getString("chaineTipNouvelleFigureAvecRepere"), MtgFrame.iconNouvelleFigureAvecRepere, null);
	}
	
  @Override
  public int idMenu() {
    return 33013;
  }	
	
	public void actionPerformed(ActionEvent e) {
		MtgInternalFrame cadreActif = frame.cadreActif();
		// Spcial version Applet : on regarde si la figure prcdente a t modifie et si oui on demande confirmation
		if (cadreActif.document.isDirty()) {
			if (!frame.dialogueConfirmation_YES_NO(getString("ch184"))) return;
		}
		cadreActif.retireComposants(); // Ajout version 4.5.2
		CMathGrafDoc doc = new CMathGrafDoc(frame);
		doc.listePrincipale.ajouteConstantePi();
		doc.listePrincipale.uniteAngle = frame.pref_angleUnit;
		NouvelleFigureAvecRepereDlg dlg = new NouvelleFigureAvecRepereDlg(frame, doc.listePrincipale);
		dlg.setLocationRelativeTo(null);
		dlg.setVisible(true);
		if (dlg.retourOK()) {
			// Modification version 3.5 : Si la prcdente fentre a t maximise, on maximise la nouvelle
			// boolean maximise = cadreActif == null ? false : cadreActif.isMaximum(); Supprim version Applet
			// MtgInternalFrame cadre = new MtgInternalFrame(frame, "MathGraph" + Integer.toString(frame.instanceNouvelleFenetre()), doc); Supprim version Applet
			// Dimension sizefen = new Dimension(cadre.getWidth(), cadre.getHeight());  Supprim version Applet
			// frame.mtgDesktopPane.add(cadre); Supprim version Applet
			/*mtgDesktopPane.setSelectedFrame(cadre);
	    try {
	      cadre.setSelected(true);
	    } catch (java.beans.PropertyVetoException err) {};	          
			*/
			// Modification version 3.5.1
			/*  Supprim version Applet
			frame.mtgDesktopPane.getDesktopManager().activateFrame(cadre);
			try {
				cadre.setSelected(true);
			} catch (PropertyVetoException e1) {
			}
			if (maximise) {
				try {
					cadre.setMaximum(true);
				} catch (PropertyVetoException e1) {
					frame.mtgDesktopPane.getDesktopManager().maximizeFrame(cadre);
				}
			}
			else {
				frame.preparePositionPourFenenetre(sizefen);
				cadre.setLocation(frame.xfenetre, frame.yfenetre);
			}
			*/
			cadreActif.setDocument(doc);
			cadreActif.creePaneAffichageVariables();	// Dtruira un ventuel panneau d'affichage de variables
			frame.metAJourIconesEtMenus(doc);
			// cadreActif.redessineAvecCalcul(); // Modifi version 4.7.2.1 pour envoyer un ordre repaint
			frame.recalculateAndRedraw(false);
			frame.activeOutilCapture();
			if (dlg.rbGraduationSimple.isSelected()) {
				CListeObjets liste = cadreActif.listePr;
	  		String nom1 = "nbgradx";
	  		CCalcul calcul1 = new CCalcul(liste, null, false, nom1, "20", new CConstante(liste, 20));
	  		frame.ajouteElement(calcul1);
	  		String nom2 = "nbgrady";
	  		CCalcul calcul2 = new CCalcul(liste, null, false, nom2, "20", new CConstante(liste, 20));
	  		frame.ajouteElement(calcul2);
	    	CPrototype proto = frame.chargeConstructionDansDossierMtgFrame("GraduationAxesRepere.mgc");
	    	//On implmente maintenant ce prototype dans la figure
	  		((CElementGenerique)proto.get(0)).elementAssocie = liste.premierRepere();
	  		((CElementGenerique)proto.get(1)).elementAssocie = calcul1;
	  		((CElementGenerique)proto.get(2)).elementAssocie = calcul2;
	  		CImplementationProto impProto = new CImplementationProto(liste, proto);
	  		impProto.implemente(liste, proto, cadreActif);
	  		proto.retireTout(); // Ajout version 4.6.2
	  		frame.cadreActif().redessineAvecCalcul();
			}
			else {
				if (dlg.rbGraduationTrigo.isSelected()) {
					CListeObjets liste = cadreActif.listePr;
					liste.uniteAngle = UniteAngle.Radian;
		  		String nom1 = "nbgradx";
		  		CCalcul calcul1 = new CCalcul(liste, null, false, nom1, "20", new CConstante(liste, 20));
		  		frame.ajouteElement(calcul1);
		  		String nom2 = "nbgrady";
		  		CCalcul calcul2 = new CCalcul(liste, null, false, nom2, "20", new CConstante(liste, 20));
		  		frame.ajouteElement(calcul2);
		    	CPrototype proto = frame.chargeConstructionDansDossierMtgFrame("GraduationReperePourTrigo.mgc");
		    	//On implmente maintenant ce prototype dans la figure
		  		((CElementGenerique)proto.get(0)).elementAssocie = liste.premierRepere();
		  		((CElementGenerique)proto.get(1)).elementAssocie = calcul1;
		  		((CElementGenerique)proto.get(2)).elementAssocie = calcul2;
		  		CImplementationProto impProto = new CImplementationProto(liste, proto);
		  		impProto.implemente(liste, proto, cadreActif);
		  		frame.cadreActif().redessineAvecCalcul();
					
				}
			}
			try {
				cadreActif.gestionnaire.initialise();
			} catch (IOException e1) {}
			
		}
		else {
			doc.listePrincipale = null;
			doc = null;
			// System.gc(); // Supprim version 4.6.2
		}
	}
}
