/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import javax.swing.JFileChooser;
import javax.swing.KeyStroke;
import javax.swing.filechooser.FileSystemView;

import dialogs.MtgFileView;
import objets.CMathGrafDoc;
import types.NatureObjet;
import types.UniteAngle;
import entreesSorties.MathGraphDataInputStream;
import mathgraph32.Global;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

/**
 * 
 * @author Yves
 */
@SuppressWarnings("serial")
public class ActionFichierOuvrir extends MtgAction {
	public ActionFichierOuvrir(MtgFrame frameInit) {
		super(frameInit, null, getString("chaineMenuFichierOuvrir"), getString("chaineTipFichierOuvrir"),
				MtgFrame.iconOuvrir, KeyStroke.getKeyStroke("control O"));
	}

	@Override // Ajout version 4.3
  public boolean estDansBarreHorizontale() {
  	return true;
  }
	
	public void actionPerformed(ActionEvent e) {
		// Spcial version Applet : on regarde si la figure prcdente a t modifie et si oui on demande confirmation
		if (frame.cadreActif().document.isDirty()) {
			if (!frame.dialogueConfirmation_YES_NO(getString("ch184"))) return;
		}
		class MyFilter extends javax.swing.filechooser.FileFilter {
			public boolean accept(File file) {
				if (file.isDirectory())
					return true;
				String filename = file.getName();
				return filename.endsWith(Global.GLOBAL_COMMA_MGF) || filename.endsWith(Global.GLOBAL_COMMA_MGJ);
			}

			public String getDescription() {
				return "*.mgf;*.mgj";
			}
		}
		// Modifi version Applet
		MtgInternalFrame cadre = frame.cadreActif();
		JFileChooser fc;
		if (MtgFrame.repertoireFichiers.isEmpty()) {
			FileSystemView vueSysteme = FileSystemView.getFileSystemView(); 
			File defaut = vueSysteme.getDefaultDirectory(); 
			fc = new JFileChooser(defaut);
		}
		else {
			fc = new JFileChooser(MtgFrame.repertoireFichiers);
		}
		fc.setFileView(new MtgFileView());
		fc.setFileFilter(new MyFilter());
		// int rVal = fc.showOpenDialog(frame); // Modifi version 4.7 spcial applet
		int rVal = fc.showOpenDialog(null);
    if(rVal == JFileChooser.APPROVE_OPTION) {		
			File file = fc.getSelectedFile();
			String ch = file.getAbsolutePath();
			String chlc = ch.toLowerCase();
			if (!chlc.endsWith(Global.GLOBAL_COMMA_MGJ) && !chlc.endsWith(Global.GLOBAL_COMMA_MGF)) {
				ch = ch + Global.GLOBAL_COMMA_MGJ;
				file = new File(ch);
			}			
			if (file != null) {
				// MtgInternalFrame cad = frame.cadreParChemin(file.getAbsolutePath()); 
				// if (cad == null) { // On n'ouvre pas un fichier dj ouvert Supprim version Applet
				try {
					// On actualise la variable repertoireFichiers du MtgFrame pre pour
					// qu' la prochaine ouverture
					// on soit plac dans le mme rpertoire
					// String ch = file.getAbsolutePath();
					int ind = ch.lastIndexOf(File.separatorChar);
					MtgFrame.repertoireFichiers = ch.substring(0, ind);
					FileInputStream fis = new FileInputStream(file);
					MathGraphDataInputStream inps = new MathGraphDataInputStream(fis);
					try {
						// MtgInternalFrame cadreActif = frame.cadreActif();
						// if (cadreActif != null) cadreActif.cacheDialogueAffichageVariables();
						String titre = ch.substring(ind + 1);
						cadre.retireComposants(); // Ajout version 4.5.2
						CMathGrafDoc doc = new CMathGrafDoc(frame);
						doc.path = ch;
						try {
							doc.chargeDepuisFlux(inps, false); // false car pas de chargemen en ligne
						}
						finally {
							inps.close();				
						}
						// cadre = new MtgInternalFrame(frame, "", doc); Modifi version Applet
						cadre.setDocument(doc);
						cadre.setTitle(titre);
						cadre.path = file.getAbsolutePath();
						cadre.gestionnaire.initialise();
						// cadre.setVisible(true);
						// frame.mtgDesktopPane.add(cadre); Supprim version Applet
						// On adapte les icnes et le menu au document charg
						frame.metAJourIconesEtMenus(doc);
						frame.labelUniteAngle.setText((cadre.listePr.uniteAngle == UniteAngle.Degre) ? getString("radioDegre") : getString("radioRadian"));
						// Les deux lignes suivantes rajoutes version 4.4
						if (cadre.listePr.nombreObjetsParNature(NatureObjet.NMacro) != 0) frame.activeOutilExecutionMacro();
						else frame.activeOutilCapture();
					} catch (IOException ioerr) {
						frame.dialogueAvertissement(getString("ch35"));
						cadre.listePr.retablitComposants();
						return;
					}
				} catch (FileNotFoundException err) {
					frame.dialogueAvertissement(getString("ch35"));
					return;
				}
				cadre.creePaneAffichageVariables();	// Ajout version Applet		
				// cadre.redessineAvecCalcul();  //// Modifi version 4.7.2.1 pour envoyer un oordre repaint
				frame.recalculateAndRedraw(false);
				//} else {
				//	frame.activeCadre(cad);
				//}
			}
    }
	}

}
