/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

import objets.CListeObjets;
import objets.CValeurAffichee;
import types.NatureCarSpe;

import mathgraph32.MtgFrame;
// Modifi version 3.5 : Quand l'affichage de valeur est un objet final de construction on ne donne pas la
// possibilit de choisir la valeur  afficher
@SuppressWarnings("serial")
public class AffichageValeurDlg extends MtgDlg{
	private boolean modification; // True si on modifie les coordonnes d'un point dj cr
	private CValeurAffichee valeurAffichee;
	private int indiceMax;
	private EditeurIntituleValeurReelleOuComplexe editeur;
	private PaneListeDecimales paneListeDecimales;
	private JButton buttonListeValeurs;
  private javax.swing.JButton btnCancel;
  private javax.swing.JButton btnOK;
  private TextFieldAvecCarSpeciaux editeurEnTete;
  private TextFieldAvecCarSpeciaux editeurSuiviDe;
  private PaneTaillePolice paneTaillePolice;
  private PaneModeAffichage paneModeAffichage;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
	
	public AffichageValeurDlg(MtgFrame frameInit, CValeurAffichee valeurAfficheeInit, boolean modificationInit)	{
		super(getString("affichageValeurDlg1"), true, frameInit);
		valeurAffichee = valeurAfficheeInit;
		modification = modificationInit;
		// Spcial version applet
    // CListeObjets liste = frame.cadreActif().listePr;
		CListeObjets liste = frame.listePourConstruction();
    // Dans le cas ou on cre un nouvel objet, celui-ci n'a pas encore t ajout
    //  la liste des objets de l'applet.
    if (modification) indiceMax = liste.indexOf(valeurAfficheeInit) - 1;
    else indiceMax = liste.size() - 1;
 		editeur = new EditeurIntituleValeurReelleOuComplexe(frame, indiceMax);
 		JLabel label1 = new JLabel(getString("affichageValeurDlg2"));
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    
 		buttonListeValeurs = new JButton(getString("listeValeurs"));
 		buttonListeValeurs.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnListeValeursActionPerformed(evt);
      }
 		});
 		// Modification version 3.5 : Si l'afficgae de valeur est un objet final de construction
 		// on ne donne pas la possibilit de changer la valeur qu'on affiche mais seulement
 		// la faon dont elle est affiche
 		if (valeurAfficheeInit.estElementFinal) {
 			editeur.setEditable(false);
 			buttonListeValeurs.setVisible(false);
 		}
 		paneListeDecimales = new PaneListeDecimales(valeurAffichee.nombreDecimales);
 		JPanel paneLeftHigh = new JPanel();
 		paneLeftHigh.setLayout(new BoxLayout(paneLeftHigh, BoxLayout.Y_AXIS));
 		paneLeftHigh.add(label1);
 		paneLeftHigh.add(Box.createRigidArea(new Dimension(0,3)));
 		paneLeftHigh.add(editeur);
 		paneLeftHigh.add(Box.createRigidArea(new Dimension(0,3)));
 		paneLeftHigh.add(buttonListeValeurs);
 		JPanel paneRightHigh = new JPanel();
 		paneRightHigh.setLayout(new BoxLayout(paneRightHigh, BoxLayout.Y_AXIS));
 		paneRightHigh.add(btnOK);
		paneRightHigh.add(Box.createRigidArea(new Dimension(0, 5))); 		 		
 		paneRightHigh.add(btnCancel);
 		JPanel paneHigh = new JPanel();
 		paneHigh.setLayout(new FlowLayout());
 		paneHigh.add(paneLeftHigh);
 		paneHigh.add(paneListeDecimales);
 		paneHigh.add(paneRightHigh);
 		
 		JLabel label2 = new JLabel(getString("affichageValeurDlg3"));
 		JLabel label3 = new JLabel(getString("affichageValeurDlg4"));
 		editeurEnTete = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
 		editeurEnTete.setColumns(25);
 		editeurSuiviDe = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
 		editeurSuiviDe.setColumns(25);
 		if (modification) {
 			editeur.setText(valeurAffichee.valeurAssociee.nom());
 			editeurEnTete.setText(valeurAffichee.enTete);
 			editeurSuiviDe.setText(valeurAffichee.postChaine);
 		}
 		
 		JPanel paneCenter = new JPanel();
    GridBagConstraints constraints = new GridBagConstraints();
    paneCenter.setLayout(new GridBagLayout());
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 20, 35, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneCenter.add(label2, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 80, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    constraints.insets.top = 0;
    paneCenter.add(editeurEnTete, constraints);
     setConstraints(constraints, 0, 1, 1, 1, 0, 35, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneCenter.add(label3, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneCenter.add(editeurSuiviDe, constraints);
    
    paneTaillePolice = new PaneTaillePolice(valeurAffichee.indiceTaillePolice);
    paneModeAffichage = new PaneModeAffichage(frame, valeurAffichee, true); // Modifi version 4.5.2
    JPanel paneLow = new JPanel();
    paneLow.setLayout(new FlowLayout());
    paneLow.add(paneTaillePolice);
    paneLow.add(paneModeAffichage);
    
    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(paneHigh, "North");
    getContentPane().add(paneCenter, "Center");
    getContentPane().add(paneLow, "South");
    getRootPane().setDefaultButton(btnOK);

    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(5);
    order.add(editeur);
    order.add(editeurEnTete);
    order.add(editeurSuiviDe);
    order.add(btnOK);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    
    pack();
	}
	
	public final void btnListeValeursActionPerformed(ActionEvent evt) {
    ListeValeursReellesOuComplexesRemplacementDlg dlg = new ListeValeursReellesOuComplexesRemplacementDlg
    	(frame, editeur, indiceMax, false);
    dlg.setLocationRelativeTo(this); // Pour centrer la boite de dialogue sur l'cran
    dlg.setVisible(true);		
	}
	
  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
  	if (!valeurAffichee.estElementFinal) {
	  	if (editeur.verifieValidite()) valeurAffichee.valeurAssociee = editeur.pointeurValeur; 
	  	else return;
  	}
		valeurAffichee.nombreDecimales = paneListeDecimales.nombreDecimales();
		valeurAffichee.enTete = editeurEnTete.getText();
		valeurAffichee.postChaine = editeurSuiviDe.getText();
		valeurAffichee.indiceTaillePolice = paneTaillePolice.indiceTaille();
		valeurAffichee.encadrement = paneModeAffichage.styleEncadrement();
		valeurAffichee.effacementFond = paneModeAffichage.effacementFond();
		valeurAffichee.couleurFondAffichage = paneModeAffichage.couleurFond();
		// Ajouts version 4.5
		valeurAffichee.alignementHorizontal = paneModeAffichage.alignementHorizontal();
		valeurAffichee.alignementVertical = paneModeAffichage.alignementVertical();
		//
		frame.pref_IndiceTaillePoliceAffichages = valeurAffichee.indiceTaillePolice;
		frame.cadreActif().couleurFondAffichage = valeurAffichee.couleurFondAffichage;
		setRetourOK(true);
		dispose();
  }
	
  private void btnCancelActionPerformed(ActionEvent evt) {                                          
    dispose();
  }         
}
