/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import objets.CMathGrafDoc;
import mathgraph32.MtgFrame;
import mathgraph32.MtgMenu;

@SuppressWarnings("serial")
public class ChoixOutilsDlg extends MtgDlg {

	private MtgMenu menuFichier, menuEdition, menuCreer, menuCalculs, menuModifier;
	private MtgMenu menuMacros,menuConstructions, menuHelp;
	private JLabel label1;
	private JRadioButton radioItemCochesInterdits;
	private JRadioButton radioItemCochesPermis;
	private MtgMenuTree tree;
	private DefaultTreeModel model;
	private DefaultMutableTreeNode principalNode;
	private JScrollPane scrollPaneTree;
	private ButtonGroup btnGroup;
	private JButton btnCocherTout;
	private JButton btnDecocherTout;
	private boolean bInitialItemCochesInterdits;
	private DefaultMutableTreeNode nodeFichier;
	private DefaultMutableTreeNode nodeEdition;
	private DefaultMutableTreeNode nodeCreer;
	private DefaultMutableTreeNode nodeCalculs;
	private DefaultMutableTreeNode nodeModifier;
	// private DefaultMutableTreeNode nodeMouvoir;
	private DefaultMutableTreeNode nodeMacros;
	// private DefaultMutableTreeNode nodeOptions;
	private DefaultMutableTreeNode nodeConstructions;
	private DefaultMutableTreeNode nodeHelp;
	
	
	public ChoixOutilsDlg(MtgFrame frame) {
		super(getString("chaineMenuChoixOutils"), true, frame);
		initComponents();
		bInitialItemCochesInterdits = radioItemCochesInterdits.isSelected();
	}
	
	private void initComponents() {
    JButton btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    JButton btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
		
		label1 = new JLabel(getString("preferencesMenu1"));
		btnGroup = new ButtonGroup();
		radioItemCochesInterdits = new JRadioButton(getString("preferencesMenu2"));
		// Ajout version 4.3. Si on clique sur le bouton on dcochera tout.
		radioItemCochesInterdits.addActionListener(new ActionListener() {	
			public void actionPerformed(ActionEvent arg0) {
				btnDecocherToutPerformed(true);			
			}
		});
		btnGroup.add(radioItemCochesInterdits);
		radioItemCochesPermis = new JRadioButton(getString("preferencesMenu3"));
		// Ajout version 4.3. Si on clique sur le bouton on cochera tout.
		radioItemCochesPermis.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				btnCocherToutPerformed(true);
			}
		});
		btnGroup.add(radioItemCochesPermis);
		btnCocherTout = new JButton(getString("btnCocherTout"));
		btnCocherTout.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCocherToutPerformed(false);
	    }
	  });    
		btnDecocherTout = new JButton(getString("btnDecocherTout"));
		btnDecocherTout.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnDecocherToutPerformed(false);
	    }
	  });    
		prepareItems();
		tree.setCellRenderer(new RenduComposant());
		tree.setCellEditor(new EditComposant(tree));
		tree.setEditable(true);
		tree.setBackground(Color.white);

		scrollPaneTree = new JScrollPane(tree, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPaneTree.setPreferredSize(new Dimension(400, 200));
		radioItemCochesInterdits.setSelected(frame.cadreActif().document.itemsCochesInterdits);
		radioItemCochesPermis.setSelected(!radioItemCochesInterdits.isSelected());
    setLayout(new GridBagLayout());
    
    JPanel panePrincipal = new JPanel();
    panePrincipal.setLayout(new BorderLayout());
    JPanel paneRight = new JPanel();
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    JPanel paneLeft = new JPanel();
    paneLeft.setLayout(new GridBagLayout());
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    MtgDlg.setConstraints(constraints, 0, 0, 1, 1, 100, 60, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST);
    paneLeft.add(scrollPaneTree, constraints);
    MtgDlg.setConstraints(constraints, 0, 1, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(label1, constraints);
    MtgDlg.setConstraints(constraints, 0, 2, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(radioItemCochesInterdits, constraints);
    MtgDlg.setConstraints(constraints, 0, 3, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(radioItemCochesPermis, constraints);		
    MtgDlg.setConstraints(constraints, 0, 4, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    paneLeft.add(btnCocherTout, constraints);		
    MtgDlg.setConstraints(constraints, 0, 5, 1, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    paneLeft.add(btnDecocherTout, constraints);
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    panePrincipal.add(paneLeft,BorderLayout.WEST);
    panePrincipal.add(paneRight, BorderLayout.EAST);
    setContentPane(panePrincipal);
    pack();
    
	}
	
	
	private void prepareItems() {
		CMathGrafDoc docRef = frame.cadreActif().document;
		menuFichier = new MtgMenu(getString("menuFichier"));
		menuEdition = new MtgMenu(getString("menuEdition"));
		menuCreer = new MtgMenu(getString("menuCreer"));
		menuModifier = new MtgMenu(getString("menuModifier"));
		// menuMouvoir = new MtgMenu(getString("menuMouvoir"));
		menuCalculs = new MtgMenu(getString("menuCalculs"));
		menuMacros = new MtgMenu(getString("menuMacros"));
		// menuOptions = new MtgMenu(getString("menuOptions"));		
		menuConstructions = new MtgMenu(getString("menuConstructions"));
		menuHelp = new MtgMenu(getString("menuHelp"));
		
		// Nouveau version 4.3 : On n'autorise que les items du niveau de fonctionnement choisi
		// CMathGrafDoc doc = frame.docPreferencesNiveauPredefini[frame.pref_niveau];
		// Modif version 4.6 : Au niveau lmaximum tous les outils doivet tre permis
		CMathGrafDoc doc = frame.pref_niveau > 1 ? frame.docPreferencesNiveauPredefini[frame.pref_niveau] : null;
		frame.menuFichierInitial.addCloneAvecCheck(menuFichier, doc, docRef);
		frame.menuEditionInitial.addCloneAvecCheck(menuEdition, doc, docRef);
		frame.menuCreerInitial.addCloneAvecCheck(menuCreer, doc, docRef);
		frame.menuCalculsInitial.addCloneAvecCheck(menuCalculs, doc, docRef);
		frame.menuModifierInitial.addCloneAvecCheck(menuModifier, doc, docRef);
		// frame.menuMouvoirInitial.addCloneAvecCheck(menuMouvoir, doc, docRef);
		frame.menuMacrosInitial.addCloneAvecCheck(menuMacros, doc, docRef);
		// frame.menuOptionsInitial.addCloneAvecCheck(menuOptions, doc, docRef);
		frame.menuConstructionsInitial.addCloneAvecCheck(menuConstructions, doc, docRef);
		frame.menuHelpInitial.addCloneAvecCheck(menuHelp, doc, docRef);
		principalNode = new DefaultMutableTreeNode();
		creeNoeuds();
		model = new DefaultTreeModel(principalNode);
		tree = new MtgMenuTree(model);
	}

	private void btnOKActionPerformed() {
		CMathGrafDoc doc = frame.cadreActif().document;
		doc.initialiseItemsMenu();
		CMathGrafDoc docRef = new CMathGrafDoc(frame);	
		doc.itemsCochesInterdits = radioItemCochesInterdits.isSelected();
		docRef.itemsCochesInterdits = doc.itemsCochesInterdits;
		if (doc.itemsCochesInterdits) {
		// Nouveau version 4.3 : On interdit les items interdits pour le niveau en cours
			MtgMenu menuFichier0, menuEdition0, menuCreer0, menuCalculs0, menuModifier0;
			MtgMenu menuMacros0, menuConstructions0, menuHelp0;
			menuFichier0 = new MtgMenu(getString("menuFichier"));
			menuEdition0 = new MtgMenu(getString("menuEdition"));
			menuCreer0 = new MtgMenu(getString("menuCreer"));
			menuModifier0 = new MtgMenu(getString("menuModifier"));
			// menuMouvoir0 = new MtgMenu(getString("menuMouvoir"));
			menuCalculs0 = new MtgMenu(getString("menuCalculs"));
			menuMacros0 = new MtgMenu(getString("menuMacros"));
			// menuOptions0 = new MtgMenu(getString("menuOptions"));
			menuConstructions0 = new MtgMenu(getString("menuConstructions"));
			menuHelp0 = new MtgMenu(getString("menuHelp"));
			frame.menuFichierInitial.addCloneAvecCheck(menuFichier0, null, null);
			frame.menuEditionInitial.addCloneAvecCheck(menuEdition0, null, null);
			frame.menuCreerInitial.addCloneAvecCheck(menuCreer0, null, null);
			frame.menuCalculsInitial.addCloneAvecCheck(menuCalculs0, null, null);
			frame.menuModifierInitial.addCloneAvecCheck(menuModifier0, null, null);
			// frame.menuMouvoirInitial.addCloneAvecCheck(menuMouvoir0, null, null);
			frame.menuMacrosInitial.addCloneAvecCheck(menuMacros0, null, null);
			// frame.menuOptionsInitial.addCloneAvecCheck(menuOptions0, null, null);
			frame.menuConstructionsInitial.addCloneAvecCheck(menuConstructions0, null, null);
			frame.menuHelpInitial.addCloneAvecCheck(menuHelp0, null, null);
			menuFichier0.ajouteTousItemsADocument(doc);
			menuEdition0.ajouteTousItemsADocument(doc);
			menuCreer0.ajouteTousItemsADocument(doc);
			menuCalculs0.ajouteTousItemsADocument(doc);
			menuModifier0.ajouteTousItemsADocument(doc);
			// menuMouvoir0.ajouteTousItemsADocument(doc);
			menuMacros0.ajouteTousItemsADocument(doc);
			// menuOptions0.ajouteTousItemsADocument(doc);
			menuConstructions0.ajouteTousItemsADocument(doc);
			menuHelp0.ajouteTousItemsADocument(doc);
		}
		
		menuFichier.adapteDocument(doc);
		menuEdition.adapteDocument(doc);
		menuCreer.adapteDocument(doc);
		menuCalculs.adapteDocument(doc);
		menuModifier.adapteDocument(doc);
		// menuMouvoir.adapteDocument(doc);
		menuMacros.adapteDocument(doc);
		// menuOptions.adapteDocument(doc);
		menuConstructions.adapteDocument(doc);
		menuHelp.adapteDocument(doc);
		
		frame.metAJourIconesEtMenus(doc);
		
		setRetourOK( menuFichier.modifie() || menuEdition.modifie() || menuCreer.modifie() || menuCalculs.modifie()
			|| menuModifier.modifie() || menuMacros.modifie()
			|| menuConstructions.modifie()
			|| (bInitialItemCochesInterdits != radioItemCochesInterdits.isSelected()));
		dispose();
		frame.validate(); // Pour rafficher la barre d'outils
		frame.recalculateAndRedraw(false); // Rajout version 4.7.2.1
	}
	
	private void btnCancelActionPerformed() {
		dispose();
	}
	
	private void creeNoeuds() {
		nodeFichier = new DefaultMutableTreeNode(menuFichier.getText());
		menuFichier.ajouteNoeud(nodeFichier);
		nodeEdition = new DefaultMutableTreeNode(menuEdition.getText());
		menuEdition.ajouteNoeud(nodeEdition);
		nodeCreer = new DefaultMutableTreeNode(menuCreer.getText());
		menuCreer.ajouteNoeud(nodeCreer);
		nodeCalculs = new DefaultMutableTreeNode(menuCalculs.getText());
		menuCalculs.ajouteNoeud(nodeCalculs);
		nodeModifier = new DefaultMutableTreeNode(menuModifier.getText());
		menuModifier.ajouteNoeud(nodeModifier);
		// nodeMouvoir = new DefaultMutableTreeNode(menuMouvoir.getText());
		// menuMouvoir.ajouteNoeud(nodeMouvoir);
		nodeMacros = new DefaultMutableTreeNode(menuMacros.getText());
		menuMacros.ajouteNoeud(nodeMacros);
		// nodeOptions = new DefaultMutableTreeNode(menuOptions.getText());
		// menuOptions.ajouteNoeud(nodeOptions);
		nodeConstructions = new DefaultMutableTreeNode(menuConstructions.getText());
		menuConstructions.ajouteNoeud(nodeConstructions);
		nodeHelp = new DefaultMutableTreeNode(menuHelp.getText());
		menuHelp.ajouteNoeud(nodeHelp);
		principalNode.add(nodeFichier);
		principalNode.add(nodeEdition);
		principalNode.add(nodeCreer);
		principalNode.add(nodeCalculs);
		// principalNode.add(nodeMouvoir);
		principalNode.add(nodeModifier);
		principalNode.add(nodeMacros);
		// principalNode.add(nodeOptions);
		principalNode.add(nodeConstructions);
		principalNode.add(nodeHelp);
	}
	
	private void btnCocherToutPerformed(boolean verbose) {
		if (verbose) frame.dialogueAvertissement(this, getString("outilsReinitialises"));
		boolean b = radioItemCochesInterdits.isSelected();
		menuFichier.selectAll(b);
		menuEdition.selectAll(b);
		menuCreer.selectAll(b);
		menuCalculs.selectAll(b);
		menuModifier.selectAll(b);
		// menuMouvoir.selectAll();
		menuMacros.selectAll(b);
		// menuOptions.selectAll(b);
		menuConstructions.selectAll(b);
		menuHelp.selectAll(b);
		principalNode.removeAllChildren();
		creeNoeuds();
		tree.updateUI();
	}

	private void btnDecocherToutPerformed(boolean verbose) {
		if (verbose) frame.dialogueAvertissement(this, getString("outilsReinitialises"));
		boolean b = !radioItemCochesInterdits.isSelected();
		menuFichier.deselectAll(b);
		menuEdition.deselectAll(b);
		menuCreer.deselectAll(b);
		menuCalculs.deselectAll(b);
		menuModifier.deselectAll(b);
		// menuMouvoir.deselectAll();
		menuMacros.deselectAll(b);
		// menuOptions.deselectAll(b);
		menuConstructions.deselectAll(b);
		menuHelp.deselectAll(b);
		principalNode.removeAllChildren();
		creeNoeuds();
		tree.updateUI();
	}
	
}

