/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import mathgraph32.MtgFrame;
import objets.CPrototype;
import types.FonteNom;

@SuppressWarnings("serial")
public class ConstructionDlg extends MtgDlg {
	private boolean modification; // True si on modifie les coordonnes d'une construction dj cre
	private CPrototype prototype;
	private JLabel label1, label2;
	private JTextField editeurNomPrototype;
	private JTextArea editeurinfoPrototype;
	private JScrollPane scrollPaneInfo;
	private JButton btnOK, btnCancel;
	String nomProto;

	public ConstructionDlg(MtgFrame frameInit, CPrototype prototypeInit, boolean modificationInit)	{
		super(getString("constructionDlg1"), true, frameInit);
		prototype = prototypeInit;
		modification = modificationInit;
		initComponents();
		getRootPane().setDefaultButton(btnOK);
	}
	
	private void initComponents() {
		label1 = new JLabel(getString("choixPrototypeDansRepertoireDlg2"));
		label2 = new JLabel(getString("Infos"));
  	editeurNomPrototype = new JTextField(30);
    editeurinfoPrototype = new JTextArea();
    editeurinfoPrototype.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1));    
    scrollPaneInfo = new JScrollPane(editeurinfoPrototype, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    editeurinfoPrototype.setColumns(20);
    editeurinfoPrototype.setLineWrap(false);
    editeurinfoPrototype.setRows(12);
    scrollPaneInfo.setPreferredSize(new Dimension(450, 240));    

    if (modification) {
    	editeurNomPrototype.setText(prototype.nom);
    	editeurinfoPrototype.setText(prototype.commentaire);
    	editeurinfoPrototype.setCaretPosition(0);
    }
    
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());    
    JPanel panePrincipal = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    JPanel paneRight = new JPanel();
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 40, 10, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(editeurNomPrototype, constraints);
    setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label2, constraints);
    setConstraints(constraints, 0, 2, 2, 1, 0, 70, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneInfo, constraints);
    setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.BOTH, GridBagConstraints.NORTH);
    panePrincipal.add(paneRight, constraints);
    
    setContentPane(panePrincipal);
    pack();
	}
	
  private void btnOKActionPerformed() {
  	if (validationNomProto()) {
  		prototype.nom = nomProto;
  		prototype.commentaire = editeurinfoPrototype.getText();
  		setRetourOK(true);
  		dispose();
  	}
	}
  
  private void btnCancelActionPerformed() {
  	dispose();
  }
  
  private boolean validationNomProto() {
  	StringBuffer stb = new StringBuffer(editeurNomPrototype.getText());
  	// On retire les ventuels espaces de fin
  	while (stb.charAt(stb.length() - 1) == ' ') stb.deleteCharAt(stb.length() - 1);
  	if (stb.length() == 0) return false;
  	for (int i = 0; i < stb.length(); i++) {
  		char car = stb.charAt(i);
  		if (!FonteNom.chiffre(car) && !FonteNom.lettre(car) && (car != ' ') && (car != '_')) {
    		editeurNomPrototype.requestFocus();
    		dialogueAvertissement(getString("ch1"));
  			return false;
  		}
  	}
  	nomProto = stb.toString();
  	// On regarde si ce n'est pas le nom d'une autre construction de la figure
  	// Modifi version 3.5.1
  	// if (modification && frame.cadreActif().document.existePrototype(nomProto, prototype)) {
  	if (frame.cadreActif().document.existePrototype(nomProto, prototype)) {
  		editeurNomPrototype.requestFocus();
  		dialogueAvertissement(getString("ch147"));
			return false;
  	}
  	return true;
  }
}
