/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.text.NumberFormat;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import objets.CListeObjets;
import objets.CValeurAssocieeAVariable;
import types.FonteNom;
import types.NatureCarSpe;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class CourbeFonctionDlg extends MtgDlg {
	private javax.swing.JButton btnCancel;
	private javax.swing.JButton btnOK;
	private ListeFonctionsReelles listeFonctions;
	private ListeReperes listeReperes;
	private JScrollPane scrollPaneFonctions;
	private JScrollPane scrollPaneReperes;
	private JLabel label1, label2, label3, label4, label5, label6, label7, label8;
	private JLabel labela, labelb;
	private TextFieldAvecCarSpeciaux editeurNomPointLieGenerateur;
	private JTextField editeurNomAbscisse;
	private JTextField editeurNomImage;
	private JFormattedTextField editeurNombrePoints;
	private JTextArea infoFormule;
	private JRadioButton radioCourbeSurR;
	private JRadioButton radioCourbeSurab;
	private JRadioButton radioCourbeAPartirDea;
	private JRadioButton radioCourbeJusquea;
	private EditeurValeurDynamique editeura;
	private EditeurValeurDynamique editeurb;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsa;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsb;
	private ButtonGroup group;
	private JCheckBox checkPointLieGenerateurCache;
	private JCheckBox checkPointGenerateurCache;
	private JCheckBox checkGestionAutoDiscontinuites;
	private CValeurAssocieeAVariable vala;
	private CValeurAssocieeAVariable valb;

	public CourbeFonctionDlg(MtgFrame frameInit) {
		super(getString("CourbeFonctionDlg0"), true, frameInit);
		initComponents();
		editeurNombrePoints.setText(MtgFrame.nombrePointsPourLieuPointsParDefaut);
		checkPointLieGenerateurCache.setSelected(true);
		checkPointGenerateurCache.setSelected(true);
		checkGestionAutoDiscontinuites.setSelected(true);
		radioCourbeSurR.setSelected(true);
		// Par dfaut courbe sur R
		labela.setVisible(false);
		editeura.setVisible(false);
		paneListeValeurEtFonctionsa.setVisible(false);
		labelb.setVisible(false);
		editeurb.setVisible(false);
		paneListeValeurEtFonctionsb.setVisible(false);
		pack();
		// Spcial version applet
		// CListeObjets liste = frame.cadreActif().listePr;
		CListeObjets liste = frame.listePourConstruction();
		editeurNomPointLieGenerateur.setText(liste.genereNomPourPointOuDroiteAvecChiffre('x'));
		editeurNomAbscisse.setText(liste.genereNomPourCalcul("x", true));
		editeurNomImage.setText(liste.genereNomPourCalcul("y", true));
		getRootPane().setDefaultButton(btnOK);
	}

	private void initComponents() {
		btnOK = new javax.swing.JButton(getString("btnOK"));
		btnOK.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnOKActionPerformed();
			}
		});
		btnCancel = new javax.swing.JButton(getString("btnCancel"));
		btnCancel.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnCancelActionPerformed();
			}
		});
		btnOK.setMaximumSize(btnCancel.getPreferredSize());
		// Spcial version applet
		// CListeObjets liste = frame.cadreActif().listePr;
		CListeObjets liste = frame.listePourConstruction();
		int indmax = liste.size() - 1;
		listeFonctions = new ListeFonctionsReelles(liste, indmax, false);
		listeFonctions.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent evt) {
				listeFonctionsSelectionChanged();
			}
		});
		scrollPaneFonctions = new JScrollPane(listeFonctions, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		listeReperes = new ListeReperes(frame.cadreActif().listePr, indmax);
		scrollPaneReperes = new JScrollPane(listeReperes, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		label1 = new JLabel(getString("CourbeFonctionDlg1"), JLabel.LEFT);
		label2 = new JLabel(getString("CourbeFonctionDlg2"), JLabel.LEFT);
		label3 = new JLabel(getString("CourbeFonctionDlg3"), JLabel.LEFT);
		label4 = new JLabel(getString("informationFonction"), JLabel.LEFT);
		label5 = new JLabel(getString("CourbeFonctionDlg8"), JLabel.LEFT);
		label6 = new JLabel(getString("CourbeFonctionDlg9"), JLabel.LEFT);
		label7 = new JLabel(getString("CourbeFonctionDlg10"), JLabel.LEFT);
		label8 = new JLabel(getString("CourbeFonctionDlg14"), JLabel.LEFT);
		labela = new JLabel(getString("solutionEquationDlg5"));
		labelb = new JLabel(getString("solutionEquationDlg6"));
    infoFormule = new JTextArea();
    infoFormule.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1));    
    infoFormule.setEditable(false);
    infoFormule.setColumns(30);
    infoFormule.setRows(1);
    JScrollPane scrollPaneInfoFormule = new JScrollPane(infoFormule, JScrollPane.VERTICAL_SCROLLBAR_NEVER, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		NumberFormat format;
		format = NumberFormat.getInstance();
		format.setParseIntegerOnly(true);
		format.setGroupingUsed(false);
		editeurNombrePoints = new JFormattedTextField(format);
		editeurNombrePoints.setColumns(4);
		editeurNombrePoints.setText(MtgFrame.nombrePointsPourLieuPointsParDefaut);
		editeurNomPointLieGenerateur = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Grec);
		editeurNomPointLieGenerateur.setColumns(8);
		editeurNomAbscisse = new javax.swing.JTextField(null, 8);
		editeurNomImage = new javax.swing.JTextField(null, 8);
		radioCourbeSurR = new JRadioButton(getString("CourbeFonctionDlg4"));
		radioCourbeSurR.addActionListener(new ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				radioCourbeSurRActionPerformed();
			}
		});
		radioCourbeSurab = new JRadioButton(getString("CourbeFonctionDlg5"));
		radioCourbeSurab.addActionListener(new ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				radioCourbeSurabActionPerformed();
			}
		});
		radioCourbeAPartirDea = new JRadioButton(getString("CourbeFonctionDlg6"));
		radioCourbeAPartirDea.addActionListener(new ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				radioCourbeAPartirDeaActionPerformed();
			}
		});
		radioCourbeJusquea = new JRadioButton(getString("CourbeFonctionDlg7"));
		radioCourbeJusquea.addActionListener(new ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				radioCourbeJusqueaActionPerformed();
			}
		});

		vala = new CValeurAssocieeAVariable(liste);
		editeura = new EditeurValeurDynamique(frame, vala, null);
		editeura.setColumns(20);
		valb = new CValeurAssocieeAVariable(liste);
		editeurb = new EditeurValeurDynamique(frame, valb, null);
		editeurb.setColumns(20);
    paneListeValeurEtFonctionsa = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeura, true, indmax, false);
    paneListeValeurEtFonctionsb = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurb, true, indmax, false);
		group = new ButtonGroup();
		group.add(radioCourbeSurR);
		group.add(radioCourbeSurab);
		group.add(radioCourbeAPartirDea);
		group.add(radioCourbeJusquea);
		checkPointLieGenerateurCache = new JCheckBox(getString("CourbeFonctionDlg11"));
		checkPointGenerateurCache = new JCheckBox(getString("CourbeFonctionDlg12"));
		checkGestionAutoDiscontinuites = new JCheckBox(getString("GestionAutoDiscontinuites"));

		JPanel panePrincipal = new JPanel();
		JPanel paneRight = new JPanel();
		panePrincipal.setLayout(new GridBagLayout());
		paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));

		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 5;
		constraints.insets.top = 5;
		constraints.insets.left = 5;
		constraints.insets.right = 5;

		setConstraints(constraints, 0, 0, 1, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label2, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneFonctions, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneReperes, constraints);

		JPanel paneChoixTypeCourbe = new JPanel();
		paneChoixTypeCourbe.add(label3, constraints);
		paneChoixTypeCourbe.add(radioCourbeSurR, constraints);
		paneChoixTypeCourbe.add(radioCourbeSurab, constraints);
		paneChoixTypeCourbe.add(radioCourbeAPartirDea, constraints);
		paneChoixTypeCourbe.add(radioCourbeJusquea, constraints);

		setConstraints(constraints, 0, 2, 3, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneChoixTypeCourbe, constraints);

		JPanel paneab = new JPanel();
		paneab.setLayout(new GridBagLayout());
		setConstraints(constraints, 0, 0, 1, 1, 10, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneab.add(labela, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 40, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneab.add(editeura, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneab.add(paneListeValeurEtFonctionsa, constraints);
		setConstraints(constraints, 2, 0, 1, 1, 10, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneab.add(labelb, constraints);
		setConstraints(constraints, 3, 0, 1, 1, 40, 10, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneab.add(editeurb, constraints);
		setConstraints(constraints, 3, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneab.add(paneListeValeurEtFonctionsb, constraints);

		setConstraints(constraints, 0, 3, 3, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneab, constraints);

		JPanel pane1 = new JPanel();
		pane1.add(label4);
		pane1.add(scrollPaneInfoFormule);
		setConstraints(constraints, 0, 4, 3, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(pane1, constraints);

		JPanel pane2 = new JPanel();
		pane2.setLayout(new GridBagLayout());
		setConstraints(constraints, 0, 0, 1, 1, 60, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
		pane2.add(label5, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 40, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		pane2.add(editeurNomAbscisse, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 0, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
		pane2.add(label6, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		pane2.add(editeurNomImage, constraints);

		setConstraints(constraints, 0, 5, 3, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(pane2, constraints);

		JPanel pane3 = new JPanel();
		pane3.add(label7);
		pane3.add(editeurNomPointLieGenerateur);

		setConstraints(constraints, 0, 6, 3, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(pane3, constraints);

		setConstraints(constraints, 0, 7, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(checkPointLieGenerateurCache, constraints);

		setConstraints(constraints, 1, 7, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.EAST);
		panePrincipal.add(checkPointGenerateurCache, constraints);

		setConstraints(constraints, 0, 8, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(checkGestionAutoDiscontinuites, constraints);

		JPanel pane4 = new JPanel();
		pane4.add(label8);
		pane4.add(editeurNombrePoints);

		setConstraints(constraints, 1, 8, 2, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
		panePrincipal.add(pane4, constraints);

		paneRight.add(btnOK);
		paneRight.add(Box.createRigidArea(new Dimension(0, 5)));
		paneRight.add(btnCancel);

		setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
		panePrincipal.add(paneRight, constraints);

		setContentPane(panePrincipal);
		listeFonctionsSelectionChanged();
		pack();
	}

	private void btnOKActionPerformed() {
		MtgInternalFrame cadre = frame.cadreActif();
		CListeObjets listePr = cadre.listePr;
		// On vrifie la validit des noms choisis pour les calculs
		String chNomAbscisse = editeurNomAbscisse.getText();
    if (!listePr.validationNomCalcul(frame, chNomAbscisse)) {
      editeurNomAbscisse.requestFocus();
      return;
    }

		String chNomOrdonnee = editeurNomImage.getText();
    if (!listePr.validationNomCalcul(frame, chNomOrdonnee)) {
      editeurNomImage.requestFocus();
      return;
    }

		String chNomPoint = editeurNomPointLieGenerateur.getText();
		if (chNomPoint.length() == 0) {
			dialogueAvertissement(getString("ch1"));
			editeurNomPointLieGenerateur.requestFocus();
			return;
		}
		else {
	    if (listePr.validationNomPoint(chNomPoint)) {
	      if (listePr.existeMemeNomNonIntermediaire(null ,chNomPoint)) {
	        dialogueAvertissementCentreEcran(getString("ch20"));
	        editeurNomPointLieGenerateur.requestFocus();
	        return;
	      }		
	    }
	    else {
        dialogueAvertissementCentreEcran(getString("ch20"));
        editeurNomPointLieGenerateur.requestFocus();
        return;
	    }
		}
		int nombrePoints;
		try {
			nombrePoints = Integer.parseInt(editeurNombrePoints.getText());
		} catch (NumberFormatException e) {
			dialogueAvertissement(getString("ch71"));
			editeurNombrePoints.requestFocus();
			return;
		}
		if ((nombrePoints < 4) || nombrePoints > 5000) {
			dialogueAvertissement(getString("ch71"));
			editeurNombrePoints.requestFocus();
			return;
		}
		if (editeura.isVisible()) {
			if (!editeura.verifieValidite()) {
				editeura.requestFocus();
				return;
			}
		}
		if (editeurb.isVisible()) {
			if (!editeurb.verifieValidite()) {
				editeurb.requestFocus();
				return;
			}
		}

		// Tout est correct. On cre la courbe
		// Cas d'une courbe dfinie sur R
		if (radioCourbeSurR.isSelected()) {
			cadre.ajouteCourbeSurR(listeReperes.repereChoisi(), listeFonctions.fonctionSelectionnee(),
					editeurNomPointLieGenerateur.getText(), editeurNomAbscisse.getText(), editeurNomImage.getText(),
					nombrePoints, checkPointLieGenerateurCache.isSelected(), checkPointGenerateurCache.isSelected(),
					checkGestionAutoDiscontinuites.isSelected());
		}
		else {
			if (radioCourbeSurab.isSelected()) {
				cadre.ajouteCourbeSurab(listeReperes.repereChoisi(), listeFonctions.fonctionSelectionnee(),
						editeurNomPointLieGenerateur.getText(), editeurNomAbscisse.getText(), editeurNomImage.getText(),
						nombrePoints, checkPointLieGenerateurCache.isSelected(), checkPointGenerateurCache.isSelected(),
						checkGestionAutoDiscontinuites.isSelected(), vala, valb);				
			}
			else {
				if (radioCourbeAPartirDea.isSelected()) {
					cadre.ajouteCourbeAPartirdea(listeReperes.repereChoisi(), listeFonctions.fonctionSelectionnee(),
							editeurNomPointLieGenerateur.getText(), editeurNomAbscisse.getText(), editeurNomImage.getText(),
							nombrePoints, checkPointLieGenerateurCache.isSelected(), checkPointGenerateurCache.isSelected(),
							checkGestionAutoDiscontinuites.isSelected(), vala);				
				}
				else {
					cadre.ajouteCourbeJusquea(listeReperes.repereChoisi(), listeFonctions.fonctionSelectionnee(),
							editeurNomPointLieGenerateur.getText(), editeurNomAbscisse.getText(), editeurNomImage.getText(),
							nombrePoints, checkPointLieGenerateurCache.isSelected(), checkPointGenerateurCache.isSelected(),
							checkGestionAutoDiscontinuites.isSelected(), vala);				
					
				}
				
			}
		}
		setRetourOK(true);
		dispose();
	}

	private void btnCancelActionPerformed() {
		dispose();
	}

	private void listeFonctionsSelectionChanged() {
		int i = listeFonctions.getSelectedIndex();
		if (i != -1) {
			infoFormule.setText(listeFonctions.fonctionSelectionnee().infoFormule());
			infoFormule.setCaretPosition(0);
			pack();
		}
	}

	private void radioCourbeSurRActionPerformed() {
		if (radioCourbeSurR.isSelected()) {
			labela.setVisible(false);
			editeura.setVisible(false);
			paneListeValeurEtFonctionsa.setVisible(false);
			labelb.setVisible(false);
			editeurb.setVisible(false);
			paneListeValeurEtFonctionsb.setVisible(false);
			pack();
		}
	}

	private void radioCourbeSurabActionPerformed() {
		if (radioCourbeSurab.isSelected()) {
			labela.setVisible(true);
			editeura.setVisible(true);
			paneListeValeurEtFonctionsa.setVisible(true);
			labelb.setVisible(true);
			editeurb.setVisible(true);
			paneListeValeurEtFonctionsb.setVisible(true);
			pack();
		}
	}

	private void radioCourbeAPartirDeaActionPerformed() {
		if (radioCourbeAPartirDea.isSelected()) {
			labela.setVisible(true);
			editeura.setVisible(true);
			paneListeValeurEtFonctionsa.setVisible(true);			
			labelb.setVisible(false);
			editeurb.setVisible(false);
			paneListeValeurEtFonctionsb.setVisible(false);			
		}
		pack();
	}

	private void radioCourbeJusqueaActionPerformed() {
		if (radioCourbeJusquea.isSelected()) {
			labela.setVisible(true);
			editeura.setVisible(true);
			paneListeValeurEtFonctionsa.setVisible(true);
			labelb.setVisible(false);
			editeurb.setVisible(false);
			paneListeValeurEtFonctionsb.setVisible(false);
		}
		pack();
	}
	
}
