/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CDeriveePartielle;
import objets.CFoncNVar;
import objets.CListeObjets;
import types.FonteNom;

@SuppressWarnings("serial")
public class DeriveePartielleDlg extends MtgDlg {
	private CDeriveePartielle deriveeAModifier;
	private boolean modification;
	private ListeFonctionsReellesNVariables listeFonctions;
	private JLabel label1, label2, label3, label4;
	private JTextField editeurNom , info;
  private JButton btnOK, btnCancel;
  private JList<String> listeVariables;
	// private Vector<String> tab; // Liste des variables partielles
	private DefaultListModel<String> listModel;
  
  public DeriveePartielleDlg(MtgFrame frameInit, CDeriveePartielle deriveeAModifierInit, boolean modificationInit) {
  	super(getString("deriveePartielleDlg1"), true, frameInit);
  	modification = modificationInit;
  	deriveeAModifier = deriveeAModifierInit;
    listModel = new DefaultListModel<String>();
    listeVariables = new JList<String>();
    listeVariables.setModel(listModel);
    initComponents();
  	listeFonctionsSelectionChanged();
    if (modification) {
    	listeVariables.setSelectedIndex(deriveeAModifier.indiceVariableDerivation);
    }
    else {
    	listeVariables.setSelectedIndex(0);
    }
    getRootPane().setDefaultButton(btnOK);      	
  }

  private void initComponents() {
    label1 = new JLabel(getString("deriveeDlg2"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new JLabel(getString("deriveeDlg3"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    label3 = new JLabel(getString("informationFonction"), JLabel.RIGHT);
    label3.setRequestFocusEnabled(false);  
    label4 = new JLabel(getString("deriveePartielleDlg4"), JLabel.RIGHT);
    label4.setRequestFocusEnabled(false);
    editeurNom = new javax.swing.JTextField(null,8);
    info = new JTextField();
    info.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1));    
    info.setEditable(false);
    info.setColumns(40);
    CListeObjets liste = frame.cadreActif().listePr;
    int indmax;
    if (modification) {
    	indmax = liste.indexOf(deriveeAModifier) - 1;
    }
    else {
    	indmax = liste.size() - 1;
    }
    listeFonctions = new ListeFonctionsReellesNVariables(false, liste, indmax, false);
    listeFonctions.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listeFonctionsSelectionChanged();
      }
    });
    if (modification)  {
    	editeurNom.setEditable(false);
    	editeurNom.setText(deriveeAModifier.nom());
    	listeFonctions.setSel(deriveeAModifier.fonctionAssociee);
    }
    JScrollPane scrollPaneListe = new JScrollPane(listeFonctions, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    
    // tab = new Vector<String>();
    listeVariables.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label1, constraints);
    
    setConstraints(constraints, 1, 0, 1, 1, 40, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(editeurNom, constraints);    
    
    setConstraints(constraints, 0, 1, 1, 1, 0, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label2, constraints);

    setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    panePrincipal.add(scrollPaneListe, constraints);

    setConstraints(constraints, 0, 2, 1, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label4, constraints);
    setConstraints(constraints, 1, 2, 1, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(listeVariables, constraints);

    setConstraints(constraints, 0, 3, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(label3, constraints);
    setConstraints(constraints, 0, 4, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(info, constraints);
    
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
    panePrincipal.add(paneRight, constraints);
    
    setContentPane(panePrincipal);
    pack(); 
  }
  
  private void listeFonctionsSelectionChanged() {
  	// Vector<String> tab = new Vector<String>();
  	listModel.removeAllElements();
  	CFoncNVar fonc = (CFoncNVar) listeFonctions.fonctionSelectionnee();
  	for (int i = 0; i < fonc.nbVar; i ++) {
  		listModel.add(i, " " + fonc.chaineVariable()[i]+ " ");
  		// tab.add(fonc.chaineVariable[i]);
  	}
  	listeVariables.setSelectedIndex(0);
  	int i = listeFonctions.getSelectedIndex();
  	if (i != -1) {
  		info.setText(fonc.infoFormule());
  	}
		pack();
  }

  private void btnOKActionPerformed() {                                      
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets listePr = cadre.listePr;
    String chNom = editeurNom.getText();
    if (!modification) {
	    // Modifi version 4.6
	    if (!listePr.validationNomCalcul(frame, chNom)) {
	      editeurNom.requestFocus();
	      return;
	    }
    }
    CFoncNVar fonc = (CFoncNVar) listeFonctions.fonctionSelectionnee();
    if (modification) {
    	deriveeAModifier.fonctionAssociee = fonc;
    	int oldindder = deriveeAModifier.indiceVariableDerivation;
    	int indder = listeVariables.getSelectedIndex();
    	if (indder != oldindder) {
    		deriveeAModifier.indiceVariableDerivation = indder;
    		deriveeAModifier.creeCalcul();
    	}
    }
    else {
    	CDeriveePartielle der = new CDeriveePartielle(listePr, null, false, chNom, fonc, listeVariables.getSelectedIndex());
	    frame.ajouteElement(der);
	    der.positionne(frame.cadreActif().infRand, frame.cadreActif().dimf);
	  	cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil158"));
    }
  	setRetourOK(true);
  	dispose();
  }

  public void btnCancelActionPerformed() {
  	dispose();
  }

}