/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Toolkit;
import java.util.ListIterator;
import java.util.Vector;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.ListSelectionModel;
import objets.CElementBase;
import objets.CListeObjets;
import types.NatureCalcul;

@SuppressWarnings("serial")
public class ListeObjetsNumeriquesSourcesPourConst extends JList<String> {
  private CListeObjets liste; // La liste contenant les valeurs  afficher
  public Vector<CElementBase> pointeurs;
  private DefaultListModel<String> listModel;


  // Constructeur remplissant la liste avec les objets de listeInit qui peuvent tre des objets sources
  // Dans la version java il s'agit des objets numriques et des repres (les repres n'tant pas autoriss dans la version C++
  public ListeObjetsNumeriquesSourcesPourConst(CListeObjets listeInit) {
    super();
    listModel = new DefaultListModel<String>();
    setModel(listModel);
    pointeurs = new Vector<CElementBase>();
    setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    liste = listeInit;
    ListIterator<CElementBase> li = liste.listIterator();
    while (li.hasNext())
    {
      CElementBase elb = li.next();
      if (!elb.estElementIntermediaire() && elb.estDeNatureCalcul(NatureCalcul.NObjetCalculPourSourcesPrototype)
      		|| (elb.getNatureCalcul() == NatureCalcul.NRepere)){
        listModel.addElement(elb.nom());
        pointeurs.add(elb);
      }
    }    
    setVisibleRowCount(10);
    setSelectedIndex(0);
  }
  
  // Constructeur remplissant la liste avec les objets de listeInit qui peuvent tre des objets sources
  //  l'exception de ceux contenus dans listeExclue
  public ListeObjetsNumeriquesSourcesPourConst(CListeObjets listeInit, CListeObjets listeExclue, int indiceMaxiInit) {
    super();
    listModel = new DefaultListModel<String>();
    setModel(listModel);
    pointeurs = new Vector<CElementBase>();
    setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    liste = listeInit;
    ListIterator<CElementBase> li = liste.listIterator();
    while (li.hasNext())
    {
      CElementBase elb = li.next();
      if (!elb.estElementIntermediaire() && (((elb.getNatureCalcul() & NatureCalcul.NObjetCalculPourSourcesPrototype) != 0)
      		|| (elb.getNatureCalcul() == NatureCalcul.NRepere))
      		&& ! listeExclue.contains(elb)) {
        listModel.addElement(elb.nom());
        pointeurs.add(elb);
      }
    }
    setVisibleRowCount(10);
    setSelectedIndex(0);
  }  
  
  public CElementBase elementChoisi() {
  	int ind = getSelectedIndex();
  	if (ind == -1) return null;
  	else return pointeurs.get(getSelectedIndex());
  }
  
  public void insereValeurDebut(CElementBase val) {
  	listModel.add(0,val.nom());
  	pointeurs.insertElementAt(val, 0);
  	setSelectedIndex(0);
  }
  
  public void insereValeurFin(CElementBase val) {
  	listModel.addElement(val.nom());
  	pointeurs.add(val);
  	setSelectedIndex(listModel.size() - 1);
  }
  
  /**
   * Insre la valeur val au point de slection de la liste.
   * @param val
   */
  public void insereValeur(CElementBase val) {
  	int ind = getSelectedIndex();
  	if (ind == -1) Toolkit.getDefaultToolkit().beep();
  	else {
	  	listModel.add(ind, val.nom());
	  	pointeurs.add(ind, val);
  	}
  }
  
  public void retireValeurSelectionnee() {
  	int ind = getSelectedIndex();
  	if (ind != -1) {
  		listModel.removeElementAt(ind);
  		pointeurs.removeElementAt(ind);
  	}
  	if (ind == listModel.size()) ind --;
  	if (ind != -1) setSelectedIndex(ind);
  }
  
  public void retireTout() {
  	listModel.removeAllElements();
  	pointeurs.removeAllElements();
  }
  
  public void decaleBasValeurSelectionnee() {
  	int ind =  getSelectedIndex();
  	if (ind >= 1) {
  		CElementBase val = pointeurs.elementAt(ind);
  		String ch = (String) listModel.elementAt(ind);
  		pointeurs.remove(ind);
  		listModel.removeElementAt(ind);
  		pointeurs.add(ind - 1, val);
  		listModel.add(ind - 1, ch);
  		setSelectedIndex(ind - 1);
  	}
  }
  
  public void decaleHautValeurSelectionnee() {
  	int ind = getSelectedIndex();
  	if ((ind != -1) && (ind < listModel.size() - 1)) {
  		CElementBase val = pointeurs.elementAt(ind);
  		String ch = (String) listModel.elementAt(ind);
  		pointeurs.remove(ind);
  		listModel.removeElementAt(ind);
  		pointeurs.add(ind + 1, val);
  		listModel.add(ind + 1, ch);
  		setSelectedIndex(ind + 1);
  	}
  }

  /**
   * metAJour la liste d'lments disponibles comme lments sources numriques pour une  construction
   * sachant que listeEltsExclus contient la liste d'lments choisis.
   * Version 4.2 : on retire aussi de la liste tous les lments qui dpendent d'un lment de listeEltsExclus
   * et tous ceux dont au moins un lment de listeEltsExclus dpend.
   */
  public void metAJourListe(ListeObjetsNumeriquesSourcesPourConst listeSourcesCalculChoisis,
  		CListeObjets listeSourcesGraphiquesChoisis) {
  	listModel.removeAllElements();
  	pointeurs.removeAllElements();
    ListIterator<CElementBase> li = liste.listIterator();
    while (li.hasNext())
    {
      CElementBase elb = li.next();
      if (!elb.estElementIntermediaire() && (((elb.getNatureCalcul() & NatureCalcul.NObjetCalculPourSourcesPrototype) != 0)
      		|| (elb.getNatureCalcul() == NatureCalcul.NRepere))
      		&& ! listeSourcesCalculChoisis.pointeurs.contains(elb)) {
      	// Nouveau version 4.2 : On retire aussi de la liste tous les lments qui dpendent d'un lment de listeEltsExclus
      	// et tous ceux dont au moins un lment de listeEltsExclus dpend
      	// ainsi que tous ceux qui dpendent ou dont dpend un des lments de listeSourcesGraphiquesChoisis
      	boolean valide = true;
      	for (int j = 0; (j < listeSourcesCalculChoisis.pointeurs.size()) && valide; j++) {
      		CElementBase el = listeSourcesCalculChoisis.pointeurs.get(j);
      		valide = !el.dependDe(elb) && !elb.dependDe(el);
      	}
      	for (int j = 0; (j < listeSourcesGraphiquesChoisis.size()) && valide; j++) {
      		CElementBase el = listeSourcesGraphiquesChoisis.get(j);
      		valide = !el.dependDe(elb) && !elb.dependDe(el);
      	}
      	if (valide) {
	        listModel.addElement(elb.nom());
	        pointeurs.add(elb);
      	}
      }
    }
    if (listModel.size() != 0) setSelectedIndex(0);
  }
  
  
}
