/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import mathgraph32.MtgFrame;
import objets.CListeObjets;
import objets.CMacroAffectationValeurVariable;
import types.FonteNom;
import types.NatureCarSpe;

@SuppressWarnings("serial")
public class MacroAffectationValeurVariableDlg extends MtgDlg {
	private boolean modification;
	private CMacroAffectationValeurVariable macro;
	private JLabel label1, label2, label3, label4;
	private TextFieldAvecCarSpeciaux	editeurIntitule;
	private JTextArea editeurCommentaire;
	private EditeurValeurDynamique editeurValeur;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions;
	private ListeVariables listeVariables;
	private PaneTaillePolice paneTaillePolice;
	private PaneModeAffichagePourMacro paneModeAffichage;
	private JButton btnOK, btnCancel;
	private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;

  public MacroAffectationValeurVariableDlg(MtgFrame frameInit, CMacroAffectationValeurVariable macroInit, boolean modificationInit) {
    super (getString("macroAffectationValeurVariableDlg1"), true, frameInit);
    macro = macroInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);
  }
  
  private void initComponents() {
		label1 = new JLabel(getString("macroApparitionDlg2"));
		label2 = new JLabel(getString("macroApparitionDlg3"));
		label3 = new JLabel(getString("macroAffectationValeurVariableDlg2"));
		label4 = new JLabel(getString("macroAffectationValeurVariableDlg3"));
		editeurIntitule = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
		editeurCommentaire = new JTextArea();
		editeurCommentaire.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1));    
		editeurCommentaire.setColumns(20);
		editeurCommentaire.setLineWrap(false);
		editeurCommentaire.setRows(4);
    JScrollPane scrollPaneCommentaire = new JScrollPane(editeurCommentaire,
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneCommentaire.setPreferredSize(new Dimension(300, 90));
    // Inutile ici d'utiliser un objet interdiaire CValeurAssocieeAVariable car il y a un seul champ  remplir
    // Il ne sera affect que si tout est correct
    editeurValeur = new EditeurValeurDynamique(frame, macro.valeurAssociee, macro);
    if (modification) editeurValeur.setText(macro.valeurAssociee.chaineInfo());
    btnOK = new JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
	  paneTaillePolice = new PaneTaillePolice(macro.indiceTaillePolice);
	  paneModeAffichage = new PaneModeAffichagePourMacro(frame, macro);
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
  	// Spcial version applet
  	// CListeObjets liste = frame.cadreActif().listePr;
  	CListeObjets liste = frame.listePourConstruction();
    listeVariables = new ListeVariables(liste, liste.size()-1, false);
    listeVariables.setVisibleRowCount(5);
    JScrollPane scrollPaneListeVariables = new JScrollPane(listeVariables, 
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

  	int indmax;
  	if (modification) indmax = frame.cadreActif().listePr.indexOf(macro) - 1;
  	else indmax = frame.cadreActif().listePr.size() - 1;
    paneListeValeurEtFonctions = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeur, true, indmax, false);
    if (modification)  {
    	editeurIntitule.setText(macro.intitule);
    	editeurCommentaire.setText(macro.commentaireMacro);
    	editeurCommentaire.setCaretPosition(0);
    	editeurValeur.setText(macro.valeurAssociee.chaineInfo());
    	listeVariables.setSel(macro.variableAssociee);
    }
    
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    JPanel paneMilieu = new JPanel();
    JPanel paneBas = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 0;
		constraints.insets.top = 2;
		constraints.insets.left = 5;
		constraints.insets.right = 5;
		
		// paneMilieu est le JPanel qui contient l'diteur de valeur avec dessous le bouton
		// de liste des valeurs et  droite un JLabel avec au-dessous la liste des variables
		paneMilieu.setLayout(new GridBagLayout());
		setConstraints(constraints, 0, 0, 1, 1, 50, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneMilieu.add(label3, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneMilieu.add(label4, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneMilieu.add(editeurValeur, constraints);
		setConstraints(constraints, 0, 2, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST);
		paneMilieu.add(paneListeValeurEtFonctions, constraints);
		setConstraints(constraints, 1, 1, 1, 2, 0, 20, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneMilieu.add(scrollPaneListeVariables, constraints);
		
		setConstraints(constraints, 0, 0, 1, 1, 20, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 60, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurIntitule, constraints);
		setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label2, constraints);
		setConstraints(constraints, 0, 2, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneCommentaire, constraints);
		setConstraints(constraints, 0, 3, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneMilieu, constraints);
		paneBas.add(paneTaillePolice);
		paneBas.add(paneModeAffichage);
		setConstraints(constraints, 0, 4, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneBas, constraints);
 		setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneRight, constraints);
		
		setContentPane(panePrincipal);
		// On change l'ordre par dfaut des composants
		Vector<Component> order = new Vector<Component>(8);
		order.add(editeurIntitule);
		order.add(editeurCommentaire);
		order.add(editeurValeur);
		order.add(listeVariables);
		order.add(paneTaillePolice.listeTailles);
		order.add(paneModeAffichage.effacementFond);
		order.add(btnOK);
		order.add(btnCancel);
		mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
		setFocusTraversalPolicy(mtgFocusTraversalPolicy);
		pack();			
  }
  
  private void btnOKActionPerformed() {
  	String chintitule = editeurIntitule.getText();
  	if (chintitule.length() == 0) {
  		dialogueAvertissement(getString("ch115"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	CListeObjets liste = frame.cadreActif().listePr;
  	if (!liste.validationIntituleMacro(chintitule, macro)) {
  		dialogueAvertissement(getString("ch116"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	if (!editeurValeur.verifieValidite()) {
  		editeurValeur.requestFocus();
  		return;
  	}
  	macro.intitule = chintitule;
  	macro.commentaireMacro = editeurCommentaire.getText();
  	// L'affectation de la valeur dynamique a dj t faite par editeurValeur
  	macro.variableAssociee = listeVariables.variableSelectionnee();
  	macro.indiceTaillePolice = paneTaillePolice.indiceTaille();
  	macro.effacementFond = paneModeAffichage.effacementFond();
  	macro.couleurFondAffichage = paneModeAffichage.couleurFond();
		// Ajouts version 4.5
		macro.alignementHorizontal = paneModeAffichage.alignementHorizontal();
		macro.alignementVertical = paneModeAffichage.alignementVertical();
		//
		frame.pref_IndiceTaillePoliceAffichages = macro.indiceTaillePolice;
		frame.cadreActif().couleurFondAffichage = macro.couleurFondAffichage;
  	setRetourOK(true);
  	dispose();
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }    
}
