/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import objets.CAffichageLieAPoint;
import types.StyleEncadrement;


import mathgraph32.MtgColorButton;
import mathgraph32.MtgFrame;

/**
 * Classe associant un JPane  un CAffichageLieAPoint
 * Si affichafeAssocie est null, il est initialis sans encadrement et couleur de fond blanc
 * @author Yves
 *
 */
//  Modifi version 4.5.2
@SuppressWarnings("serial")
public class PaneModeAffichage extends JPanel {
	private boolean bmodificationAlignement; // Ajout version 4.5.2
	private MtgFrame frame;
	private CAffichageLieAPoint affichageAssocie;
	public JCheckBox checkEffacementFond;
	public JRadioButton radioSansEncadrement;
	public JRadioButton radioEncadrementSimple;
	public JRadioButton radioEncadrement3D;
	private JLabel label1, label2;
	// Lignes suivante passes public version 4.7.1
	public JRadioButton rbAligHorGauche,rbAligHorCentre,rbAligHorDroit;
	public JRadioButton rbAligVertHaut, rbAligVertCentre, rbAligVertBas;
	// Buf=g de Java : si on cre un JRadioButton avec comme paramtre une icne, plus de case  cocher !
	// Je mets donc des JLabel devant avec une icne
	private JLabel labelAligHorGauche, labelAligHorCentre, labelAligHorDroit;
	private JLabel labelAligVertHaut, labelAligVertCentre, labelAligVertBas;
	
	public MtgColorButton colorButton;
	// public PaneModeAffichage(MtgFrame frameInit, CAffichageLieAPoint affichageAssocieInit) {
	public PaneModeAffichage(MtgFrame frameInit, CAffichageLieAPoint affichageAssocieInit,
			boolean bmodificationAlignementInit) {
		super();
		frame = frameInit;
		affichageAssocie = affichageAssocieInit;
		bmodificationAlignement = bmodificationAlignementInit;
		checkEffacementFond = new JCheckBox(MtgFrame.getString("paneAffichage1"));
		checkEffacementFond.addActionListener(new ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				effacementFondActionPerformed();
			}
		});
		checkEffacementFond.setSelected(affichageAssocie == null ? true : affichageAssocie.effacementFond);
		radioSansEncadrement = new JRadioButton(MtgFrame.getString("paneAffichage2"),true);
		radioEncadrementSimple = new JRadioButton(MtgFrame.getString("paneAffichage3"));
		radioEncadrement3D = new JRadioButton(MtgFrame.getString("paneAffichage4"));
		ButtonGroup btnGroup = new ButtonGroup();
		btnGroup.add(radioSansEncadrement);
		btnGroup.add(radioEncadrementSimple);
		btnGroup.add(radioEncadrement3D);
		if (bmodificationAlignement) {
	    URL adr = MtgFrame.class.getResource("images/AlignementHorGauche.gif");    
	    labelAligHorGauche = new JLabel(new ImageIcon(adr));
			adr = MtgFrame.class.getResource("images/AlignementHorCentre.gif");
	    labelAligHorCentre= new JLabel(new ImageIcon(adr));
			adr = MtgFrame.class.getResource("images/AlignementHorDroit.gif");
	    labelAligHorDroit = new JLabel(new ImageIcon(adr));
			adr = MtgFrame.class.getResource("images/AlignementVertHaut.gif");
			labelAligVertHaut= new JLabel(new ImageIcon(adr));
			adr = MtgFrame.class.getResource("images/AlignementVertCentre.gif");
			labelAligVertCentre= new JLabel(new ImageIcon(adr));
			adr = MtgFrame.class.getResource("images/AlignementVertBas.gif");
			labelAligVertBas= new JLabel(new ImageIcon(adr));
			rbAligHorGauche = new JRadioButton();
			rbAligHorCentre = new JRadioButton();
			rbAligHorDroit = new JRadioButton();
			rbAligVertHaut = new JRadioButton();
			rbAligVertCentre = new JRadioButton();
			rbAligVertBas = new JRadioButton();
			ButtonGroup btngroup1 = new ButtonGroup();
			ButtonGroup btngroup2 = new ButtonGroup();
			btngroup1.add(rbAligHorGauche);
			btngroup1.add(rbAligHorCentre);
			btngroup1.add(rbAligHorDroit);
			btngroup2.add(rbAligVertHaut);
			btngroup2.add(rbAligVertCentre);
			btngroup2.add(rbAligVertBas);
		}
		if (affichageAssocie == null) {
			radioSansEncadrement.setSelected(true);
			rbAligHorGauche.setSelected(true);
			rbAligVertHaut.setSelected(true);			
		}
		else {
			switch (affichageAssocie.encadrement) {
				case StyleEncadrement.Sans :
				default :
					radioSansEncadrement.setSelected(true);
					break;
				case StyleEncadrement.Simple :
					radioEncadrementSimple.setSelected(true);
					break;
				case StyleEncadrement.Effet3D :
					radioEncadrement3D.setSelected(true);
			}
			if (bmodificationAlignement) {
				switch (affichageAssocieInit.alignementHorizontal) {
					case CAffichageLieAPoint.alignementHorizontalGauche :
						rbAligHorGauche.setSelected(true);
						break;
					case CAffichageLieAPoint.alignementHorizontalCentre :
						rbAligHorCentre.setSelected(true);
						break;
					case CAffichageLieAPoint.alignementHorizontalDroit :
						rbAligHorDroit.setSelected(true);
						break;
					}
					switch (affichageAssocieInit.alignementVertical) {
					case CAffichageLieAPoint.alignementVerticalHaut :
						rbAligVertHaut.setSelected(true);
						break;
					case CAffichageLieAPoint.alignementVerticalCentre :
						rbAligVertCentre.setSelected(true);
						break;
					case CAffichageLieAPoint.alignementVerticalBas :
						rbAligVertBas.setSelected(true);
						break;
				}
			}
		}
		label1 = new JLabel(MtgFrame.getString("paneAffichage5"));
		label2 = new JLabel(MtgFrame.getString("paneAffichage6"));
		colorButton = new MtgColorButton(frame);
		colorButton.couleur = affichageAssocie == null ? frame.cadreActif().couleurFond : affichageAssocie.couleurFondAffichage;
		if (!checkEffacementFond.isSelected()) {
			label1.setVisible(false);
			label2.setVisible(false);
			colorButton.setVisible(false);
		}
		JPanel paneLeft = new JPanel();
		JPanel paneRight = new JPanel();
		paneLeft.setLayout(new BoxLayout(paneLeft,BoxLayout.Y_AXIS));
		paneRight.setLayout(new BoxLayout(paneRight,BoxLayout.Y_AXIS));
		paneLeft.add(checkEffacementFond);
		paneLeft.add(radioSansEncadrement);
		paneLeft.add(radioEncadrementSimple);
		paneLeft.add(radioEncadrement3D);
		if (affichageAssocie == null) {
			label1.setVisible(true);
			label2.setVisible(true);
			colorButton.setVisible(true);
		}
		else {
			if (!affichageAssocie.effacementFond) {
				label1.setVisible(false);
				label2.setVisible(false);
				colorButton.setVisible(false);
			}
		}
		paneRight.add(label1);
		paneRight.add(label2);
		paneRight.add(colorButton);
		JPanel paneMode = new JPanel();
		paneMode.setLayout(new FlowLayout());
		paneMode.add(paneLeft);
		paneMode.add(paneRight);
		paneMode.setBorder(BorderFactory.createTitledBorder(MtgFrame.getString("paneAffichage7")));
		add(paneMode);
		
		if (bmodificationAlignement) {
			GridBagConstraints constraints = new GridBagConstraints();
			constraints.insets.bottom = 5;
			constraints.insets.top = 5;
			constraints.insets.left = 5;
			constraints.insets.right = 5;
	
			JPanel paneAlignement = new JPanel();
			paneAlignement.setLayout(new GridBagLayout());
			MtgDlg.setConstraints(constraints, 0, 0, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
			paneAlignement.add(labelAligHorGauche, constraints);
			MtgDlg.setConstraints(constraints, 1, 0, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
			paneAlignement.add(labelAligHorCentre, constraints);
			MtgDlg.setConstraints(constraints, 2, 0, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
			paneAlignement.add(labelAligHorDroit, constraints);
			MtgDlg.setConstraints(constraints, 0, 1, 1, 1, 30, 10, GridBagConstraints.CENTER, GridBagConstraints.WEST);
			paneAlignement.add(rbAligHorGauche, constraints);
			MtgDlg.setConstraints(constraints, 1, 1, 1, 1, 30, 10, GridBagConstraints.CENTER, GridBagConstraints.WEST);
			paneAlignement.add(rbAligHorCentre, constraints);
			MtgDlg.setConstraints(constraints, 2, 1, 1, 1, 30, 10, GridBagConstraints.CENTER, GridBagConstraints.WEST);
			paneAlignement.add(rbAligHorDroit, constraints);
			MtgDlg.setConstraints(constraints, 0, 2, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
			paneAlignement.add(labelAligVertHaut, constraints);
			MtgDlg.setConstraints(constraints, 1, 2, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
			paneAlignement.add(labelAligVertCentre, constraints);
			MtgDlg.setConstraints(constraints, 2, 2, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
			paneAlignement.add(labelAligVertBas, constraints);
			MtgDlg.setConstraints(constraints, 0, 3, 1, 1, 30, 10, GridBagConstraints.CENTER, GridBagConstraints.WEST);
			paneAlignement.add(rbAligVertHaut, constraints);
			MtgDlg.setConstraints(constraints, 1, 3, 1, 1, 30, 10, GridBagConstraints.CENTER, GridBagConstraints.WEST);
			paneAlignement.add(rbAligVertCentre, constraints);
			MtgDlg.setConstraints(constraints, 2, 3, 1, 1, 30, 10, GridBagConstraints.CENTER, GridBagConstraints.WEST);
			paneAlignement.add(rbAligVertBas, constraints);
			paneAlignement.setBorder(BorderFactory.createTitledBorder(MtgFrame.getString("paneAffichage8")));
	
			add(paneAlignement);
		}

	}
	
	/**
	 *  Quand on change la case  cocher Effacement du fond il faut ragir en cachant ou montrant
	 *  les deux JLabel et le bouton de choix de couleur
	 */
	private void effacementFondActionPerformed() {
		if (checkEffacementFond.isSelected()) {
			label1.setVisible(true);
			label2.setVisible(true);
			colorButton.setVisible(true);
			// Il faut informer le dialogue contenant le paneau qu'il doit se redimensionner
			MtgDlg owner = (MtgDlg) getTopLevelAncestor();
			owner.pack();
		}
		else {
			label1.setVisible(false);
			label2.setVisible(false);
			colorButton.setVisible(false);
		}
	}
	
	public int styleEncadrement() {
		if (radioSansEncadrement.isSelected()) return StyleEncadrement.Sans;
		else if (radioEncadrementSimple.isSelected()) return StyleEncadrement.Simple;
		else return StyleEncadrement.Effet3D;
	}
	
	public boolean effacementFond() {
		return checkEffacementFond.isSelected();
	}
	
	public Color couleurFond() {
		return colorButton.couleur;
	}
	 
	// Rajout version 4.5
	/*
	 * Renvoie le style d'alignement horizontal choisi
	 */
	public int alignementHorizontal() {
		if (rbAligHorGauche.isSelected()) return CAffichageLieAPoint.alignementHorizontalGauche;
		else 	if (rbAligHorCentre.isSelected()) return CAffichageLieAPoint.alignementHorizontalCentre;
		else return CAffichageLieAPoint.alignementHorizontalDroit;	
	}
	
	// Rajout version 4.5
	/*
	 * Renvoie le style d'alignement horizontal choisi
	 */
	public int alignementVertical() {
		if (rbAligVertHaut.isSelected()) return CAffichageLieAPoint.alignementVerticalHaut;
		else 	if (rbAligVertCentre.isSelected()) return CAffichageLieAPoint.alignementVerticalCentre;
		else return CAffichageLieAPoint.alignementVerticalBas;	
	}
	
}
