/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class PanePreferencesLangue extends PaneOptionsPreferences {
	public JList<String> listeLangues;
	private JLabel label1;
	private JScrollPane scrollpane;
	private Vector<String> langues;
	private Vector<String> suffixes;

	public PanePreferencesLangue(MtgFrame frameInit) {
		super(frameInit);
		initComponents();
	}
	
	private void initComponents() {
		label1 = new JLabel(MtgFrame.getString("langue"));
		langues = new Vector<String>();
		suffixes = new Vector<String>();
		langues.add("Franais");
		suffixes.add("fra");
		langues.add("English");
		suffixes.add("eng");
		langues.add("Espaol");
		suffixes.add("esp");
		listeLangues = new JList<String>(langues);
		listeLangues.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		listeLangues.setLayoutOrientation(JList.VERTICAL);
		listeLangues.setVisibleRowCount(5);
		listeLangues.setSelectedIndex(suffixes.indexOf(frame.pref_language));
		listeLangues.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
    scrollpane = new JScrollPane(listeLangues, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    setLayout(new GridBagLayout());
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    MtgDlg.setConstraints(constraints, 0, 0, 1, 1, 50, 100, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST);
    add(label1, constraints);
    MtgDlg.setConstraints(constraints, 1, 0, 1, 1, 50, 100, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    add(scrollpane, constraints);
	}
	
	
	@Override
	public void sauveDonnees() {
		int ind = listeLangues.getSelectedIndex();
		// Si il y a un changement de langue
		String suf = suffixes.get(ind);
		if (!suf.equals(frame.pref_language)) {
			frame.pref_language = suf;
			frame.dialogueAvertissement(MtgFrame.getString("redemarrageNecessaire"));
		}

	}

	@Override
	public String titreOnglet() {
		return MtgFrame.getString("ongletLangues");
	}

}
