/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CElementBase;
import objets.CListeObjets;
import types.FonteNom;
import types.NatureObjet;

@SuppressWarnings("serial")
public class ReclassementDebutValeurDlg extends MtgDlg {
	
	private JLabel label1;
	private ListeObjetsNumeriquesPourReclassement listeValeurs;
	private JButton btnReclasserDebut, btnFermer;
	private JTextArea info;

  public ReclassementDebutValeurDlg(MtgFrame frameInit) {
    super(getString("reclassementDebutValeurDlg1"), true, frameInit);
    initComponents();
    getRootPane().setDefaultButton(btnFermer);
    listValSelectionChanged();
  }
  
  private void initComponents() {
  	CListeObjets liste = frame.cadreActif().listePr;
  	label1 = new JLabel(getString("listeValeurs"));
  	btnReclasserDebut = new javax.swing.JButton(getString("btnReclasserDebut"));
  	btnReclasserDebut.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnReclasserFinActionPerformed();
      }
    });
  	btnFermer = new javax.swing.JButton(getString("btnFermer"));
  	btnFermer.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnFermerActionPerformed();
      }
    });
  	btnFermer.setMaximumSize(btnReclasserDebut.getPreferredSize());
  	
  	listeValeurs = new ListeObjetsNumeriquesPourReclassement(liste, liste.size() - 1, false); // Pas d'objets intermdiaires dans la liste
  	JScrollPane scrollPane = new JScrollPane(listeValeurs, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    listeValeurs.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listValSelectionChanged();
      }
    });
  	
    info = new JTextArea();
    info.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1));    
    JScrollPane scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    info.setEditable(false);
    info.setColumns(20);
    info.setEditable(false);
    info.setLineWrap(false);
    info.setRows(4);
    scrollPaneInfo.setPreferredSize(new Dimension(450, 90));    
  	
    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());
 		JPanel paneRight = new JPanel();
 		paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
 		paneRight.add(btnReclasserDebut);
		paneRight.add(Box.createRigidArea(new Dimension(0, 5))); 		
 		paneRight.add(btnFermer);
    

    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 70, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1, constraints);
    setConstraints(constraints, 0, 1, 1, 1, 0, 60, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    pane.add(scrollPane, constraints);
    setConstraints(constraints, 0, 2, 2, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    pane.add(scrollPaneInfo, constraints);
    setConstraints(constraints, 1, 0, 1, 2, 30, 0, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    pane.add(paneRight, constraints);
    
    setContentPane(pane);
    pack();
  	
  }
  
  private void btnReclasserFinActionPerformed() {
  	MtgInternalFrame cadre = frame.cadreActif();
  	CListeObjets liste = cadre.listePr;
  	CElementBase val =listeValeurs.valeurSelectionnee();
  	if (liste.reclasseVersDebutAvecDependants(val)) {
  		dialogueAvertissement(getString("reclassementDebutObjetGraphique2"));
			cadre.listePr.metAJourParNatureObjet(NatureObjet.NCommentaireOuLatex); // Ajout version 4.6 pour que les dpendances de commentaires et laTeX dynamiques soient remis  jour
			cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil102"));
			cadre.repaintFigureAvecCalcul();
			listeValeurs.metAJourListe();
	    cadre.videListesPourConstructions();  // On remet  jour une construction ventuellement commence
  	}
  	else dialogueAvertissement(getString("reclassementDebutObjetGraphique3"));  		
  }
  
  private void btnFermerActionPerformed() {
  	dispose();
  }
  
  public void listValSelectionChanged() {
  	int i = listeValeurs.getSelectedIndex();
  	if (i != -1) {
  		info.setText(listeValeurs.pointeurs.get(i).infoPourHistorique());
  		info.setCaretPosition(0);
  		pack();
  	}
  }
}
