/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import mathgraph32.MtgFrame;

import objets.CListeObjets;
import objets.CValeurAssocieeAVariable;
import types.FonteNom;

@SuppressWarnings("serial")
public class TangenteDlg extends MtgDlg {
	private JLabel label1, label2, label3, label4, label5;
	private JButton btnOK, btnCancel;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions;	
	public ListeFonctionsReelles listeFonctions;
	public ListeReperes listeReperes;
	private JTextArea info;
	private EditeurValeurDynamique editeurValeurAbs;
	private JScrollPane scrollPaneListeFonctions, scrollPaneListeReperes;
	public JTextField editeurNomCoeff;
	public CValeurAssocieeAVariable valAbs;
	
  public TangenteDlg(MtgFrame frameInit) {
    super(getString("tangenteDlg1"), true, frameInit);
    valAbs = new CValeurAssocieeAVariable();    
    initComponents();
    getRootPane().setDefaultButton(btnOK);
  }
  
  private void initComponents() {
		// Spcial version applet
		// CListeObjets liste = frame.cadreActif().listePr;
		// CListeObjets liste = frame.cadreActif().listePr;
		CListeObjets liste = frame.listePourConstruction();
		int indmax = liste.size() - 1;
  	label1 = new JLabel(getString("tangenteDlg2"));
  	label2 = new JLabel(getString("informationFonction"));
  	label3 = new JLabel(getString("pointSurCourbeDlg2"));
    label4 = new JLabel(getString("tangenteDlg3"));
    label5 = new JLabel(getString("tangenteDlg4"));
  	
		btnOK = new javax.swing.JButton(getString("btnOK"));
		btnOK.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnOKActionPerformed();
			}
		});
		btnCancel = new javax.swing.JButton(getString("btnCancel"));
		btnCancel.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				btnCancelActionPerformed();
			}
		});
		btnOK.setMaximumSize(btnCancel.getPreferredSize());
		listeFonctions = new ListeFonctionsReelles(liste, indmax, false);
		// Par dfaut 10 lignes c'est trop ici
		listeFonctions.setVisibleRowCount(6);
		listeFonctions.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent evt) {
				listeFonctionsSelectionChanged();
			}
		});
		scrollPaneListeFonctions = new JScrollPane(listeFonctions, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		listeReperes = new ListeReperes(liste, indmax);
		scrollPaneListeReperes = new JScrollPane(listeReperes, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		
    info = new JTextArea();
    info.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1));    
    info.setEditable(false);
    info.setColumns(30);
    info.setRows(1);
    JScrollPane scrollPaneInfo = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_NEVER, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    editeurNomCoeff = new JTextField(8);
    editeurNomCoeff.setText(liste.genereNomPourCalcul("m", true));
		
		editeurValeurAbs = new EditeurValeurDynamique(frame, valAbs, null);
    paneListeValeurEtFonctions = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurAbs, true, indmax, false);
		JPanel panePrincipal = (JPanel)getContentPane();
		panePrincipal.setLayout(new GridBagLayout());
		JPanel paneRight = new JPanel();
		paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
		
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 5;
		constraints.insets.top = 5;
		constraints.insets.left = 5;
		constraints.insets.right = 5;

		setConstraints(constraints, 0, 0, 1, 1, 20, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneListeFonctions, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label2, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneInfo, constraints);
		setConstraints(constraints, 0, 2, 1, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label3, constraints);
		setConstraints(constraints, 1, 2, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneListeReperes, constraints);
		setConstraints(constraints, 0, 3, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label4, constraints);
		setConstraints(constraints, 1, 3, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurValeurAbs, constraints);
		setConstraints(constraints, 1, 4, 1, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneListeValeurEtFonctions, constraints);
		setConstraints(constraints, 0, 5, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label5, constraints);
		setConstraints(constraints, 1, 5, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurNomCoeff, constraints);
		
		paneRight.add(btnOK);
		paneRight.add(Box.createRigidArea(new Dimension(0, 5)));
		paneRight.add(btnCancel);

		setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
		panePrincipal.add(paneRight, constraints);
		listeFonctionsSelectionChanged();
		pack();		  	
  }
	
  private void btnOKActionPerformed() {
  	// On vrifie si le nom choisi pour l'image de l'abscisse est valide
  	// Spcial version applet
    // if (!frame.cadreActif().listePr.validationNomCalcul(frame, editeurNomCoeff.getText())) {
    if (!frame.listePourConstruction().validationNomCalcul(frame, editeurNomCoeff.getText())) {
      editeurNomCoeff.requestFocus();
      return;
    }
    if (!editeurValeurAbs.verifieValidite()) {
    	editeurValeurAbs.requestFocus();
    	return;
    }
  	setRetourOK(true);
  	dispose();
  }  
  
  private void btnCancelActionPerformed() {
  	dispose();
  }  
  
  private void listeFonctionsSelectionChanged() {
		int i = listeFonctions.getSelectedIndex();
		if (i != -1) {
			info.setText(listeFonctions.fonctionSelectionnee().infoFormule());
			info.setCaretPosition(0);
			pack();
		}
	}  
}
