/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CListeObjets;
import objets.CTestEqNatOp;
import types.NatureCalcul;

@SuppressWarnings("serial")
public class TestEqNatOpDlg extends MtgDlg {
	private CTestEqNatOp testAssocie;
	private boolean modification;
	private JLabel label1, label2;
	private JTextField editeurNom;
	private ListeCalculsOuFonctionsParFormule listeGauche, listeDroite;
	private JScrollPane scrollPaneListeGauche, scrollPaneListeDroite;
	private JButton btnOK, btnCancel;

	public TestEqNatOpDlg(MtgFrame frameInit, CTestEqNatOp testAssocieInit, boolean modificationInit) {
		super(getString("ch196"), true, frameInit);
		testAssocie = testAssocieInit;
		modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);
    pack();
	}

	private void initComponents() {
  	MtgInternalFrame cadre = frame.cadreActif();
  	CListeObjets liste = cadre.listePr;
		label1 = new JLabel(getString("creerCalculDlg1"));
		label2 = new JLabel(getString("testEquivalenceDlg2"));
    editeurNom = new javax.swing.JTextField(null,8);
    int indmax;
    if (modification) indmax = liste.indexOf(testAssocie) - 1;
    else indmax = liste.size() - 1;
		listeGauche = new ListeCalculsOuFonctionsParFormule(liste, indmax, false, NatureCalcul.NCalculOuFonctionParFormule);
		listeDroite = new ListeCalculsOuFonctionsParFormule(liste, indmax, false, NatureCalcul.NCalculOuFonctionParFormule);
    scrollPaneListeGauche = new JScrollPane(listeGauche, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
    		JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );	
    scrollPaneListeDroite = new JScrollPane(listeDroite, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
    		JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );	
	  btnOK = new javax.swing.JButton(getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new javax.swing.JButton(getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setMaximumSize(btnCancel.getPreferredSize());
		if (modification) {
      editeurNom.setText(testAssocie.nom());
      editeurNom.setEditable(false);
			listeGauche.select(testAssocie.calcul1);
			listeDroite.select(testAssocie.calcul2);
			listeGauche.ensureIndexIsVisible(listeGauche.getSelectedIndex());
			listeDroite.ensureIndexIsVisible(listeGauche.getSelectedIndex());
		}
		else {
	    listeDroite.metAJour(listeGauche.calculSelectionne().getNatureCalcul(), listeGauche.calculSelectionne());
		}
    JPanel panePrincipal = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    JPanel paneRight = new JPanel();
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 2;
		constraints.insets.top = 2;
		constraints.insets.left = 5;
		constraints.insets.right = 5;
		
		JPanel paneCenter = new JPanel();
		paneCenter.setLayout(new GridBagLayout());
		setConstraints(constraints, 0, 0, 1, 1, 50, 90, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneCenter.add(scrollPaneListeGauche, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneCenter.add(scrollPaneListeDroite, constraints);		
		
		setConstraints(constraints, 0, 0, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(editeurNom, constraints);
		setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label2, constraints);
		setConstraints(constraints, 0, 2, 2, 1, 0, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneCenter, constraints);	
 		setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneRight, constraints);
		setContentPane(panePrincipal);	  
	}
	
	private void btnOKActionPerformed() {
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets listePr = cadre.listePr;
    String chNom = editeurNom.getText();
    if (!modification) {
	    // Modifi version 4.6
	    if (!listePr.validationNomCalcul(frame, chNom)) {
	      editeurNom.requestFocus();
	      return;
	    }
	    testAssocie.nomCalcul = chNom;
    }
		testAssocie.calcul1 = listeGauche.calculSelectionne();
		testAssocie.calcul2 = listeDroite.calculSelectionne();
		setRetourOK(true);
		dispose();
	}
	
	private void btnCancelActionPerformed() {
		dispose();
	}

}
