/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


package objets;

import types.Complexe;
import types.ErreurCalculException;

public class CAppelFonctionComplexeNVar extends CAppelFonctionNVar {
  // Attribut utilis pour optimisation

  public CAppelFonctionComplexeNVar() {
  }
  
  /** Constructeur */
  public CAppelFonctionComplexeNVar(CListeObjets liste, int nbVarInit, CFoncComplexeNVar fonctionInit, CCalculBase[] operandeInit)
  {
    nbVar = nbVarInit;
   	listeProprietaire = liste;
    fonctionAssociee = fonctionInit;
    operande = operandeInit;
  }  
  
  public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
  	CCalculBase[] operandeClone = new CCalculBase[nbVar];
  	for (int i = 0; i < nbVar;i++)
	    operandeClone[i] = operande[i].getClone(listeSource, listeCible); 
    //  revoir car la fonction a dj t cre
    int ind1 = listeSource.indexOf(fonctionAssociee);    
    CCalculBase ptcalc = new CAppelFonctionComplexeNVar(listeCible, nbVar, (CFoncComplexeNVar)(listeCible.get(ind1, CFoncComplexeNVar.class)),
      operandeClone);
    return ptcalc;
  }
  
  @Override
  public void resultatComplexe(boolean infoRandom, Complexe zRes) throws ErreurCalculException {
  	Complexe[] z1 = new Complexe[nbVar];
  	for (int i =0; i < nbVar; i++) z1[i] = new Complexe();
  	for (int i = 0; i < nbVar; i++) operande[i].resultatComplexe(infoRandom, z1[i]);
    ((CFoncComplexeNVar) fonctionAssociee).rendValeurFonctionComplexe(infoRandom, z1, zRes);
  }
  
  @Override
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre, Complexe zRes) 
    throws ErreurCalculException {
  	Complexe[] z1 = new Complexe[nbVar];
  	for (int i =0; i < nbVar; i++) z1[i] = new Complexe();
  	for (int i = 0; i < nbVar; i++)  operande[i].resultatFonctionComplexe(infoRandom, valeurParametre, z1[i]);
  	((CFoncComplexeNVar) fonctionAssociee).rendValeurFonctionComplexe(infoRandom, z1, zRes);
  }
  
  @Override
  void  resultatFonctionComplexe(boolean infoRandom, Complexe[] valeurParametre, Complexe zRes)
    throws ErreurCalculException {
  	Complexe[] z1 = new Complexe[nbVar];
  	for (int i =0; i < nbVar; i++) z1[i] = new Complexe();
  	for (int i = 0; i < nbVar; i++) operande[i].resultatFonctionComplexe(infoRandom, valeurParametre, z1[i]);
  	((CFoncComplexeNVar) fonctionAssociee).rendValeurFonctionComplexe(infoRandom, z1, zRes);
  }
  
  public CCalculBase getCopie() {
  	CCalculBase[] operandeCopie = new CCalculBase[nbVar];
  	for (int i = 0; i < nbVar; i++) operandeCopie[i] = operande[i].getCopie();
    return new CAppelFonctionComplexeNVar(listeProprietaire, nbVar, (CFoncComplexeNVar) fonctionAssociee, operandeCopie);
  }
  
  // Ajout version 4.6
  @Override
  public CCalculBase calculAvecValeursRemplacees() {
  	int nbvar = fonctionAssociee.nombreVariables();
  	CCalculBase[] tabcal = new CCalculBase[fonctionAssociee.nombreVariables()];
  	for (int i = 0; i < nbvar; i++) {
  		tabcal[i] = operande[i].calculAvecValeursRemplacees();
  	}
  	return new CAppelFonctionComplexeNVar(listeProprietaire, nbvar, (CFoncComplexeNVar) fonctionAssociee, tabcal);
  }

}
