/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.geom.*;
import java.awt.Color;


import mathgraph32.MtgFrame;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

public class CAutrePointIntersectionCercles extends CPointAncetre {
  private CCercle C1;
  private CCercle C2;
  private CPointAncetre pointExclu;
  // Pour le poitionnement
  private Point2D.Double pointInt1 = new Point2D.Double();
  private Point2D.Double pointInt2 = new Point2D.Double();
  
  /** Creates new CAutrePointIntersectionCercles */
  public CAutrePointIntersectionCercles() {
  }
  
  // Constructeur pour clonage
  public CAutrePointIntersectionCercles(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit,
    String nomInit, int indiceTailleNomInit, int motifInit, CCercle C1Init, CCercle C2Init,
    CPointAncetre pointExcluInit, boolean marquePourTraceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
      dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
    C1 = C1Init;
    C2 = C2Init;
    pointExclu = pointExcluInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(C1);
    int ind2 = listeSource.indexOf(C2);
    int ind3 = listeSource.indexOf(pointExclu);
    int ind4 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CAutrePointIntersectionCercles(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom, indiceTailleNom, motif,
      (CCercle)(listeCible.get(ind1, CCercle.class)), (CCercle)(listeCible.get(ind2, CCercle.class)),
      (CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)), marquePourTrace);
    return ptelb;
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || C1.dependDe(p) || C2.dependDe(p) || pointExclu.dependDe(p));
  }
 
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || C1.dependDePourReclassement(p)
    	|| C2.dependDePourReclassement(p) || pointExclu.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || C1.dependDePourBoucle(p) || C2.dependDePourBoucle(p)
    || pointExclu.dependDePourBoucle(p));
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    byte position;
    boolean point1Defini = false;
    boolean point2Defini = false;
    
    existe = C1.existe && C2.existe;
    if (!existe) return;
    position = Kernel.intersectionCercleCercle(C1.centre_x,C1.centre_y,C1.rayon,
      C2.centre_x,C2.centre_y,C2.rayon,pointInt1,pointInt2);
    double x1 = pointInt1.x;
    double y1 = pointInt1.y;
    double x2 = pointInt2.x;
    double y2 = pointInt2.y;
    switch (position) {
      case PositionCercleCercle.Vide:
      case PositionCercleCercle.Confondus:
        existe = false;
        break;
      case PositionCercleCercle.Secants:
        if (C1.surArc(x1,y1) && C2.surArc(x1,y1))
          point1Defini = true;
        else point1Defini = false;
        if (C1.surArc(x2,y2) && C2.surArc(x2,y2))
          point2Defini = true;
        else point2Defini = false;
        break;
      case PositionCercleCercle.Tangents:
        if (C1.surArc(x1,y1) && C2.surArc(x1,y1)) {
          existe = true;
          super.placeEn(x1,y1);
          super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
          return;
        } else {
          existe = false;
          return;
        }
    }
    // On ne peut arriver ici que si les deux cercles sont scants}
    if (!point1Defini && !point2Defini) {
      existe = false;
      return;
    }
    if (point1Defini && !point2Defini) {
      //  revoir
      if (Kernel.distancePointPoint(x1,y1,pointExclu.x,pointExclu.y)<0.0001) {
        existe = false;
        return;
      } else {
        super.placeEn(x1,y1);
        super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
        return;
      }
    }
    if (point2Defini && !point1Defini) {
      if (Kernel.distancePointPoint(x2,y2,pointExclu.x,pointExclu.y)<0.0001) {
        existe = false;
        return;
      } else {
        super.placeEn(x2,y2);
        super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
        return;
      }
    }
    //  revoir
    // If Point1defini and Point2Defini
    // then
    // Les deux points sont dfinis
    if (Kernel.zero(Kernel.distancePointPoint(x1,y1,pointExclu.x,pointExclu.y)))
      super.placeEn(x2,y2);
    else super.placeEn(x1,y1);
    super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo148");
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(C1);
    liste.add(C2);
  }   
  
  /** Supprim version 4.7.6. Inutile
  @Override
  public long getNature() {
    return NatureObjet.NPoint;
  }
  */

  @Override
	public boolean confonduAvec(CElementBase p) {
    CAutrePointIntersectionCercles pt;
    if (p.getClass() == this.getClass()) {
      pt = (CAutrePointIntersectionCercles) p;
  		return ((pointExclu == pt.pointExclu)
  				&&
  				(((C1 == pt.C1) && (C2 == pt.C2))
  				|| ((C1 == pt.C2) && (C2 == pt.C1))));
    }
    else return false;
	}

	@Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(C1) || C1.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(C2) || C2.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(pointExclu) || pointExclu.estDefiniParObjetDans(listeOb));
	}  	  
    
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    int ind3 = inps.readCInt();
    C1 = (CCercle) (list.get(ind1,CCercle.class));
    C2 = (CCercle) (list.get(ind2,CCercle.class));
    pointExclu = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }
  
  /* Enregistrement dans un flux java */  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    int ind3 = inps.readInt();
    C1 = (CCercle) (list.get(ind1,CCercle.class));
    C2 = (CCercle) (list.get(ind2,CCercle.class));
    pointExclu = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
    
  }  
  /* Enregistrement dans un flux java */
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(C1);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(C2);
    oups.writeInt(ind2);
    int ind3 = list.indexOf(pointExclu);
    oups.writeInt(ind3);  
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (pointExclu == ancienPoint) pointExclu = nouveauPoint;
  }
  
  @Override
  public boolean appartientParDefinition(CCercle cercle)
  {
    return super.appartientParDefinition(cercle) || (cercle == C1) || (cercle == C2);
  }
}
