/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.geom.*;
import java.awt.Color;


import mathgraph32.MtgFrame;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

public class CAutrePointIntersectionDroiteCercle extends CPointAncetre {
	private CCercle C;
  private CDroiteAncetre D;
	private CPointAncetre pointExclu;
  // Pour le positionnement
  private  Point2D.Double pointInt1 = new Point2D.Double();
  private Point2D.Double pointInt2 = new Point2D.Double();
  
  /** Creates new CAutrePointIntersectionDroiteCercle */
  public CAutrePointIntersectionDroiteCercle() {
  }
  
  // Constructeur pour clonage
  public CAutrePointIntersectionDroiteCercle(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit,
      double dec_yInit, boolean masqueInit, String nomInit, int indiceTailleNomInit, int motifInit,
      CCercle CInit, CDroiteAncetre DInit, CPointAncetre pointExcluInit, boolean marquePourTraceInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
      dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
    C = CInit;
    D = DInit;
    pointExclu = pointExcluInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(C);
    int ind2 = listeSource.indexOf(D);
    int ind3 = listeSource.indexOf(pointExclu);
    int ind4 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CAutrePointIntersectionDroiteCercle(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom, indiceTailleNom, motif,
      (CCercle)(listeCible.get(ind1, CCercle.class)), (CDroiteAncetre)(listeCible.get(ind2, CDroiteAncetre.class)),
      (CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)), marquePourTrace);
    return ptelb;
  }    

  /* Supprim version 4.7.6 : Inutile
  @Override
  public long getNature() {
    return NatureObjet.NPoint;
  }
  */

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	return memoriseDependance(super.dependDe(p) || C.dependDe(p) || D.dependDe(p) || pointExclu.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	return super.dependDePourReclassement(p) || C.dependDePourReclassement(p)
  		|| D.dependDePourReclassement(p) || pointExclu.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
  	return ((p == this) || C.dependDePourBoucle(p) || D.dependDePourBoucle(p)
        || pointExclu.dependDePourBoucle(p));
  }  
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    byte position;
    boolean point1Defini = false;
    boolean point2Defini = false;

    existe = C.existe && D.existe;
    if (!existe) return;
    position = Kernel.intersectionDroiteCercle(D.point_x,D.point_y,D.vecteur,
      C.centre_x,C.centre_y,C.rayon,pointInt1,pointInt2);
    double x1 = pointInt1.x;
    double y1 = pointInt1.y;
    double x2 = pointInt2.x;
    double y2 = pointInt2.y;
    switch (position)
    {
      case PositionDroiteCercle.Vide:
          existe = false;
          return; 
      case PositionDroiteCercle.Secants:
          if (C.surArc(x1,y1) && D.appartientA(x1,y1))
            point1Defini = true;
          else point1Defini = false;
          if (C.surArc(x2,y2) && D.appartientA(x2,y2))
            point2Defini = true;
          else point2Defini = false;
          break;
      case PositionDroiteCercle.Tangents:
          if (C.surArc(x1,y1) && D.appartientA(x1,y1))
          {
            existe = true;
            super.placeEn(x1,y1);
            super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
            return;
          }
          else
          {
            existe = false;
            return;
          }
    }
    // On ne peut arriver ici que si la droite et le cercle sont scants
    if (!point1Defini && !point2Defini)
    {
      existe = false;
      return;
    }
    if (point1Defini && !point2Defini)
    {
      if (Kernel.distancePointPoint(x1,y1,pointExclu.x,pointExclu.y)<0.0001)
      {
        existe = false;
        return;
      }
      else
      {
        super.placeEn(x1,y1);
        super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
        return;
      }
    }
    if (point2Defini && !point1Defini)
    {
      if (Kernel.distancePointPoint(x2,y2,pointExclu.x,pointExclu.y)<0.0001)
      {
        existe = false;
        return;
      }
      else
      {
        super.placeEn(x2,y2);
        super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
        return;
      }
    }
    // Les 2 points sont forcments dfinis
    if (Kernel.zero(Kernel.distancePointPoint(x1,y1,pointExclu.x,pointExclu.y)))
      super.placeEn(x2,y2);
    else super.placeEn(x1,y1);
    super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
  } 
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo149");
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(C);
    liste.add(D);
  }   
  
  @Override
	public boolean confonduAvec(CElementBase p) {
    CAutrePointIntersectionDroiteCercle pt;
    if (p.getClass() == this.getClass()) {
      pt = (CAutrePointIntersectionDroiteCercle) p;
      return (C==pt.C) && (D==pt.D) && (pointExclu==pt.pointExclu);
    }
    else return false;
	}
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(C) || C.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(D) || D.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(pointExclu) || pointExclu.estDefiniParObjetDans(listeOb));
	}  	    
  
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     int ind2 = inps.readCInt();
     int ind3 = inps.readCInt();
     C = (CCercle) (list.get(ind1,CCercle.class));
     D = (CDroiteAncetre) (list.get(ind2,CDroiteAB.class));
     pointExclu = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }   
  /* Enregistrement dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
   int ind1 = inps.readInt();
   int ind2 = inps.readInt();
   int ind3 = inps.readInt();
   C = (CCercle) (list.get(ind1,CCercle.class));
   // D = (CDroiteAncetre) (list.get(ind2,CDroiteAB.class)); Modifi version 4.7.7
   D = (CDroiteAncetre) (list.get(ind2,CDroite.class));
   pointExclu = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(C);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(D);
    oups.writeInt(ind2);
    int ind3 = list.indexOf(pointExclu);
    oups.writeInt(ind3);  
  }
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (pointExclu == ancienPoint) pointExclu = nouveauPoint;
  }
  
  @Override
  public boolean appartientParDefinition(CDroiteAncetre droite)
  {
    return super.appartientParDefinition(droite) ||  (droite == D);
  }
  /** Fonction ajoute pour la version 2.1 en vue d'amliorer l'outil de cration d'intersection
   * Renvoie true si le point est sur la droite par dfinition
   * A redfinir pour les descendants concerns
   * */
  public boolean appartientParDefinition(CCercle cercle)
  {
    return super.appartientParDefinition(cercle) || (cercle == C);
  }
}
