/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.Color;
import java.io.IOException;

import mathgraph32.MtgFrame;

import types.DimensionFenetre;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;


public class CCentreCercle extends CPointAncetre {
	private CCercle cercleAssocie;
	
  /**
   * Constructeur pour serialization
  */
  public CCentreCercle() { 
  }
	
  /** Constructeur pour clonage*/
  public CCentreCercle(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit,
    String nomInit, int indiceTailleNomInit, int motifInit, CCercle cercleAssocieInit, boolean marquePourTraceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
      dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
    cercleAssocie = cercleAssocieInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(cercleAssocie);
    int ind2 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CCentreCercle(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom, indiceTailleNom, motif,
      (CCercle)(listeCible.get(ind1, CCercle.class)), marquePourTrace);
    return ptelb;
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || cercleAssocie.dependDe(p));
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || cercleAssocie.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || cercleAssocie.dependDePourBoucle(p);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = cercleAssocie.existe;
    if (existe) {
    	super.placeEn(cercleAssocie.centre_x, cercleAssocie.centre_y);
    	super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
    }
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo150");
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(cercleAssocie);
  }   
  
  	
  @Override
	public boolean confonduAvec(CElementBase p) {
    CCentreCercle pt;
    if (p.getClass() == this.getClass()) {
      pt = (CCentreCercle) p;
      return cercleAssocie == pt.cercleAssocie;
    }
    else return false;
	}
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(cercleAssocie) || cercleAssocie.estDefiniParObjetDans(listeOb);
	}
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    cercleAssocie = (CCercle) (list.get(ind1,CCercle.class));
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    cercleAssocie = (CCercle) (list.get(ind1,CCercle.class));
    
  }  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(cercleAssocie);
    oups.writeInt(ind1);
  }   
}
