/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;

public class CDroiteDirectionFixe extends CDroite {
  CPointAncetre A;
  private boolean droiteHorizontale; //la droite sera horizontale si true
  //et verticale sinon
  private double longueurVecteur;
  
  /** Creates new CDroiteDirectionFixe */
  public CDroiteDirectionFixe() {
    super(); // Pour que le vecteur soit cr
  }
  
  // Constructeur pour clonage
  public CDroiteDirectionFixe(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit,
    String nomInit, int indiceTailleNomInit, int styleInit, double abscisseNomInit,
    CPointAncetre AInit, boolean droiteHorizontaleInit, double longueurVecteurInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
      dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, styleInit, abscisseNomInit);
    A = AInit;
    droiteHorizontale  = droiteHorizontaleInit;
    longueurVecteur = longueurVecteurInit;
    if (droiteHorizontale) {
      vecteur.x = longueurVecteur;
      vecteur.y = 0;
    }
    else {
      vecteur.x = 0;
      vecteur.y = longueurVecteur;
    }
  }
  /**  Constructeur simple */
  public CDroiteDirectionFixe(CListeObjets liste, Color couleurInit, int styleInit,
    CPointAncetre AInit, boolean droiteHorizontaleInit, int indiceTailleNomInit) {
    super(liste,null, false, couleurInit, false, styleInit, indiceTailleNomInit);
    A = AInit;
    droiteHorizontale = droiteHorizontaleInit;
    longueurVecteur = 1; // 10 dans la version C++
    if (droiteHorizontale) {
      vecteur.x = longueurVecteur;
      vecteur.y = 0;
    }
    else {
      vecteur.x = 0;
      vecteur.y = longueurVecteur;
    }
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(A);
    int ind2 = listeSource.indexOf(impProto);    
    CElementBase ptelb = new CDroiteDirectionFixe(listeCible,(CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom, indiceTailleNom,
      style, abscisseNom, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)), droiteHorizontale, longueurVecteur);
    return ptelb;
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CDroiteDirectionFixe pt;
    if (p.getClass() == this.getClass()) {
      pt = (CDroiteDirectionFixe) p;
      return (droiteHorizontale == pt.droiteHorizontale) && (A == pt.A);
    }
    else return false;
  }  
  
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || A.dependDe(p));
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || A.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || A.dependDePourBoucle(p));
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = A.existe;
    if (!existe) return;
    point_x = A.x;
    point_y = A.y;
    super.positionne(infoRandom, dimfen);
  }
  
  @Override
  public String infoPourHistorique() {
    if (droiteHorizontale) return getString("chinfo50");
    else return getString("chinfo51");
  }
  
  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(A);
  } 
  
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (A == ancienPoint) A = nouveauPoint;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(A) || A.estDefiniParObjetDans(listeOb);
	}
  
  /**
   * Fonction utilise quand on zoome ou dzoome sur la figure
   * @param rapport
   */
  public final void zoom(double rapport) {
    longueurVecteur = longueurVecteur*rapport;
    if (droiteHorizontale) vecteur.x = longueurVecteur;
  	else vecteur.y = longueurVecteur;  	
  }
  
  @Override
  public void adaptePourHauteResolution(double coef) {
  	super.adaptePourHauteResolution(coef);
		longueurVecteur *= coef;
		vecteur.x *= coef;
		vecteur.y *= coef;
  }  
 
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    A = (CPointAncetre) (list.get(ind1, CPointAncetre.class));
    droiteHorizontale = inps.readBoolean();
    longueurVecteur = inps.readCDouble()/10;
    // A revoir plus tard. Inutile d'enregistrer LongueurVecteur dans le flux
    if( droiteHorizontale ) {
      vecteur.x = longueurVecteur;
      vecteur.y = 0;
    } else {
      vecteur.x = 0;
      vecteur.y = longueurVecteur;
    }
  }   
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    A = (CPointAncetre) (list.get(ind1, CPointAncetre.class));
    droiteHorizontale = inps.readBoolean();
    longueurVecteur = inps.readDouble();
    // A revoir plus tard. Inutile d'enregistrer LongueurVecteur dans le flux
    if( droiteHorizontale ) {
      vecteur.x = longueurVecteur;
      vecteur.y = 0;
    } else {
      vecteur.x = 0;
      vecteur.y = longueurVecteur;
    }
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(A);
    oups.writeInt(ind1);
    oups.writeBoolean(droiteHorizontale);
    oups.writeDouble(longueurVecteur);
  }  
  /**
   * Fonction rajoute pour la version 2.1 et servant  savoir si un point
   * est par dfinition sur une droite. Renverra true par exemple
   * si la droite est dfinie par deux points et le point est un de ces deux points
   * ou si le point est li  la droite
   * A t rajout pour amliorer l'outil d'intersection
   * A redfinir pour les descendants quand ncessaire
   * */
  @Override
  public boolean contientParDefinition(CPointAncetre po)
  {
    return (A == po);
  }
  
  @Override
  public boolean estParalleleParDefinition(CDroiteAncetre dte){
  	return (dte == this)
  	|| ((dte.getClass() == CDroiteDirectionFixe.class) &&  (((CDroiteDirectionFixe)dte).droiteHorizontale = droiteHorizontale))
  	|| dte.estParalleleParDefinition(this);
  }
  
  @Override
  public boolean estPerpendiculaireParDefinition(CDroiteAncetre dte) {
  	return ((dte.getClass() == CDroiteDirectionFixe.class) &&  (((CDroiteDirectionFixe)dte).droiteHorizontale != droiteHorizontale))
  	|| dte.estParalleleParDefinition(this);  
  }
  
  
}
