/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import javax.imageio.ImageIO;
import mathgraph32.MtgFrame;
import dialogs.ImageDlg;
import dialogs.MtgDlg;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CImage extends CAffichageLieAPoint {
	// Nouveau version 4.3 : on mmorise le type de l'image (3 types possibles : png (par dfaut) ou gig ou jpeg)
	
	public byte natImage;
  public BufferedImage image; // l'image. Celle-ci est charge avec les autres images lors
  // de la serialization du document.
  // Modifi version 4.4. Plus de imagePath. L'image est incorpore dans le fichier mgj
	public String imagePath = "";  // Le chemin d'accs relatif au fichier. Plus utilis  partir de la version 4.4
	// mais maintenu pour compatibilit avec les anciennes versions.
	// String absolutePath; // Le chemin total d'accs. Ne sera utilis que quand
	// on dite une figure incorpore dans un document ( supposer que la figure
	// n'ait pas t dplace ... Ne sert pas en java

	/**
	 * Modification pour la version 4.4 : les images sont directement enregistres dans le flix bineaire et on
	 * ne sauvegarde plus le chemin d'accs au fichier image.
	 */
  @Override
  public int numeroVersion() {
  	return 2;
  }  
	
  /** Creates new CImage */
  public CImage() {
  }

  /* Modifi version 4.4 : on ne momorise plus imagePath
  public CImage(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit,
      double dec_yInit, boolean masqueInit, CPointAncetre pointLieInit, int indiceTaillePoliceInit,
      int encadrementInit, boolean effacementFondInit, Color couleurFondInit, String imagePathInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
      dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit, encadrementInit,
      effacementFondInit, couleurFondInit);
    imagePath = imagePathInit;
  }
  */

  public CImage(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit,
      double dec_yInit, boolean masqueInit, CPointAncetre pointLieInit, int indiceTaillePoliceInit,
      int encadrementInit, boolean effacementFondInit, Color couleurFondInit, byte natImageInit,
      BufferedImage imageInit, int 	alignementHorizontalInit, int alignementVerticalInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
      dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit, encadrementInit,
      effacementFondInit, couleurFondInit, alignementHorizontalInit, alignementVerticalInit);
    natImage = natImageInit;
    image = imageInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(pointLie);
    int ind2 = listeSource.indexOf(impProto);
    /*
    CElementBase ptelb = new CImage(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y,
      masque, (CPointAncetre)listeCible.get(ind1, CPointAncetre.class), indiceTaillePolice, encadrement,
      effacementFond, couleurFondAffichage, imagePath);
    */
    CElementBase ptelb = new CImage(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
        estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y,
        masque, (CPointAncetre)listeCible.get(ind1, CPointAncetre.class), indiceTaillePolice, encadrement,
        effacementFond, couleurFondAffichage, natImage, image, alignementHorizontal, alignementVertical);
    // A revoir version 4.7.7 : les deux lignes suivantes sont inutiles
    // ((CImage)ptelb).natImage = natImage;
    // ((CImage)ptelb).image = image;
    ((CImage)ptelb).rectangleAffichage.width = rectangleAffichage.width;    
    ((CImage)ptelb).rectangleAffichage.height = rectangleAffichage.height;    
    return ptelb;
  }

  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    super.positionne(infoRandom, dimfen);
    if (! existe) return;
    rectangleAffichage.x = (int) (x_nom + dec_x);
    rectangleAffichage.y = (int) (y_nom + dec_y);
    // Modification version 4.5
    switch (alignementHorizontal) {
    case alignementHorizontalCentre :
    	rectangleAffichage.x -= rectangleAffichage.width/2;
    	break;
    case alignementHorizontalDroit :
    	rectangleAffichage.x -= rectangleAffichage.width;
    }
	  switch (alignementVertical) {
	  	case alignementVerticalCentre :
	  		rectangleAffichage.y -= rectangleAffichage.height/2;
	  		break;
	  	case alignementVerticalBas :
	  		rectangleAffichage.y -= rectangleAffichage.height;
	  }
	  //
    xEncadrement = rectangleAffichage.x;
    yEncadrement = rectangleAffichage.y;
    widthEncadrement = rectangleAffichage.width;
    heightEncadrement = rectangleAffichage.height;
 
    switch (encadrement) {
	    case StyleEncadrement.Simple :
	      xEncadrement -= 1;
	      yEncadrement -= 1;
	      widthEncadrement += 2;
	      heightEncadrement += 2;
	      break;
	    case StyleEncadrement.Effet3D :
	      xEncadrement -= 1;
	      yEncadrement -= 1;
	      widthEncadrement += 3;
	      heightEncadrement += 3;
    }
  }  
 
  @Override
  public void affiche(Graphics2D pDC, boolean masquage, Color couleurFond, float transparence) {
    if (!masque) trace(pDC, couleurFond, transparence);
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    if (effacementFond)
    {
      //Affichage avec effacement du fond
      // pDC.setColor(couleurFond);
      pDC.fillRect(rectangleAffichage.x, rectangleAffichage.y, rectangleAffichage.width,
      		rectangleAffichage.height);
      pDC.drawImage(image,rectangleAffichage.x, rectangleAffichage.y, rectangleAffichage.width,
        rectangleAffichage.height, couleurFond, null);
    }
    else
    {
      pDC.drawImage(image,rectangleAffichage.x, rectangleAffichage.y, rectangleAffichage.width,
        rectangleAffichage.height, null);
    }
    if (encadrement != 0)  //  revoir
    {
      pDC.setColor(couleur);
      pDC.drawRect(xEncadrement, yEncadrement, rectangleAffichage.width + 1, rectangleAffichage.height + 1);
    }
    if (encadrement == StyleEncadrement.Effet3D) {
      int x1 = xEncadrement + rectangleAffichage.width + 2;
      int y1 = yEncadrement + rectangleAffichage.height + 2;
      pDC.drawLine(xEncadrement + 1, y1, x1, y1);
      pDC.drawLine(x1, yEncadrement + 1, x1, y1);
    }
    
  }

  // Doit tre redfinie car est de base Abstrac mais ne sert pas pour les images
  protected String rendChaineAffichage(boolean infoRandom) {
    return "";
  }
  
  @Override
  public String infoPourHistorique() {
    String ch;
    if (pointLie == null) ch = getString("chinfo172") + " " + getString("chinfo48");
    else ch = getString("chinfo172") + " " + getString("chinfo49");
    // ch = ch + "\n" + getString("chinfo173") + imagePath; // Supprim version 4.4
    return ch;
  }
  
  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    if (pointLie != null) liste.add(pointLie);
  }   
  
  /** Supprim version 4.4
  @Override
  public boolean confonduAvec(CElementBase p) {
    CImage pt;
    if (p.getClass() == this.getClass()) {
      pt = (CImage) p;
      return (imagePath == pt.imagePath);
    }
    else return false;
  }
  */
  
  @Override
  public long getNature() {
    return NatureObjet.NImage;
  }
  
  @Override
  public boolean modifiableParMenu() {
  	return true;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new ImageDlg(frame, this, true);
  }
  
   @Override
   public String chaineDesignation() {
     return getString("designationImage");
   }
   
   @Override
   public void adaptePourHauteResolution(double coef) {
   	super.adaptePourHauteResolution(coef);
  		rectangleAffichage.width *= coef;
  		rectangleAffichage.height *= coef;
   }   
  
   @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    imagePath = inps.readCString();
    @SuppressWarnings("unused")
		String absolutePath = inps.readCString();
    rectangleAffichage.width = inps.readCInt()/10;
    rectangleAffichage.height = inps.readCInt()/10;
    prepareImage(list);
  }

   @Override
   public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    if (nVersion < 2) {
    	natImage = MathGraphDataOutputStream.nat_png;
	    imagePath = inps.readUTF();
	     // AbsolutePath n'est plus enregistr dans le flux en java
	    // absolutePath = inps.readUTF();
	    // Bien que le programme java n'ait pas besoin de connatre des dimensions de l'image, l'applet en a besoin
	    // On enregistre donc dans le flux la largeur et la hauteur de l'image
	    rectangleAffichage.width = inps.readInt();
	    rectangleAffichage.height = inps.readInt();
	    prepareImage(list);
    }
    else {
	    natImage = inps.readByte();
	    rectangleAffichage.width = inps.readInt();
	    rectangleAffichage.height = inps.readInt();
    	image = inps.readBufferedImage();
    }
  }  

   @Override
   public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    // oups.writeUTF(imagePath); Plus utilis version 4.4
    // AbsolutePath n'est plus enregistr dans le flux en java
    // oups.writeUTF(absolutePath);
    // Bien que le programme java n'ait pas besoin de connatre des dimensions de l'image, l'applet en a besoin
    // On enregistre donc dans le flux la largeur et la hauteur de l'image
    oups.writeByte(natImage);
    oups.writeInt(rectangleAffichage.width);
    oups.writeInt(rectangleAffichage.height);
    oups.writeBufferedImage(image, natImage);
  }
  
  public void prepareImage(CListeObjets list) throws IOException {
    if (imagePath.length() != 0) {
    	natImage = MathGraphDataOutputStream.nat_png;
    	String path = list.documentProprietaire.cheminRacine();
    	// ImageIcon imic = new ImageIcon(path + imagePath);
    	// image = (BufferedImage) imic.getImage();
    	File file = new File(path + imagePath);
    	image = ImageIO.read(file);
    	rectangleAffichage.width = image.getWidth(null);
     	rectangleAffichage.height = image.getHeight(null);
    }
  }
}
