	
/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.util.ArrayList;
import java.util.ListIterator;

import mathgraph32.MtgInternalFrame;
import types.FonteNom;
import types.NatureCalcul;
import types.NatureObjet;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 * 
 * @author Yves
 */
public class CImplementationProto extends CElementBase {
	public String nomProto; // Le nom du prototype qui l'a gnr
	public int nbIntermediaires;
	public int nbFinaux;
	public CSousListeObjets listeSources;

	/** Creates a new instance of CImplementationProto */
	public CImplementationProto() {
		listeSources = new CSousListeObjets();
	}

	/**
	 * Constructeur pour clonage
	 */
	public CImplementationProto(CListeObjets liste, String nomProtoInit, int nbIntermediairesInit,
			int nbFinauxInit, CSousListeObjets listeSourcesInit) {
		super(liste, null, false);
		nomProto = nomProtoInit;
		nbIntermediaires = nbIntermediairesInit;
		nbFinaux = nbFinauxInit;
		listeSources = listeSourcesInit;
	}

	/**
	 * Constructeur pour implmentation  partir d'un protoype Avanat appel les
	 * lments gnriques du prototype ont t mis  pointer sur les objets
	 * sources dans liste
	 * 
	 * @param liste
	 * @param nomProtoInit
	 * @param nbIntermediairesInit
	 * @param nbFinauxInit
	 */
	public CImplementationProto(CListeObjets liste, CPrototype proto) {
		super(liste, null, false);
		nomProto = proto.nom;
		listeSources = new CSousListeObjets();
		int nbSources = proto.nbObjSources;
		for (int i = 0; i < nbSources; i++) {
			listeSources.add(((CElementGenerique) proto.get(i)).elementAssocie);
		}
		nbFinaux = proto.nbObjFinaux;
		nbIntermediaires = proto.size() - nbSources - nbFinaux;
	}

  @Override
	public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
		CSousListeObjets listeSourcesClone = new CSousListeObjets();
		listeSourcesClone.setImage(listeSources, listeSource, listeCible);
		CElementBase ptelb = new CImplementationProto(listeCible, nomProto, nbIntermediaires, nbFinaux,
				listeSourcesClone);
		return ptelb;
	}

	public final CElementBase dernierObjetFinal() {
		int ind = listeProprietaire.indexOf(this);
		return listeProprietaire.get(ind + nbIntermediaires + nbFinaux);
	}

	@Override
	public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
		return memoriseDependance(super.dependDe(p) || listeSources.dependDe(p));
	}

	@Override
	public boolean dependDePourReclassement(CElementBase p) {
		return super.dependDePourReclassement(p) || listeSources.dependDePourReclassement(p);
	}

	/**
	 * Fonction renvoyant true lorsque el dpend de au moins un des objets finaux
	 * 
	 * @param el
	 */
	public final boolean dependDeUnObjetFinal(CElementBase el) {
		int ind = listeProprietaire.indexOf(this);
		int indFin = ind + nbIntermediaires + nbFinaux;
		for (int i = ind + 1; i <= indFin; i++) {
			CElementBase elb = (CElementBase) listeProprietaire.get(i);
			if (elb.estElementFinal) {
				if (el.dependDe(elb)) {
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public String infoPourHistorique() {
		return getString("chinfo138") + "\n" + nomProto;
	}

	@Override
	public long getNature() {
		return NatureObjet.NImplementationProto;
	}

  @Override
	public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
		listeSources.remplacePoint(ancienPoint, nouveauPoint);
	}

	/**
	 * Implmentation dans la liste qui contiendra l'implmentation du protoype,
	 * suivie des objets intermdiares et des objets finaux. Pendant cette
	 * implmentation les listes listeSources, listeIntermediaires et listeFinaux
	 * sont mises  jour. Renvoie une chane vide si l'implmentation est possible
	 * et sinon le message d'erreur  afficher Avant l'appel les lments
	 * gnriques du prototype proto ont vu leur pointeur elementAssocie pointer
	 * sur l'objet source correspondant de la liste propritaire de tous les
	 * objets
	 * Version 4.5 : Ajout d'un troisime paramtre cadre
	 */
	public void implemente(CListeObjets listePourImp, CPrototype proto, MtgInternalFrame cadre) {
		int i;
		ArrayList<CElementGraphique> listePointsOuDroitesARenommer = new ArrayList<CElementGraphique>();
		ArrayList<CValeurDynamique> listeCalculsARenommer = new ArrayList<CValeurDynamique>();
		CListeObjets listeEltsAjoutes = new CListeObjets();
		nomProto = proto.nom;
		proto.setListePourImplementation(listeProprietaire);
		// On ajoute l'objet implementationPrototype  la liste
		listeProprietaire.add(this);
		// Important que que les indices soient corrects
		proto.indicePremierObjetPourImplementation = listeProprietaire.size();
		// On implmente la liste d'objets sources gre par l'implmentation de
		// prototype
		// en recherchant dans la liste des lments sources les points et droites qui ont un nom
		// dans le prototype et on leur gnre un nom pour que les ventuelles mesures
		// et calculs l'utilisant aient leur formules ractualises
		for (i = 0; i < proto.nbObjSources; i++) {
			CElementBase elb = ((CElementGenerique) proto.get(i)).elementAssocie;
			// listeSources.add(elb);
			if (elb.estDeNature(NatureObjet.NToutPoint|NatureObjet.NDroite)) {
				CElementGenerique ptgen = (CElementGenerique) proto.get(i);
				CElementGraphique ptp = (CElementGraphique) elb;
				String nomGen = ptgen.nom;
				// On regarde si l'lment correspondant tait nomm.
				// Si oui on rajout cet lment  la liste des points devant tre
				// nomms
				if ((nomGen.length() != 0) && (ptp.rendNom().length() == 0)) {
					ptp.donneNom(nomGen);
					listePointsOuDroitesARenommer.add(ptp);
				}
			}
		}
		// Puis on ajoute des clones des objets intermdiaires et finaux du
		// prototype
		for (i = proto.nbObjSources; i < proto.size(); i++) {
			CElementBase elb = proto.get(i);
			CElementBase clone = elb.getClone(proto, listeProprietaire);
			clone.estElementFinal = elb.estElementFinal;
			clone.impProto = this; // Important
			listePourImp.add(clone);
			listeEltsAjoutes.add(clone);
			// On regarde s'il y a des points ou des droites dans les lments
			// intermdiaires
			// ou finaux ayant le mme nom que d'autres dj dfinis et si oui on les
			// renomme.
			if (clone.estDeNature(NatureObjet.NToutPoint)) {
				CPointAncetre ptp = (CPointAncetre) clone;
				if (listeProprietaire.existeMemeNomNonIntermediaire(ptp, ptp.rendNom()))
					listePointsOuDroitesARenommer.add(ptp);
			} else {
				if (clone.getNature() == NatureObjet.NDroite) {
					CDroite ptd = (CDroite) clone;
					if (listeProprietaire.existeMemeNomNonIntermediaire(ptd, ptd.rendNom()))
						ptd.donneNom(listeProprietaire.genereNomPourPointOuDroite('D'));
				} else {
					if (clone.estDeNatureCalcul(NatureCalcul.NToutCalculNomme)) {
						CValeurDynamique vald = (CValeurDynamique) clone;
						if (listeProprietaire.existeCalculOuVariableOuFonctionOuParametreMemeNom(null, vald.nom(), false))
							listeCalculsARenommer.add(vald);
					}
				}
			}
		}
		// On renomme maintenant les points ou droites qui ont besoin d'tre renomms
		ListIterator<CElementGraphique> li1 = listePointsOuDroitesARenommer.listIterator();
		while (li1.hasNext()) {
			CElementGraphique ptp = li1.next();
			String ancienNom = ptp.rendNom();
			String nouveauNom = listeProprietaire.genereNomPourPointOuDroite(ancienNom.charAt(0), ptp);
			ptp.donneNom(nouveauNom);
			// listeEltsAjoutes.MetAJourFormulesCalculsDependantDe(ptp); Ne pas appeler  ce niveau
			// car des fomules peuvent dpendre des objets sources
		}
		// On renomme les calculs intermdiaires qui ont besoin d'tre renomms
		// en mettant  jour les affichages de commentaires dynamiques qui en
		// dpendent
		ListIterator<CValeurDynamique> li2 = listeCalculsARenommer.listIterator();
		while (li2.hasNext()) {
			CValeurDynamique val = li2.next();
			String ancienNom = val.nom();
			// Amlioration par rapport au C++. On gnre un nouveau nom avec
			// seulement les premiers caractres qui ne sont pas des chiffres
			String nouveauNom = listeProprietaire.genereNomPourCalcul(FonteNom
					.premiersCaracteresNonChiffres(ancienNom), val);
			listeEltsAjoutes.remplaceNomValeurDynamiqueDansCommentairesOuLatex(ancienNom, nouveauNom);
			val.donneNom(nouveauNom);
		}
		// On reconstitue les formules des calculs ou fonctions intermdiaires et
		// finaux dpendant des objets sources
		ListIterator<CElementBase> li3 = listeEltsAjoutes.listIterator();
		while (li3.hasNext()) {
			CElementBase elb = li3.next();
			if (elb.estDeNatureCalcul(NatureCalcul.NCalculOuFonctionNonConstante)) {
				CCalcul calc = (CCalcul) elb;
				calc.reconstruitChaineCalcul();
			}
		}
		for (i = 0; i < proto.nbObjSources; i++) {
			CElementBase elb = ((CElementGenerique) proto.get(i)).elementAssocie;
			if (elb.estDeNatureCalcul(NatureCalcul.NTouteValeurPourCommentaireDynamique)) {
				CElementGenerique gen = (CElementGenerique) proto.get(i);
				CValeurDynamique val = (CValeurDynamique) elb;
				listeEltsAjoutes.remplaceNomValeurDynamiqueDansCommentairesOuLatex(gen.nom, val.nom());
			}
		}
		// On met  jour les commentaires des lments rajouts au cas o ils
		// contiendraient
		// des affichages de valeurs dynamiques
		listeEltsAjoutes.determineDependancesCommentaires();
		listeEltsAjoutes.metAJourEtPositionne(cadre.infRand, cadre.dimf);
	}

	/**
	 * Fonction renvoyant le nombre d'objets finaux de type calcul ayant un nom
	 * Est utilis lors de la fin de l'implmentation d'une construction pour
	 * proposer de renommer ces calculs
	 * 
	 * @return
	 */
	public int nombreObjetsFinauxTypeCalculNommes() {
		int res = 0;
		int ind = listeProprietaire.indexOf(this);
		for (int i = ind + 1; i <= ind + nbFinaux + nbIntermediaires; i++) {
			CElementBase elb = listeProprietaire.get(i);
			if (elb.estElementFinal && elb.estDeNatureCalcul(NatureCalcul.NToutCalculNommeSaufConst))
				res++;
		}
		return res;
	}
	
	/**
	 * Fonction rajout version 3.6.1 pour corriger des bugs lors du reclassement d'objets
	 * @return
	 */
	public void  determineNombreObjetsIntermediaires() {
		int indimp = listeProprietaire.indexOf(this);
		int nb = 0;
		for (int i = indimp+1; i < listeProprietaire.size(); i++) {
			CElementBase elb = listeProprietaire.get(i);
			if ((elb.impProto == this) && !elb.estElementFinal) nb++;
			else break;
		}
		nbIntermediaires = nb;
	}	

	// C++ : chargeDepuisFlux a t ajout  la version 2.5 pour cause d'implmentation
	// de prototype
  @Override
	public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
		super.chargeDepuisFlux(inps, list);
		nomProto = inps.readCString();
		nbIntermediaires = inps.readCInt();
		nbFinaux = inps.readCInt();
		listeSources.chargeDepuisFlux(inps, list);
	}

  @Override
	public void read(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
		super.read(inps, list);
		nomProto = inps.readUTF();
		nbIntermediaires = inps.readInt();
		nbFinaux = inps.readInt();
		listeSources.read(inps, list);
		// Pour palier bug sur certaines figures
	}

  @Override
	public void write(MathGraphDataOutputStream oups, CListeObjets list) throws IOException {
		super.write(oups, list);
		oups.writeUTF(nomProto);
		oups.writeInt(nbIntermediaires);
		oups.writeInt(nbFinaux);
		listeSources.write(oups, listeProprietaire);
	}
}
