/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;

import dialogs.MacroIncrementationVariableDlg;
import dialogs.MtgDlg;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class CMacroIncrementationVariable extends CMacro {
  public CVariableBornee variableAssociee;
  CListeObjets listeDep;
  /** Creates new CMacroAffectationValeurVariable */
  public CMacroIncrementationVariable() {
    // Il faut crer la liste des lments dont dpend le point  tracer
    listeDep = new CListeObjets();    
  }
  
  // Constructeur pour clonage
  public CMacroIncrementationVariable(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit, int encadrementInit,
      boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit,
      CVariableBornee variableAssocieeInit, int alignementHorizontalInit, int alignementVerticalInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit,
          effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit,
					alignementHorizontalInit, alignementVerticalInit);
        variableAssociee = variableAssocieeInit;
      // Il faut crer la liste des lments dpendant de la variable
      listeDep = new CListeObjets();    
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
   int ind1 = listeSource.indexOf(variableAssociee);
   int ind2 = listeSource.indexOf(pointLie);
   int ind3 = listeSource.indexOf(impProto);       
   CElementBase ptelb = new CMacroIncrementationVariable(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, masque,
      (CPointAncetre)listeCible.get(ind2, CPointAncetre.class), indiceTaillePolice, 
      encadrement, effacementFond, couleurFondAffichage, intitule, commentaireMacro,
      (CVariableBornee)(listeCible.get(ind1, CVariableBornee.class)), alignementHorizontal, alignementVertical);
    listeDep.initialise(listeCible.uniteAngle, listeCible.pointeurLongueurUnite);
    // On ne peut pas  ce niveau faire une image de la liste ListeDep dans la liste clone car tous
    // les lments n'ont pas encor t chargs.
    //((CMacroIncrementationVariable)ptelb).listeDep.setImage(listeDep, listeProprietaire,
    //  ptelb.listeProprietaire);
    return ptelb;
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || variableAssociee.dependDe(p));
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || variableAssociee.dependDePourReclassement(p);
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return super.dependDePourBoucle(p) || variableAssociee.dependDePourBoucle(p);
  }

  @Override
  public boolean agitSurVariable(CVariableBornee var) {
  	return (var == variableAssociee);
  }

  @Override
  public void actionDansPaint(MtgInternalFrame cadre) {
    boolean infor = cadre.infRand;
    DimensionFenetre dimfen = cadre.dimf;
    // Modifi pour la version 2.0
    // listeDep.effaceTout(cadre.gbuffer, dimfen, true);
    variableAssociee.incremente();
    listeDep.positionne(infor, dimfen);
    // Ligne suivante ajoute version 2.0
    cadre.effacegbuffer();
    cadre.afficheToutDansgbuffer();   
    executionEnCours = false;
  }

  /** Mthode charge de remettre la figure en tat initial
   * et n'existant pas dans l'applet. Charge de mettre  jour
   * le dialogue ventuel d'affichage des variables en bas  droite
   */
  public void termineAction(MtgInternalFrame cadre) {
    super.termineAction(cadre);
    // cadre.metAJourDialogueAffichageVariables(variableAssociee);
    cadre.metAJourPaneAffichageVariables(variableAssociee);
  }
  
  @Override
  public boolean executionPossible() {
    boolean infr = false; // Pas de recalcul alatoire
    double nouvelleValeur = variableAssociee.rendValeur(infr) + variableAssociee.valeurPas;
    return (nouvelleValeur >= variableAssociee.valeurMini)
        && (nouvelleValeur <= variableAssociee.valeurMaxi);
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo121") + " " + variableAssociee.nom();
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
		return super.estDefiniParObjetDans(listeOb)
		&& (listeOb.contains(variableAssociee) || variableAssociee.estDefiniParObjetDans(listeOb));
	}  
  
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new MacroIncrementationVariableDlg(frame, this, true);
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    variableAssociee = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    // Il faut crer la liste des lments dpendant de la variable modifie
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la lsite mre
    listeDep = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    variableAssociee = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    // Il faut crer la liste des lments dpendant de la variable modifie
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la lsite mre
    listeDep = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(variableAssociee);
    oups.writeInt(ind1);
  }
  
  @Override
  public void etablitListesInternes() {
    listeDep = new CListeObjets(listeProprietaire.uniteAngle, listeProprietaire.pointeurLongueurUnite);        
    listeDep.ajouteObjetsDependantsSauf(variableAssociee, listeProprietaire, this);
  }
  
  // Suprim version 2.0 car provoques des stackoverflow
  /*
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
  	listeDep.remplacePoint(ancienPoint, nouveauPoint);
  }
  */
  
}
