/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import java.applet.AudioClip;
import java.awt.Color;
import mathgraph32.MtgInternalFrame;

public class CMacroJouantSon extends CMacro {
  public Clip clip;
  public AudioClip audioClip; // Spcial version applet
  // Si c'est via une URL qu'on a charg la figure, c'est audioClip qui sera utilis
  public String wavePath;
  // Plus de membre executionArrierePlan dans la version java
	// boolean executionArrierePlan;

  /** Creates new CMacroJouantSon */
  public CMacroJouantSon() {
    clip = null;
    audioClip= null; // Spcial version applet
  }
  
  // Constructeur pour clonage
  public CMacroJouantSon(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit,  boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit, boolean effacementFondInit, Color couleurFondInit,
      String intituleInit, String commentaireMacroInit, String wavePathInit,
    	int alignementHorizontalInit, int alignementVerticalInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit,
          effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit,
  				alignementHorizontalInit, alignementVerticalInit);
        wavePath = wavePathInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(pointLie); 
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMacroJouantSon(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, masque,
      (CPointAncetre)listeCible.get(ind1, CPointAncetre.class), indiceTaillePolice,
      effacementFond, couleurFondAffichage, intitule, commentaireMacro, wavePath,
      alignementHorizontal, alignementVertical);
    ((CMacroJouantSon) ptelb).clip = this.clip;
    ((CMacroJouantSon) ptelb).audioClip = this.audioClip; // Spcial version applet
    return ptelb;  
  }
  
  // Le clip audio est maintenant charg par un Thread en tche de fond
  public void actionAvantExecution(MtgInternalFrame cadre) {
  	super.actionAvantExecution(cadre);  // Met executioEnCours  true
  	executionEnCours = false;
  	if (clip != null) {
	    if (clip.isRunning()) clip.stop();
	    clip.setFramePosition(0);
	    clip.start();
  	}
  	else {
  		if (audioClip != null) {
  	    audioClip.stop();
  	    audioClip.play();  			
  		}
  	}
  }
  
  public boolean executionPossible() {
  	// Spcial version applet
    // return (clip != null);
  	return (clip != null) || (audioClip != null);
  }
  
  /**
   * mthode qui ne sera dfinie que pour les macros jouant un fichier sonore.
   * Pour celles-ci, le fichier sonore est charg via un Thread et il faut savoir
   * quand le clip est fini de charger par le Thread
   * @return
   */
  boolean chargementSonOK() {
  	// Spcial version applet
    // return (clip != null);
  	return (clip != null) || (audioClip != null);
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo123") + " " + wavePath;
  }
  
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
		return false;
	}  
  
  public void prepareClip(CListeObjets list) {
    try {
	    if (list.documentProprietaire != null) {
	    	String racine = list.documentProprietaire.cheminRacine();
	    	String ch = racine + wavePath;
	    	File file = new File(ch);
        AudioInputStream audioInputStream = AudioSystem.getAudioInputStream(file);
	    	// On charge les donnes sonores dans clip
	    	clip = AudioSystem.getClip();
	    	clip.open(audioInputStream);
	    } 
    } catch (LineUnavailableException e) {
	    	clip = null;
	  } catch (UnsupportedAudioFileException ex) {
	  	clip = null;
	  }
	  catch (IOException e) {
	  	clip = null;
		}  	
  }
  
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 3.5
  	// return true;
  	// Spcial version applet
  	// return !estElementFinal;
  	return false;
  }
  
  /* Spcial version applet
  @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new MacroJouantSonDlg(frame, this, true);
  }
  */
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    wavePath = inps.readCString();
    // Plus utilis dans la version java
    @SuppressWarnings("unused")
		boolean executionArrierePlan = inps.readBoolean();
    prepareClip(list);
  }
  
  @Override
  public void chargeDepuisFluxFromURL(MathGraphDataInputStream inps,
  		CListeObjets list) throws IOException {
  	// TODO Auto-generated method stub
  	super.chargeDepuisFlux(inps, list);
    wavePath = inps.readCString();
    // Plus utilis dans la version java
    @SuppressWarnings("unused")
		boolean executionArrierePlan = inps.readBoolean();
    // prepareClip(list); / Pas de prepareClick si on charge en ligne
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    wavePath = inps.readUTF();
    // executionArrierePlan = inps.readBoolean();    
    prepareClip(list);
  }
  
  @Override
  public void readFromURL(MathGraphDataInputStream inps, CListeObjets list)
  		throws IOException {
    super.read(inps, list);
    wavePath = inps.readUTF();
  }

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeUTF(wavePath);
    // oups.writeBoolean(executionArrierePlan);
  }  
}
