/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

// Java :  noter que le fonctionnement de cette macro est amlior par rapport au C++ avec
// l'utilisation d'une liste d'objets sur les lements dpendant de la variable modifie
import java.io.*;
import java.awt.Color;

import dialogs.MacroModificationVariableDlg;
import dialogs.MtgDlg;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class CMacroModificationVariable extends CMacro {
	public CValeurAssocieeAVariable valeurMiniAssociee;
 	public CValeurAssocieeAVariable valeurMaxiAssociee;
 	public CValeurAssocieeAVariable valeurPasAssociee;
 	public CValeurAssocieeAVariable valeurActuelleAssociee;
	public CVariableBornee variableAssociee;
  private CListeObjets listeDep;

  /** Creates new CMacroModificationVariable */
  public CMacroModificationVariable() {
  }
  
  // Constructeur pour clonage
  public CMacroModificationVariable(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit,
      boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit,
      CValeurAssocieeAVariable valeurMiniAssocieeInit, CValeurAssocieeAVariable valeurMaxiAssocieeInit,
      CValeurAssocieeAVariable valeurPasAssocieeInit, CValeurAssocieeAVariable valeurActuelleAssocieeInit,
      CVariableBornee variableAssocieeInit, int alignementHorizontalInit, int alignementVerticalInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit,
          effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit,
					alignementHorizontalInit, alignementVerticalInit);
        valeurMiniAssociee = valeurMiniAssocieeInit;
        valeurMaxiAssociee = valeurMaxiAssocieeInit;
        valeurPasAssociee = valeurPasAssocieeInit;
        valeurActuelleAssociee = valeurActuelleAssocieeInit;
        variableAssociee = variableAssocieeInit;
        // Il faut crer la liste des lments dpendant de la variable
        listeDep = new CListeObjets();    
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CValeurAssocieeAVariable valeurMiniClone = (CValeurAssocieeAVariable) valeurMiniAssociee.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable valeurMaxiClone = (CValeurAssocieeAVariable) valeurMaxiAssociee.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable valeurPasClone = (CValeurAssocieeAVariable) valeurPasAssociee.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable valeurActuelleClone = (CValeurAssocieeAVariable) valeurActuelleAssociee.getClone(listeSource, listeCible);
    int ind1 = listeSource.indexOf(variableAssociee);
    int ind2 = listeSource.indexOf(pointLie); 
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMacroModificationVariable(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, masque,
      (CPointAncetre)listeCible.get(ind2, CPointAncetre.class), indiceTaillePolice, 
      effacementFond, couleurFondAffichage, intitule, commentaireMacro, valeurMiniClone, valeurMaxiClone, 
      valeurPasClone, valeurActuelleClone, (CVariableBornee)(listeCible.get(ind1, CVariableBornee.class)),
      alignementHorizontal, alignementVertical);
    listeDep.initialise(listeCible.uniteAngle, listeCible.pointeurLongueurUnite);
    // On ne peut pas  ce niveau faire une image de la liste ListeDep dans la liste clone car tous
    // les lments n'ont pas encor t chargs.
    //((CMacroModificationVariable)ptelb).listeDep.setImage(listeDep, listeProprietaire,
    //  ptelb.listeProprietaire);
    return ptelb;
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(
    		super.dependDe(p) || variableAssociee.dependDe(p)
      || valeurMiniAssociee.dependDe(p)
      || valeurMaxiAssociee.dependDe(p)
      || valeurPasAssociee.dependDe(p)
      || valeurActuelleAssociee.dependDe(p));
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || variableAssociee.dependDePourReclassement(p)
      || valeurMiniAssociee.dependDePourReclassement(p)
      || valeurMaxiAssociee.dependDePourReclassement(p)
      || valeurPasAssociee.dependDePourReclassement(p)
      || valeurActuelleAssociee.dependDePourReclassement(p);
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return super.dependDePourBoucle(p) || variableAssociee.dependDePourBoucle(p)
      || valeurMiniAssociee.dependDePourBoucle(p)
      || valeurMaxiAssociee.dependDePourBoucle(p)
      || valeurPasAssociee.dependDePourBoucle(p)
      || valeurActuelleAssociee.dependDePourBoucle(p);
  }

  // Ajout pour version 3.0
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	super.positionne(infoRandom, dimfen);
  	valeurMiniAssociee.positionne(infoRandom, dimfen);
  	valeurMaxiAssociee.positionne(infoRandom, dimfen);
  	valeurPasAssociee.positionne(infoRandom, dimfen);
  	valeurActuelleAssociee.positionne(infoRandom, dimfen);
  	existe = existe && valeurMiniAssociee.existe() && valeurMaxiAssociee.existe()
  		&& valeurPasAssociee.existe() && valeurActuelleAssociee.existe();
  }
  
  @Override
  public boolean agitSurVariable(CVariableBornee var) {
    return (var == variableAssociee);
  }
  
  @Override
  public boolean affecteValeurAVariable(CValeurDynamique val) {
  	return valeurMiniAssociee.calcul.dependDe(val) || valeurMaxiAssociee.calcul.dependDe(val)
  		|| valeurPasAssociee.calcul.dependDe(val) || valeurActuelleAssociee.calcul.dependDe(val) ;
  }
  
  @Override
  public boolean peutEtreMacroFinBoucle() {
  	return true;
  }
  
  

  // Remarque : Dans l'applet cette fonction est diffrente et ne modifie pas
  // les chanes de calcul des valeurs mini, maxi et du pas
  // A revoir version 3.0
  @Override
  public void actionDansPaint(MtgInternalFrame cadre) {
  	// A revoir pour l'applet
    boolean infor = cadre.infRand;
    DimensionFenetre dimfen = cadre.dimf;
  	// Important : lignes suivante ncessaires pour la version 3.0 (changement de CValeurDynamique)
  	valeurMiniAssociee.positionne(infor, dimfen);
  	valeurMaxiAssociee.positionne(infor, dimfen);
  	valeurPasAssociee.positionne(infor, dimfen);
  	valeurActuelleAssociee.positionne(infor, dimfen);
  	// Modification version 2.0
    // listeDep.effaceTout(cadre.gbuffer, dimfen, true);
    double resultatValeurActuelle = valeurActuelleAssociee.rendValeur(infor);
    double nouvelleValeurMini = valeurMiniAssociee.rendValeur(infor);
    variableAssociee.valeurMini = nouvelleValeurMini;
    if (valeurMiniAssociee.estConstant) {
    	variableAssociee.chaineValeurMini = valeurMiniAssociee.chaineInfo();
    	}
    	else {
    		variableAssociee.chaineValeurMini = Kernel.chaineNombre(nouvelleValeurMini,12);
    }
    double nouvelleValeurMaxi = valeurMaxiAssociee.rendValeur(infor);
    variableAssociee.valeurMaxi = nouvelleValeurMaxi;
    if (valeurMaxiAssociee.estConstant) {
    	variableAssociee.chaineValeurMaxi = valeurMaxiAssociee.chaineInfo();
    	}
    	else {
    		variableAssociee.chaineValeurMaxi = Kernel.chaineNombre(nouvelleValeurMaxi,12);
    }
    double nouvelleValeurPas = valeurPasAssociee.rendValeur(infor);
    variableAssociee.valeurPas = nouvelleValeurPas;
    if (valeurPasAssociee.estConstant) {
    	variableAssociee.chaineValeurPas = valeurPasAssociee.chaineInfo();
    	}
    	else {
    		variableAssociee.chaineValeurPas = Kernel.chaineNombre(nouvelleValeurMini,12);
    }
    variableAssociee.donneValeur(resultatValeurActuelle);
    listeDep.positionne(infor, dimfen);
  	cadre.effacegbuffer();
    cadre.afficheToutDansgbuffer();
    cadre.metAJourPaneAffichageVariables(variableAssociee); // Correction oubli version 4.2.2
    executionEnCours = false;
  }

  @Override
  public void executeSimple(MtgInternalFrame cadre) {
  	// A revoir pour l'applet
    boolean infor = cadre.infRand;
    DimensionFenetre dimfen = cadre.dimf;
 	// Important : lignes suivante ncessaires pour la version 3.0 (changement de CValeurDynamique)
  	valeurMiniAssociee.positionne(infor, dimfen);
  	valeurMaxiAssociee.positionne(infor, dimfen);
  	valeurPasAssociee.positionne(infor, dimfen);
  	valeurActuelleAssociee.positionne(infor, dimfen);
    executionEnCours = false;
    double resultatValeurActuelle = valeurActuelleAssociee.rendValeur(infor);
    double nouvelleValeurMini = valeurMiniAssociee.rendValeur(infor);
    variableAssociee.valeurMini = nouvelleValeurMini;

    double nouvelleValeurMaxi = valeurMaxiAssociee.rendValeur(infor);
    variableAssociee.valeurMaxi = nouvelleValeurMaxi;
    double nouvelleValeurPas = valeurPasAssociee.rendValeur(infor);
    variableAssociee.valeurPas = nouvelleValeurPas;
    variableAssociee.donneValeur(resultatValeurActuelle);
    // Ligne suivante supprime version 2.4.0
    // listeDep.positionne(infor, dimfen);
  }
  
  /** Mthode charge de remettre la figure en tat initial
   * et n'existant pas dans l'applet. Charge de mettre  jour
   * le dialogue ventuel d'affichage des variables en bas  droite
   */
  public void termineAction(MtgInternalFrame cadre) {
    super.termineAction(cadre);
    // cadre.metAJourDialogueAffichageVariables(variableAssociee);
    cadre.metAJourPaneAffichageVariables(variableAssociee);
  }


  @Override
  public boolean executionPossible() {
    boolean resultat = valeurMiniAssociee.existe()
      && valeurMaxiAssociee.existe()
      && valeurPasAssociee.existe()
      && valeurActuelleAssociee.existe()
      && variableAssociee.existe;

    if (resultat)
    {
      boolean infr = false; // Pas de recalcul alatoire
      double nouvelleValeurMini = valeurMiniAssociee.rendValeur(infr);
      double nouvelleValeurMaxi = valeurMaxiAssociee.rendValeur(infr);
      double nouvelleValeurPas = valeurPasAssociee.rendValeur(infr);
      double nouvelleValeurActuelle = valeurActuelleAssociee.rendValeur(infr);
      resultat = 
        (nouvelleValeurMini < nouvelleValeurMaxi)
        && (nouvelleValeurPas < (nouvelleValeurMaxi - nouvelleValeurMini))
        && (nouvelleValeurActuelle >= nouvelleValeurMini)
        && (nouvelleValeurActuelle <= nouvelleValeurMaxi);
    }
    return resultat;
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo133") + " " + variableAssociee.nom();
  }

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
		return super.estDefiniParObjetDans(listeOb)
			&& valeurMiniAssociee.estDefiniParObjetDans(listeOb)
			&& valeurMaxiAssociee.estDefiniParObjetDans(listeOb)
			&& valeurPasAssociee.estDefiniParObjetDans(listeOb)
			&& valeurActuelleAssociee.estDefiniParObjetDans(listeOb)
			&& (listeOb.contains(variableAssociee) || variableAssociee.estDefiniParObjetDans(listeOb));
	}  
  
  @Override
  public boolean modifiableParMenu() {
  	// Modifi Version 3.5
  	// return true;
  	return ! estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new MacroModificationVariableDlg(frame, this, true);
  }

   // Ajout version 4.7.4
   @Override
   public void metAJour() {
   	etablitListesInternes();
   }

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    valeurMiniAssociee = new CValeurAssocieeAVariable();
    valeurMiniAssociee.chargeDepuisFlux(inps, list);
    valeurMaxiAssociee = new CValeurAssocieeAVariable();
    valeurMaxiAssociee.chargeDepuisFlux(inps, list);
    valeurPasAssociee = new CValeurAssocieeAVariable();
    valeurPasAssociee.chargeDepuisFlux(inps, list);
    valeurActuelleAssociee = new CValeurAssocieeAVariable();
    valeurActuelleAssociee.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    variableAssociee = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    // Il faut crer la liste des lments dpendant de la variable modifie
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la lsite mre
    listeDep = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    valeurMiniAssociee = new CValeurAssocieeAVariable();
    valeurMiniAssociee.read(inps, list);
    valeurMaxiAssociee = new CValeurAssocieeAVariable();
    valeurMaxiAssociee.read(inps, list);
    valeurPasAssociee = new CValeurAssocieeAVariable();
    valeurPasAssociee.read(inps, list);
    valeurActuelleAssociee = new CValeurAssocieeAVariable();
    valeurActuelleAssociee.read(inps, list);
    int ind1 = inps.readInt();
    variableAssociee = (CVariableBornee) (list.get(ind1,CVariableBornee.class));
    // Il faut crer la liste des lments dpendant de la variable modifie
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la lsite mre
    listeDep = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    valeurMiniAssociee.write(oups, list);
    valeurMaxiAssociee.write(oups, list);
    valeurPasAssociee.write(oups, list);
    valeurActuelleAssociee.write(oups, list);
    int ind1 = list.indexOf(variableAssociee);
    oups.writeInt(ind1);
  }
  
  @Override
  public void etablitListesInternes() {
    listeDep = new CListeObjets(listeProprietaire.uniteAngle, listeProprietaire.pointeurLongueurUnite);        
    listeDep.ajouteObjetsDependantsSauf(variableAssociee, listeProprietaire, this);
  }
  
  // Suprim version 2.0 car provoques des stackoverflow
  /*
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
  	listeDep.remplacePoint(ancienPoint, nouveauPoint);
  }
  */
  
}
