/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.*;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

import mathgraph32.MtgFrame;


public class CMesureAngleGeometrique extends CValeurDynamique {

	CPointAncetre point1;
	CPointAncetre point2;
	CPointAncetre point3;
	double mesure;
  
  /** Creates new CMesureAngleGeometrique */
  public CMesureAngleGeometrique() {
  }
  
  // Constructeur pour clonage
  public CMesureAngleGeometrique(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CPointAncetre point1Init, 
      CPointAncetre point2Init, CPointAncetre point3Init) {
    super(liste, impProtoInit, estElementFinalInit);
    point1 = point1Init;
    point2 = point2Init;
    point3 = point3Init;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(point1);
    int ind2 = listeSource.indexOf(point2);   
    int ind3 = listeSource.indexOf(point3);   
    int ind4 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMesureAngleGeometrique(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)), (CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)));
    return ptelb;    
  }

  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMesureAngleNonOriente;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch =  getString("chinfo29") + " " + nom();
    if (existe) ch = ch + "\n" + getString("chinfo97") + " " + Kernel.chaineNombre(mesure, 16);
    else ch = ch + "\n" + getString("ch18");
    return ch;        
  }
  
  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(point1);
    liste.add(point2);
    liste.add(point3);
  } 
  
  @Override
  public void setClone(CElementBase ptel) {
    mesure = ((CMesureAngleGeometrique)ptel).mesure;
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || point1.dependDe(p) || point2.dependDe(p) || point3.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || point1.dependDePourReclassement(p)
    	|| point2.dependDePourReclassement(p) || point3.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || point1.dependDePourBoucle(p) || point2.dependDePourBoucle(p)
      || point3.dependDePourBoucle(p);
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double prosca,deter,tangente,resul;

    existe = point1.existe && point2.existe && point3.existe
			&& !(point2.egal(point1)) && !(point2.egal(point3));
    if (!existe) return;
    // L'orientation sur l'cran est l'inverse de celle du
		// repre choisi.Il faut donc permuter les rles de u et v}
    VecteurDirecteur v = new VecteurDirecteur(point2,point1);
    VecteurDirecteur u = new VecteurDirecteur(point2,point3);
    prosca = u.x*v.x + u.y*v.y;
    deter = Math.abs(u.x*v.y - u.y*v.x);
    if (prosca == 0) resul = Math.PI/2;
    else
    {
      tangente = deter/prosca;
      if (prosca > 0)
				resul = Math.atan(tangente);
      else
      {
        resul = Math.PI - Math.atan(-tangente);
        if (resul > Math.PI) resul = resul - 2*Math.PI;
      }
      resul = Math.abs(resul);
    }
    if (listeProprietaire.uniteAngle == UniteAngle.Radian) mesure = resul;
    else mesure = resul * Kernel.ConvRadDeg;
  }

  @Override
  public boolean nomIndispensable(CElementGraphique el) {
    return ((el == point1) || (el == point2) || (el == point3));
  }
  
  @Override
  public double rendValeur(boolean infoRandom) {
    return mesure;
  }


  // Ajout pour la version 2 du CNED
  public boolean chaineCommenceParNom(String pChaine,Point longueurNom){
    // Pour une longueur AB par exemple, le nom BA est quivalent
    if (super.chaineCommenceParNom(pChaine,longueurNom)) {
      return true;
    }
    else
    {
      String chaineNom = point3.nom + point2.nom + point1.nom;
      longueurNom.x = chaineNom.length();
      int longueurChaine2 = pChaine.length();
      if (longueurChaine2 < longueurNom.x) return false;
      else
      {
        return chaineNom.equals(pChaine.substring(0,  longueurNom.x));
      }
    }
  }
  // Ajout spcifique version avec construction permettant de crer des calculs
  /**
   * Renverra true si la chaine ch est gale (sans tenir compte de la casse)
   * soit au nom de la valeur dans le cas d'un calcul ou une fonction soit 
   * la chane reprsentant la mesure dans le cas d'une mesure
   * Modifi version 3.5.0 : on diffrencie majuscules et minuscules
   * @param ch
   * @return
   */
  public boolean chaineEgaleANom(String ch) {
    if (super.chaineEgaleANom(ch)) return true;
    String ch2 = point3.rendNom() + point2.rendNom() + point1.rendNom();
    return (ch.equals(ch2));
  }
  
  @Override
  public String nom() {
    return point1.nom + point2.nom + point3.nom;
  }
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  public boolean confonduAvec(CElementBase p) {
    CMesureAngleGeometrique pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMesureAngleGeometrique) p;
      return ((point2 == pt.point2) && 
			(((point1 == pt.point1) && (point3 == pt.point3))
			||
      ((point1 == pt.point3) && (point3 == pt.point1))));
    }
    else return false;
  }    

  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (point1 == ancienPoint) point1 = nouveauPoint;
    if (point2 == ancienPoint) point2 = nouveauPoint;
    if (point3 == ancienPoint) point3 = nouveauPoint;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(point1) || point1.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(point2) || point2.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(point3) || point3.estDefiniParObjetDans(listeOb));
	}  
    
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     int ind2 = inps.readCInt();
     int ind3 = inps.readCInt();
     point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
     point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
     point3 = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    int ind3 = inps.readInt();
    point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    point3 = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(point1);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(point2);
    oups.writeInt(ind2);
    int ind3 = list.indexOf(point3);
    oups.writeInt(ind3);
  } 
}
