/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

// Java : Diffrence par rapport au C++ : Cette classe hrite de CMesureX pour gagner de la place
// abscisse joue lengthrel de l'ordonne
import glob.Kernel;
import types.*;

public class CMesureY extends CMesureX {

  /** Creates new CMesureY */
  public CMesureY() {
  }

  // Constructeur pour clonage
  public CMesureY(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CRepere repereAssocieInit, CPointAncetre pointMesureInit) {
    super(liste, impProtoInit, estElementFinalInit, repereAssocieInit, pointMesureInit);
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(repereAssocie);
    int ind2 = listeSource.indexOf(pointMesure);  
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMesureY(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, (CRepere)(listeCible.get(ind1, CRepere.class)), (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)));
    return ptelb;    
  } 
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMesureOrdonneeRepere;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch =  getString("chinfo81") + " " + pointMesure.rendNom() + "\n" + getString("chinfo32") + " "
    + repereAssocie.nom() + "\n"
    + getString("chinfo33") + " " + nom();
    if (existe) ch = ch + "\n" + getString("chinfo97") + Kernel.chaineNombre(abscisse, 16);
    else ch = ch + "\n" + getString("ch18");
    return ch;
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = repereAssocie.existe && pointMesure.existe;
    if (!existe) return;
    // L'orientation sur l'cran est l'inverse de celle du repre
		// choisi.Il faut donc permuter les rles de u et v}
    VecteurDirecteur u = new VecteurDirecteur(repereAssocie.O, repereAssocie.I);
    VecteurDirecteur v = new VecteurDirecteur(repereAssocie.O, repereAssocie.J);
		// Calcul des	abscisses et ordonnes de l'origine relle du repre dans le repre de base
    double absOrigine = repereAssocie.valAbscisseOrigine/repereAssocie.unitex;
    double ordOrigine = repereAssocie.valOrdonneeOrigine/repereAssocie.unitey;
		double xno = repereAssocie.O.x -u.x*absOrigine - v.x*ordOrigine;
		double yno = repereAssocie.O.y - u.y*absOrigine - v.y*ordOrigine;
		// Calcul de l'abscisse par rsolution d'un systme linaire
		double det = u.x*v.y-v.x*u.y;
		abscisse = (u.x*(pointMesure.y-yno)-u.y*(pointMesure.x-xno))/det*repereAssocie.unitey;
  }
  
  @Override
  public String nom() {
    return "yCoord" + "(" + pointMesure.nom + "," + repereAssocie.O.nom + "," +
      repereAssocie.I.nom + "," +repereAssocie.J.nom + ")";
  }    
}
