/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/** Classe anctre de tous les objets qui devront tre chargs depuis un flux
 * Grosse nouveaut pour la version Java : chaque objet, y compris les objets de type calcul
 * aura un membre listePropritaire
 * qui contiendra une rfrence  la liste principale qui contient l'objet.
 * Ce membre sera tabli lors de l'appel  ChargeDepuisFlux
 * */
public abstract class CObjetBase extends java.lang.Object {
  // Pointeur sur la liste possdant l'objet
  public CListeObjets listeProprietaire;
  // Numro de version de l'objet. 1 par dfaut.
  // A redfinir pour les objets ayant eu un numro de version redfini en C++ ou java
  public int numeroVersion() {
  	return 1;
  }
  // A partir de la version 1.8, numro de version gr
  public int nVersion;

  /** Creates new CObjetBase */
  public CObjetBase() {
  	nVersion = numeroVersion();
  }
  
  public CObjetBase(CListeObjets liste){
    listeProprietaire = liste;
  	nVersion = numeroVersion();
  }

  /**
   * Chargement depuis un flux C++
   * @param inps
   * @param list
   * @throws IOException
   */
  public void chargeDepuisFlux ( MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    listeProprietaire = list;
  }
  
  public void chargeDepuisFluxFromURL ( MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    chargeDepuisFlux(inps, list);
  }
  

  /**
   * Chargement depuis un flux java
   * @param inps
   * @param list
   * @throws IOException
   */
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    listeProprietaire = list;
  }
  
	/**
	 * Fonction spcial version applet.
	 * Certains objets devront tre lus diffremment suivant qu'on ouvre un fichier ou qu'on
	 * charge un fichier depuis une adresse web.
	 * En l'occurence ce sera le cas pour CMacroJouantSon
	 * @param inps
	 * @param list
	 * @throws IOException
	 */
	public void readFromURL(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
		read(inps, list);
	}
  
  
  /**
   * Enregistrement dans un flux java
   * @param oups
   * @param list
   * @throws IOException
   */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
  }
    

}
