/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.Color;
import java.io.IOException;

import mathgraph32.MtgFrame;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.NatureObjet;

/**
 * Classe anctre des points lis  un lieu de points.
 * Il y aura deux types de descendants : ceux lis  un lieu de points gnr par un point li
 * et ceux lis  un lieu de points gnr par une variable
 * @author Yves
 *
 */
public abstract class CPointLieLieuAncetre extends CPointLie {
	protected CLieuDeBase lieuAssocie;
	/*
	 * Signification de abscisse pour cet objet
	 * Dans le cas o le point est li  un lieu de points gnr par une variable, la valeur de la variable donnant la position du point
	 * Dans le cas o le point est li  un lieu de points gnr par un point li, l'abscisse de ce point li relative
	 *  l'objet auquel il est li et qui donne la position du point.
	 */
	
	// Constructeur par dfaut
	protected CPointLieLieuAncetre() {
	}
	
	// Constructeur pour clonage
  protected CPointLieLieuAncetre(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit, boolean pointLibreInit, double abscisseInit, CLieuDeBase lieuAssocieInit, boolean MarquePourTraceInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, dec_xInit, dec_yInit, 
          masqueInit, nomInit, indiceTailleNomInit, motifInit, pointLibreInit, abscisseInit, MarquePourTraceInit);
        lieuAssocie = lieuAssocieInit;
    }
	
  // Supprim version 4.7.7. Inutile car dj dfini dans CPointLie
  /*
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    abscisse = ((CPointLie)ptel).abscisse;
  }
  */
    
  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(lieuAssocie);
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo156");
  }
    
  
  @Override
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || lieuAssocie.dependDe(p);
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || lieuAssocie.dependDePourReclassement(p);
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || lieuAssocie.dependDePourBoucle(p);
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CPointLieLieuAncetre pt;
    if (p.getClass() == this.getClass()) {
      pt = (CPointLieLieuAncetre) p;
      return glob.Kernel.zero(abscisse - pt.abscisse);
    }
    else return false;
  }  
  
  @Override
  public long getNature() {
    return NatureObjet.NPointLie;
  }
  
  @Override
	public
  CElementBase lieA() {
    return lieuAssocie;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return listeOb.contains(lieuAssocie) || lieuAssocie.estDefiniParObjetDans(listeOb);
	}  
  
  
  /* Chargement depuis un flux java */
  public void read( MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    lieuAssocie = (CLieuDeBase) (list.get(ind1,CLieuDeBase.class));
    abscisse = inps.readDouble();
  }
  
  /* Enregistrement dans un flux java */
  public void write( MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(lieuAssocie);
    oups.writeInt(ind1);
    oups.writeDouble(abscisse);
  }    
   
}
