/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.geom.*;
import java.awt.*;
import java.io.*;


import mathgraph32.MtgFrame;
import dialogs.MtgDlg;
import dialogs.RepereDlg;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;


public class CRepere extends CElementLigne {
  private double absMiniH,absMaxiH;
  private double absMiniV,absMaxiV;
  
  public CPointAncetre O;
  public CPointAncetre I;
  public CPointAncetre J;
  public VecteurDirecteur u,v;
  Point2D.Double origine;
  public CValeurAssocieeAVariable abscisseOrigine;
  public CValeurAssocieeAVariable ordonneeOrigine;
  // public CDroite axeAbscisses; Abandonn pour la version java
  // public CDroite axeOrdonnees; Abandonn pour la version java
  public boolean quadrillageHorizontal;
  public boolean quadrillageVertical;
  public boolean pointilles;
  //Nouveaut version 4.0 : un repre a deux membres donnant la longueur unit sur chaque axe
  public CValeurAssocieeAVariable unite_x;
  public CValeurAssocieeAVariable unite_y;
  // Les deux valeurs que prennent les deux membres prcdents
  public double unitex;
  public double unitey;
  // Nouveau version 4.0 : on mmorise les valeurs de abscisseOrigine et ordonneeOrigine
  public double valAbscisseOrigine;
  public double valOrdonneeOrigine;
  // Pour le positionnement
  private Point2D.Double[] pointint = new Point2D.Double[4];
  
  // Java : diffrence par rapport au C++
  // La droite servant  tracer les quadrillages est cre  la cration de l'objet CRepere
  // et le servant  tracer des pointills et ce point est du type CPointBase
  // Autre grosse diffrence : la mthode de tracage a besoin de connatre les dimensions de la
  // fentre pour le trac des quadrillages. La variable membre dimensionsFenetre mmorisera donc
  // cette valeur dans positionne() ainsi que celle de infoRandom dans infoSurRandom
  private boolean infoSurRandom;
  private DimensionFenetre dimensionFenetre;
  
  
  CDroite droiteQuadrillage;
  CPointBase pointPointilles;
  
  
  public CRepere() {
  }
  
  // Constructeur pour clonage
  public CRepere(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit, CPointAncetre OInit, CPointAncetre IInit, CPointAncetre JInit,
    CValeurAssocieeAVariable abscisseOrigineInit, CValeurAssocieeAVariable ordonneeOrigineInit,
    boolean quadrillageHorizontalInit, boolean quadrillageVerticalInit, boolean pointillesInit,
    CValeurAssocieeAVariable unite_xInit, CValeurAssocieeAVariable unite_yInit) {
    super(liste, impProtoInit, estElementFinalInit,  couleurInit, masqueInit, styleInit);
    u = new VecteurDirecteur();
    v = new VecteurDirecteur();
    droiteQuadrillage = new CDroite(liste);	// On donne une liste propritaire pour que la droite soit adapte
    																				//  la haure ou basse rsolution
    droiteQuadrillage.montre();
    pointPointilles = new CPointBase(liste);
    pointPointilles.donneMotif(MotifPoint.pixel);
    pointPointilles.montre();
    origine = new Point2D.Double();
    O = OInit;
    I = IInit;
    J = JInit;
    abscisseOrigine = abscisseOrigineInit;
    ordonneeOrigine = ordonneeOrigineInit;
    quadrillageHorizontal = quadrillageHorizontalInit;
    quadrillageVertical = quadrillageVerticalInit;
    pointilles = pointillesInit;
    unite_x = unite_xInit;
    unite_y = unite_yInit;
    // Crer le tableau ne suffit pas, il faut instancier chaque membre
    for (int i = 0; i < 4; i++) {
      pointint[i] = new Point2D.Double();
    }
  }
  
  // Pour la version 4.0, introduction d'une longueur unit sur chaque axe
  // d'o changement de version
  @Override
  public int numeroVersion() {
  	return 2;
  }  
  
  
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    absMiniH = ((CRepere)ptel).absMiniH;
    absMaxiH = ((CRepere)ptel).absMaxiH;
    absMiniV = ((CRepere)ptel).absMiniV;
    absMaxiV = ((CRepere)ptel).absMaxiV;
    infoSurRandom = ((CRepere)ptel).infoSurRandom;
    dimensionFenetre = ((CRepere)ptel).dimensionFenetre;
    origine.setLocation(((CRepere)ptel).origine);
    ((CRepere)ptel).u.setCopy(u);
    ((CRepere)ptel).v.setCopy(v);
    unitex = ((CRepere)ptel).unitex;
    unitey = ((CRepere)ptel).unitey;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(O);
    int ind2 = listeSource.indexOf(I);
    int ind3 = listeSource.indexOf(J);
    int ind4 = listeSource.indexOf(impProto);
    CValeurAssocieeAVariable abscisseOrigineClone =
      (CValeurAssocieeAVariable) abscisseOrigine.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable ordonneeOrigineClone =
      (CValeurAssocieeAVariable) ordonneeOrigine.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable unite_xClone =
      (CValeurAssocieeAVariable) unite_x.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable unite_yClone =
      (CValeurAssocieeAVariable) unite_y.getClone(listeSource, listeCible);
    
    CElementBase ptelb = new CRepere(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, masque, style,
      (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)),
      abscisseOrigineClone, ordonneeOrigineClone, quadrillageVertical,
      quadrillageHorizontal, pointilles, unite_xClone, unite_yClone);
    return ptelb;
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p)
    || O.dependDe(p)
    || I.dependDe(p)
    || J.dependDe(p)
    || abscisseOrigine.dependDe(p)
    || ordonneeOrigine.dependDe(p)
    || unite_x.dependDe(p)
    || unite_y.dependDe(p));
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p)
    || O.dependDePourReclassement(p)
    || I.dependDePourReclassement(p)
    || J.dependDePourReclassement(p)
    || abscisseOrigine.dependDePourReclassement(p)
    || ordonneeOrigine.dependDePourReclassement(p)
    || unite_x.dependDePourReclassement(p)
    || unite_y.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this)
    || O.dependDePourBoucle(p)
    || I.dependDePourBoucle(p)
    || J.dependDePourBoucle(p)
    || abscisseOrigine.dependDePourBoucle(p)
    || ordonneeOrigine.dependDePourBoucle(p)
    || unite_x.dependDePourBoucle(p)
    || unite_y.dependDePourBoucle(p);
  }
  
  @Override
  public boolean nomIndispensable(CElementGraphique el) {
    return ((el == O) || (el == I) || (el == J));
  }
  
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    // Ajout version 4.0 : unit sur l'axe des abscisses et l'axe des ordonnes  positionner
  	unite_x.positionne(infoRandom, dimfen);
    unitex = unite_x.rendValeur(infoRandom);
  	unite_y.positionne(infoRandom, dimfen);
    unitey = unite_y.rendValeur(infoRandom);

    abscisseOrigine.positionne(infoRandom, dimfen);
    ordonneeOrigine.positionne(infoRandom, dimfen);
    existe = (O.existe) && (I.existe) && (J.existe)
    	&& (abscisseOrigine.existe())
    	&& (ordonneeOrigine.existe())
    	&& unite_x.existe()
    	&& unite_y.existe()
    	&& (unitex > 0)
    	&& (unitey > 0)
    	&& !(Kernel.aligne(O.x,O.y,I.x,I.y,J.x,J.y));
    if (existe) {
      // On mmorise les valeurs de l'abscisse  l'origine et de l'ordonne  l'origine.
      valAbscisseOrigine = abscisseOrigine.rendValeur(infoRandom);
      valOrdonneeOrigine = ordonneeOrigine.rendValeur(infoRandom);
      double abs[] = new double[4];
      Point pointIndices = new Point(); // Contiendra les indices des valeurs minimalse et maximales
      // Java : diffrence par rapport au C++. Ncessaire de mmoriser infoRandom et dimfen
      // pour lengthtrac de la droite servant au quadrillage
      infoSurRandom = infoRandom;
      dimensionFenetre = dimfen;
      u.setVecteur(O,I);
      v.setVecteur(O,J);
      // On calcule les relles coordonnes de l'origine
      origine.x = O.x - u.x*valAbscisseOrigine/unitex-v.x*valOrdonneeOrigine/unitey;
      origine.y = O.y - u.y*valAbscisseOrigine/unitex-v.y*valOrdonneeOrigine/unitey;
      // Prparation du quadrillage vertical ventuel;
      if (quadrillageVertical || pointilles) {
        Kernel.intersectionDroitesSecantes(0,0,u,O.x,O.y,v,pointint[0]);
        Kernel.intersectionDroitesSecantes
          (0,dimfen.longueurOrdonnees,u,O.x,O.y,v,pointint[1]);
        Kernel.intersectionDroitesSecantes
          (dimfen.longueurAbscisses,0,u,O.x,O.y,v,pointint[2]);
        Kernel.intersectionDroitesSecantes
          (dimfen.longueurAbscisses,dimfen.longueurOrdonnees,u,O.x,O.y,v,pointint[3]);
        if (v.y != 0) {
          abs[0] = (pointint[0].y-O.y)/v.y;
          abs[1] = (pointint[1].y-O.y)/v.y;
          abs[2] = (pointint[2].y-O.y)/v.y;
          abs[3] = (pointint[3].y-O.y)/v.y;
        } else {
          abs[0] = (pointint[0].x-O.x)/v.x;
          abs[1] = (pointint[1].x-O.x)/v.x;
          abs[2] = (pointint[2].x-O.x)/v.x;
          abs[3] = (pointint[3].x-O.x)/v.x;
        };
        Kernel.indiceMiniMaxi((byte)3,abs,pointIndices);
        absMiniV = Math.floor(abs[pointIndices.x]+1);
        absMaxiV = Math.floor(abs[pointIndices.y]);
        
      }
      if (quadrillageHorizontal || pointilles) {
        Kernel.intersectionDroitesSecantes(0,0,v,O.x,O.y,u,pointint[0]);
        Kernel.intersectionDroitesSecantes
          (0,dimfen.longueurOrdonnees,v,O.x,O.y,u,pointint[1]);
        Kernel.intersectionDroitesSecantes
          (dimfen.longueurAbscisses,0,v,O.x,O.y,u,pointint[2]);
        Kernel.intersectionDroitesSecantes
          (dimfen.longueurAbscisses,dimfen.longueurOrdonnees,v,O.x,O.y,u,pointint[3]);
        if (u.x != 0) {
          abs[0] = (pointint[0].x - O.x)/u.x;
          abs[1] = (pointint[1].x - O.x)/u.x;
          abs[2] = (pointint[2].x - O.x)/u.x;
          abs[3] = (pointint[3].x - O.x)/u.x;
        } else {
          abs[0] = (pointint[0].y - O.y)/u.y;
          abs[1] = (pointint[1].y - O.y)/u.y;
          abs[2] = (pointint[2].y - O.y)/u.y;
          abs[3] = (pointint[3].y - O.y)/u.y;
        }
        Kernel.indiceMiniMaxi((byte)3,abs,pointIndices);
        absMiniH = Math.floor(abs[pointIndices.x]+1);
        absMaxiH = Math.floor(abs[pointIndices.y]);
      }
    }
  }

  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    
    if (quadrillageVertical) {
      droiteQuadrillage.donneCouleur(couleur);
      droiteQuadrillage.donneStyle(style); // Contrairement  la version java les repres ont un style
      if (absMaxiV - absMiniV <= 80) {
        droiteQuadrillage.vecteur.x = u.x;
        droiteQuadrillage.vecteur.y = u.y;
        for (double p = absMiniV;p <= absMaxiV; p++) {
          if (p != 0) {
            droiteQuadrillage.point_x = O.x + p*v.x;
            droiteQuadrillage.point_y = O.y + p*v.y;
            // Les paramtres d'appel ne servent pas.
            droiteQuadrillage.positionne(infoSurRandom, dimensionFenetre);
            droiteQuadrillage.trace(pDC, couleurFond, transparence);
          }
        }
      }
    }
    if (quadrillageHorizontal) {
      droiteQuadrillage.donneCouleur(couleur);
      droiteQuadrillage.donneStyle(style);
      
      if (absMaxiH - absMiniH <= 80) {
        droiteQuadrillage.vecteur.x = v.x;
        droiteQuadrillage.vecteur.y = v.y;
        for (double p = absMiniH;p <= absMaxiH; p++) {
          if (p != 0) {
            droiteQuadrillage.point_x = O.x + p*u.x;
            droiteQuadrillage.point_y = O.y + p*u.y;
            droiteQuadrillage.positionne(infoSurRandom, dimensionFenetre);
            droiteQuadrillage.trace(pDC, couleurFond, transparence);
          }
        }
      }
    }
    if (pointilles) {
      pointPointilles.donneCouleur(couleur);
      
      if ((absMaxiH - absMiniH <= 80)
      		&& (absMaxiV - absMiniV <= 80)) {
        for (double p = absMiniH;p <= absMaxiH; p++) {
          for (double q = absMiniV;q <= absMaxiV; q++) {
            double x1 = O.x + p*u.x +q*v.x;
            double y1 = O.y + p*u.y + q*v.y;
            pointPointilles.placeEn(x1,y1);
            pointPointilles.trace(pDC, couleurFond, transparence);
          }
        }
      }
    }
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CRepere pt;
    if (p.getClass() == this.getClass()) {
      pt = (CRepere) p;
   		return (O == pt.O) && (I == pt.I) && (J == pt.J);
    }
    else return false;
  }        
    
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (O == ancienPoint) O = nouveauPoint;
    if (I == ancienPoint) I = nouveauPoint;
    if (J == ancienPoint) J = nouveauPoint;
  }

  // Attention : Dans la version java, le repre est le seul objet  la fois graphique et non graphique
  // Il est considr comme non graphique car dans les constructions il est choisi en premier dans
  // les objets non graphiques  
  @Override
  public long getNatureCalcul() {
    return NatureCalcul.NRepere;
  }
  @Override
  public long getNature() {
  	return NatureObjet.NRepere;
  }
    
  @Override
  public String infoPourHistorique() {
    String ch = getString("chinfo103") + " (" + O.rendNom() +", " + I.rendNom() + ", " + J.rendNom() +")" + "\n" +
    	getString("chinfo104") + " " + abscisseOrigine.chaineInfo() + "\n" + 
    	getString("chinfo105") + " " + ordonneeOrigine.chaineInfo() + "\n" +
    	getString("repereDlg10") + " " + unite_x.chaineInfo() + "\n" +
    	getString("repereDlg11") + " " + unite_y.chaineInfo();
    if (!existe) ch = ch + "\n" + getString("ch18");
    return ch;
  }
  
  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(O);
    liste.add(I);
    liste.add(J);
  } 
  
  
  @Override
  public String chaineDesignation() {
    return getString("designationRepere");
  }
  
  @Override
  public String nom() {
  	return "(" + O.rendNom() + "," + I.rendNom() + "," + J.rendNom() + ")";
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(O) || O.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(I) || I.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(J) || J.estDefiniParObjetDans(listeOb));
	}  
  
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new RepereDlg(frame, this, true);
  }
  
  @Override
  public String informationModification() {
  	return getString("chaineOutil78");
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    // A cause d'une erreur de conception de la version C++ qui appelait la srialization
    // de CElementGraphique alors que l'objet drive de CElementLigne.
    // Il a donc fallu que je rajoute un ancetre de tous les CElementLigne !
    super.chargeDepuisFluxSansStyleTrait(inps, list);
  	abscisseOrigine = new CValeurAssocieeAVariable(listeProprietaire, 0);
  	ordonneeOrigine = new CValeurAssocieeAVariable(listeProprietaire, 0);
    u = new VecteurDirecteur();
    v = new VecteurDirecteur();
    droiteQuadrillage = new CDroite(list);
    droiteQuadrillage.donneStyle(StyleTrait.traitPointille);
    droiteQuadrillage.montre();
    pointPointilles = new CPointBase(list);
    pointPointilles.donneMotif(MotifPoint.pixel);
    pointPointilles.montre();
    origine = new Point2D.Double();
    for (int i = 0; i < 4; i++) {
      pointint[i] = new Point2D.Double();
    }
    
    style = StyleTrait.traitPointille;
    if (this.nVersion == 1) { // Spcifique vieille version
			@SuppressWarnings("unused")
			byte b1 = inps.readByte();
    }
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    int ind3 = inps.readCInt();
    @SuppressWarnings("unused")
		int ind4 = inps.readCInt(); // Servait  charger l'axe des abscisses en C++. Abandonn
    @SuppressWarnings("unused")
		int ind5 = inps.readCInt(); // Servait  charger l'axe des ordonnes en C++. Abandonn
    O = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    I = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    J = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
    quadrillageVertical = inps.readBoolean();
    quadrillageHorizontal = inps.readBoolean();
    pointilles = inps.readBoolean();
    abscisseOrigine = new CValeurAssocieeAVariable(list, 0);
    ordonneeOrigine = new CValeurAssocieeAVariable(list, 0);
    // La version 1 ne permettait pas de dcaler l'origine du repre
    if (nVersion != 1) {
    	abscisseOrigine.chargeDepuisFlux(inps, list);
    	ordonneeOrigine.chargeDepuisFlux(inps, list);
    }
  	unite_x = new CValeurAssocieeAVariable(list,1);
  	unite_y = new CValeurAssocieeAVariable(list,1);
    nVersion = numeroVersion(); // On remet tout  zro pour la version java
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
  	abscisseOrigine = new CValeurAssocieeAVariable(listeProprietaire, 0);
  	ordonneeOrigine = new CValeurAssocieeAVariable(listeProprietaire, 0);
    u = new VecteurDirecteur();
    v = new VecteurDirecteur();
    droiteQuadrillage = new CDroite(list);
    droiteQuadrillage.donneStyle(StyleTrait.traitPointille);
    droiteQuadrillage.montre();
    pointPointilles = new CPointBase(list);
    pointPointilles.donneMotif(MotifPoint.pixel);
    pointPointilles.montre();
    origine = new Point2D.Double();
    // Crer le tableau ne suffit pas, il faut instancier chaque membre
    for (int i = 0; i < 4; i++) {
      pointint[i] = new Point2D.Double();
    }
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    int ind3 = inps.readInt();
    O = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    I = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    J = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
    quadrillageVertical = inps.readBoolean();
    quadrillageHorizontal = inps.readBoolean();
    pointilles = inps.readBoolean();
    abscisseOrigine = new CValeurAssocieeAVariable();
    abscisseOrigine.read(inps, list);
    ordonneeOrigine = new CValeurAssocieeAVariable();
    ordonneeOrigine.read(inps, list);
    // Ajout version 3.9.5 : unit sur l'axe des abscisses et l'axe des ordonnes
    if (nVersion >= 2) {
    	unite_x = new CValeurAssocieeAVariable();
	    unite_x.read(inps, list);
	    unite_y = new CValeurAssocieeAVariable();
	    unite_y.read(inps, list);
    }
    else {
    	unite_x = new CValeurAssocieeAVariable(list,1);
    	unite_y = new CValeurAssocieeAVariable(list,1);
    }
  }
  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    // Crer le tableau ne suffit pas, il faut instancier chaque membre
    for (int i = 0; i < 4; i++) {
      pointint[i] = new Point2D.Double();
    }
    int ind1 = list.indexOf(O);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(I);
    oups.writeInt(ind2);
    int ind3 = list.indexOf(J);
    oups.writeInt(ind3);
    oups.writeBoolean(quadrillageVertical);
    oups.writeBoolean(quadrillageHorizontal);
    oups.writeBoolean(pointilles);
    abscisseOrigine.write(oups, list);
    ordonneeOrigine.write(oups, list);
    // Ajout version 4.0 : unit sur l'axe des abscisses et l'axe des ordonnes
    unite_x.write(oups, list);
    unite_y.write(oups, list);
  }
}
