/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.*;
import java.awt.geom.*;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

import mathgraph32.MtgFrame;


public class CSurfaceLieuDroite extends CSurface {
	CDroite droiteAssociee;
	int[] abscisses;
  int[] ordonnees;
  CPointBase pointDebut;
	CPointBase pointProjeteDebut;
	CPointBase pointFin;
	CPointBase pointProjeteFin;
	CSegment segmentDebut;
	CSegment segmentFin;
	CSegment segmentDroite;

  // Pour le trac
  private int nbPointsLigne;
  // Pour le positionnement
  private Point2D.Double point2 = new Point2D.Double();
  private Point2D.Double point3 = new Point2D.Double();
  
  /** Creates new CSurfaceLieuDroite */
  public CSurfaceLieuDroite() {
  }
  
  // Constructeur pour clonage
  public CSurfaceLieuDroite(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit,
      int styleRemplissageInit, CElementGraphique bordInit, CDroite droiteAssocieeInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleRemplissageInit, bordInit);
    droiteAssociee = droiteAssocieeInit;
    pointDebut = new CPointBase();
    pointProjeteDebut = new CPointBase();
    pointFin = new CPointBase();
    pointProjeteFin = new CPointBase();
    segmentDebut = new CSegment(liste, pointDebut, pointProjeteDebut);
    segmentFin = new CSegment(liste, pointFin, pointProjeteFin);
    segmentDroite = new CSegment(liste, pointProjeteDebut, pointProjeteFin);
    // On cre les tableaux devant recevoir les coordonnes. Le + 3 est du aux deux points projets
    // sur la droite
    metAJourTableaux();
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(bord);
    int ind2 = listeSource.indexOf(droiteAssociee);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CSurfaceLieuDroite(listeCible, (CImplementationProto)listeCible.get(ind3, CImplementationProto.class),
      estElementFinal, couleur, masque, styleRemplissage, (CLieuDeBase)listeCible.get(ind1, CLieuDeBase.class),
      (CDroite)listeCible.get(ind2, CDroite.class));
    return ptelb;
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    if (ptel.existe) {
	    nbPointsLigne = ((CSurfaceLieuDroite)ptel).nbPointsLigne;
	    int nbPointsADecaler = nbPointsLigne + 3;
	    System.arraycopy(((CSurfaceLieuDroite)ptel).abscisses, 0, abscisses, 0, nbPointsADecaler);
	    System.arraycopy(((CSurfaceLieuDroite)ptel).ordonnees, 0, ordonnees, 0, nbPointsADecaler);
    }
  }
   
  /**
   * Fonction appele quand on change le nombre de points du lieu qui est le bord de la surface
   */
  @Override
  public void metAJourTableaux() {
  	// + 4 et non +3 car si le lieu est chang en lieu ferm, un point de plus sera rajout au lieu
    abscisses = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 4];
    ordonnees = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 4];  	
  }
  
  @Override
  public boolean aPourBord(CLieuDeBase lieu) {
  	return (lieu == bord);
  }   

  @Override
  public String infoPourHistorique() {
    return getString("chinfo64");
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(bord);
    liste.add(droiteAssociee);
  } 
  
  @Override
  public boolean confonduAvec(CElementBase p) {
  	boolean res = super.confonduAvec(p);
   	if (!res) return false;
  	CSurfaceLieuDroite pt = (CSurfaceLieuDroite) p;
  	return droiteAssociee == pt.droiteAssociee;
  }
  
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || droiteAssociee.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || droiteAssociee.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return super.dependDePourBoucle(p) || droiteAssociee.dependDePourBoucle(p);
  }

  @Override
  public boolean dependDePourCapture(CElementBase p) {
    return super.dependDePourCapture(p) || droiteAssociee.dependDePourCapture(p);
  }

  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    int styleRemplissageEffectif;
    if (masque) styleRemplissageEffectif = StyleRemplissage.QuadrillageCroise;
    else styleRemplissageEffectif = styleRemplissage;
    if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePleinTransp) {
    	if (transparence != 1) {
    		pDC.setColor(couleur);
  	    Composite originalComposite = pDC.getComposite();
  	    pDC.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, transparence));
  	    pDC.fillPolygon(abscisses, ordonnees, nbPointsLigne + 3);
  	    pDC.setComposite(originalComposite);    		
    	}
    	else {
    		pDC.setColor(couleur);
    		pDC.fillPolygon(abscisses, ordonnees, nbPointsLigne + 3);
    	}
    }
    else {
      if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePlein) {
    		pDC.setColor(couleur);
    		pDC.fillPolygon(abscisses, ordonnees, nbPointsLigne + 3);
      }
      else {
	    	TexturePaint tp = StyleRemplissage.getTexturePaint(couleur, styleRemplissageEffectif, listeProprietaire, couleurFond);
	    	pDC.setPaint(tp);
	    	pDC.fillPolygon(abscisses, ordonnees, nbPointsLigne + 3);
      }
    }       
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double x1, y1, x4, y4;

    CLieuDeBase lieu = (CLieuDeBase) bord;
    existe = (lieu.existe) && (droiteAssociee.existe);
    if (!existe) return;
    // Le lieu doit tre form d'une seule ligne
    if (!lieu.contientUneSeuleLigne())
    {
      existe = false;
      return;
    }
    // On recopie les coordonnes des points du lieu
    nbPointsLigne = lieu.infoLignes[lieu.indicePremiereLigne].nombrePoints;
    int ideb = lieu.infoLignes[lieu.indicePremiereLigne].indicePremierPoint;
    System.arraycopy(lieu.abscissesCoordonneesPoints, ideb, abscisses, 0, nbPointsLigne);
    System.arraycopy(lieu.ordonneesCoordonneesPoints, ideb, ordonnees, 0, nbPointsLigne);
          
    int indicePremierPoint = lieu.infoLignes[lieu.indicePremiereLigne].indicePremierPoint;
    int indiceDernierPoint = lieu.infoLignes[lieu.indicePremiereLigne].indicePremierPoint +
      nbPointsLigne-1;
    x1 = lieu.abscissesCoordonneesPoints[indicePremierPoint];
    y1 = lieu.ordonneesCoordonneesPoints[indicePremierPoint];
    Kernel.projetteOrtho(x1,y1,droiteAssociee.point_x, droiteAssociee.point_y,
      droiteAssociee.vecteur,point2);
    x4 = lieu.abscissesCoordonneesPoints[indiceDernierPoint];
    y4 = lieu.ordonneesCoordonneesPoints[indiceDernierPoint];
    Kernel.projetteOrtho(x4,y4,droiteAssociee.point_x, droiteAssociee.point_y,
      droiteAssociee.vecteur,point3);
    pointDebut.placeEn(x1,y1);
    pointProjeteDebut.placeEn(point2.x,point2.y);
    pointFin.placeEn(x4,y4);
    pointProjeteFin.placeEn(point3.x,point3.y);
    abscisses[nbPointsLigne] = (int) Math.round(point3.x);
    ordonnees[nbPointsLigne] = (int) Math.round(point3.y);
    abscisses[nbPointsLigne+1] = (int) Math.round(point2.x);
    ordonnees[nbPointsLigne+1] = (int) Math.round(point2.y);
    abscisses[nbPointsLigne+2] = (int) Math.round(x1);
    ordonnees[nbPointsLigne+2] = (int) Math.round(y1);
    segmentDebut.positionne(infoRandom, dimfen);
    segmentDroite.positionne(infoRandom, dimfen);
    segmentFin.positionne(infoRandom, dimfen);
    existe = (segmentDebut.existe) & (segmentDroite.existe) && (segmentFin.existe);
  }

  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
   	double dist;
  	CLieuDeBase lieu;
  	
  	if (!existe || (masquage && masque)) return -1;
  	else
  	{
  		lieu = (CLieuDeBase) bord;
  		dist = lieu.distancePointPourSurface(xp,yp,masquage);
			if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
  		dist = segmentDebut.distancePoint(xp,yp,masquage);
			if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
  		dist = segmentDroite.distancePoint(xp,yp,masquage);
			if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
  		dist = segmentFin.distancePoint(xp,yp,masquage);
			if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
  		return -1;
  	}
  }

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		super.estDefiniParObjetDans(listeOb)
		&&
		(listeOb.contains(droiteAssociee) || droiteAssociee.estDefiniParObjetDans(listeOb));
	}           
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    pointDebut = new CPointBase();
    pointProjeteDebut = new CPointBase();
    pointFin = new CPointBase();
    pointProjeteFin = new CPointBase();
    segmentDebut = new CSegment(list, pointDebut, pointProjeteDebut);
    segmentFin = new CSegment(list, pointFin, pointProjeteFin);
    segmentDroite = new CSegment(list, pointProjeteDebut, pointProjeteFin);
    int ind1 = inps.readCInt();
    droiteAssociee = (CDroite) (list.get(ind1,CDroite.class));
    // On cre les tableaux devat recevoir les coordonnes. Le + 4 est du aux deux points projets
    // sur la droite et au cas d'un lieu ferm
    abscisses = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 4];
    ordonnees = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 4];
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    pointDebut = new CPointBase();
    pointProjeteDebut = new CPointBase();
    pointFin = new CPointBase();
    pointProjeteFin = new CPointBase();
    segmentDebut = new CSegment(list, pointDebut, pointProjeteDebut);
    segmentFin = new CSegment(list, pointFin, pointProjeteFin);
    segmentDroite = new CSegment(list, pointProjeteDebut, pointProjeteFin);
    int ind1 = inps.readInt();
    droiteAssociee = (CDroite) (list.get(ind1,CDroite.class));
    // On cre les tableaux devat recevoir les coordonnes. Le + 4 est du aux deux points projets
    // sur la droite et au cas d'un lieu ferm
    abscisses = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 4];
    ordonnees = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 4];
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(droiteAssociee);
    oups.writeInt(ind1);
  }    
}