/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Modification pour la version 3.0
 * CValeurAssocieeAVariableAncetre pourra contenir un calcul faiasnt rfrence  tous les objets
 * non graphiques prcdents
 * Il maintiendra donc un objet du type CCalculBase nomm calcul
 * Il maitiendra aussi un boolen nomm estConstant qui sera mis  true si le calcul
 * n'utilise que des constantes et un double nomm valeur qui contiendra la valeur renvoye
 * Un nouveau membre existe dira si le calcul associ existe
 * Tout objet qui contient des membres du type CValeurAssocieeAVariableAncetre devra dans sa
 * fonction positionne appeler la mthode positionne de CValeurAssocieeAVariableAncetre
 * @author  Yves
 * @version 
 */
package objets;

// Important : dans la version C++, CValeurAssocieeAVariableAncetre drivait de CObject
// Il est plus intelligent de la faire driver de CElementBase;
// Mais lui n'a pas  renvoyer de nature par GetNature qui renvera donc -1.
// A noter que le nom est mal choisi : ce devrait tre CValeurAssocieeAValeurDynamiqueAncetre
import java.io.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public abstract class CValeurAssocieeAVariableAncetre extends CObjetBase {
	public double valeur;  // La valeur si aucune valeur dynaique n'est associe
	public CCalculBase calcul;
	protected boolean estConstant;
	protected boolean existe;
	// Rajout version 3.1.0 pour optimisation. A la cration ou au chargement depuis un flux, sera mis  false
	// puis dans positionne, si dejaPositionne est false, on regardera si la calcul est constant.
	// S'il l'est il ne sera plus recalcul saut si on cahnge la formule
	protected boolean dejaPositionne;

  /** Creates new CValeurAssocieeAVariableAncetre */
  public CValeurAssocieeAVariableAncetre() {
  	dejaPositionne = false;
  }
  
  public CValeurAssocieeAVariableAncetre(CListeObjets list) {
  	super(list);
  	dejaPositionne = false;
  }
  
  
  // Constructeur pour clonage
  public CValeurAssocieeAVariableAncetre(CListeObjets liste, CCalculBase calculInit) {
    super(liste);
  	dejaPositionne = false;
    calcul = calculInit;
  }

  abstract CValeurAssocieeAVariableAncetre getClone(CListeObjets listeSource, CListeObjets listeCible);
  
  /** Fonction associant  l'objet la valeur dynamique vald */
  /*
  public void associeA(CValeurDynamique vald) {
    valeurDynamiqueAssociee = vald;
  }
  */
  /**
   * Ajout pour la version 3.0
   */
  public void donneCalcul(CCalculBase calc) {
  	calcul = calc;
  	dejaPositionne = false;
  }
  
  /**
   * Ajout pour la version 3.0
   * Donne une valeur constante.
   * @param val
   */
  public void donneValeur(double val) {
  	valeur = val;
  	calcul = new CConstante(listeProprietaire, valeur);
  	dejaPositionne = true;
  	estConstant = true;
  	existe = true;
  }
  
  /**
   * Ajout pour la version 3.0
   * Renvoie true si contient un calcul qui est le rsultat de val
   * @param val
   * @return
   */
  public boolean pointeSur(CValeurDynamique val) {
  	if (calcul.getClass() == CResultatValeur.class) {
  		return (((CResultatValeur) calcul).valeurAssociee == val);
  	}
  	else return false;
  }
  
  // La classe tant abstract, inutile de redfinir getClone()
  
  public boolean dependDe(CElementBase p) {
    return calcul.dependDe(p);
  }

  public boolean dependDePourReclassement(CElementBase p) {
    return calcul.dependDePourReclassement(p);
  }
  
  public boolean dependDePourBoucle(CElementBase p) {
    return calcul.dependDePourBoucle(p);
  }

  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    if (!dejaPositionne) {
      if (calcul.estConstant()) {
      	estConstant = true;
        existe = calcul.existe(infoRandom);
        if (!existe) return;
        try {
          listeProprietaire.initialiseNombreIterations();
          // Modifi version 3.5.1
          // valeur = calcul.resultat(false);
          valeur = calcul.resultat(infoRandom);
        } 
        catch (ErreurCalculException e) {
          existe = false;
          valeur = 0;
        }
        if (Double.isNaN(valeur) || Double.isInfinite(valeur)) {
          existe = false;
        }
      }
      else estConstant = false;
      dejaPositionne = true;
    }
  	if (!estConstant) {
      existe = calcul.existe(infoRandom);
      if (!existe) return;
      try {
        listeProprietaire.initialiseNombreIterations();
        valeur = calcul.resultat(infoRandom);
      } 
      catch (ErreurCalculException e) {
        existe = false;
        valeur = 0;
      }
      if (Double.isNaN(valeur) || Double.isInfinite(valeur)) {
        existe = false;
      }
  	}
  }
  
  public boolean confonduAvec(CObjetBase p) {
    CValeurAssocieeAVariableAncetre pt;
    if (p.getClass() == this.getClass()) {
      pt = (CValeurAssocieeAVariableAncetre) p;
      // Les deux lignes suivantes corrigent un bug de la version 3.3.0 qui faisait que
      // la vrification de l'exitene d'un objet dj cr similaire ne marchait plus.
      if (!dejaPositionne) positionne(false, null);
      if (!pt.dejaPositionne) pt.positionne(false, null);
      if (estConstant && pt.estConstant) return (valeur == pt.valeur);
      else {
      	if ((calcul.getClass() == CResultatValeur.class) && (pt.calcul.getClass() == CResultatValeur.class)) {
      		CResultatValeur res1 = (CResultatValeur) calcul;
      		CResultatValeur res2 = (CResultatValeur) pt.calcul;
      		return res1.valeurAssociee == res2.valeurAssociee;
      	}
      	else return false;
      }
    } else return false;
  }
  

  public boolean existe()
  {
    return existe;
  }
  
  public double rendValeur(boolean infoRandom) {
    return valeur; ///Cas d'une constante
  }
  
  /**
   * Renvoie une chaine tant gale au nom de la valeur associe s'il y en a un
   * et sinon la chaine reprsentant le calcul associ
   * @return
   */
  public String chaineInfo()
  {
    return calcul.chaineCalculSansPar(null);
  }
  
  /**
   * Fonction renvoyant true si l'objet a t construit avec un des objest de listeOb
   * @param listeOb
   * @return
   */
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return calcul.estDefiniParObjetDans(listeOb);
	}  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     valeur = inps.readCDouble();
     int ind1 = inps.readCInt();
     if (ind1 == -1) {
    	 // Si dans la version C++ la valeur tait constante, on doit crer un CCalculBase renvoyant cette constante
    	 calcul = new CConstante(list, valeur);
    	 estConstant = true;
    	 existe = true;
     }
     else {
    	 // On cre un calcul renvoyant la valeur dynamique associe en C++
    	 CValeurDynamique valeurDynamiqueAssociee = (CValeurDynamique) (list.get(ind1,CValeurDynamique.class));
    	 calcul = new CResultatValeur(list, valeurDynamiqueAssociee);
    	 // positionne(false,null);
    	 estConstant = false;
     }
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    calcul = (CCalculBase) (inps.readObject(list));
  	// Modifi version 3.1.5
  	// Il ne faut pas faire appel  existe car par exemple si le calcul est f(3) o f
  	// est une fonction utilisateur, f n'a pas encore t positionne
    /*
    calcul = (CCalculBase) (inps.readObject(list));
    if (calcul.estConstant()) {
    	estConstant = true;
      existe = calcul.existe();
      if (!existe) return;
      try {
        valeur = calcul.resultat(false);
      } 
      catch (ErreurCalculException e) {
        existe = false;
        valeur = 0;
      }
      if (Double.isNaN(valeur) || Double.isInfinite(valeur)) {
        existe = false;
      }
    }
    else {
    	estConstant = false;
    	// existe = calcul.existe();
    }
    */
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeObject(calcul);
  }
  
  /**
   * Ajout pour la version 3.0 : renvoie true seulement si la valeur est constante
   * ou si la valeur dynamique associe est constante
   * @return
   */
	public boolean estConstant() {
		return estConstant;
	}
	
	/**
	 * Ajout pour la version 3.0. Renvoie le calcul associ sous forme d'une chane
	 * @return
	 */
	public String chaineCalculValeur() {
		return calcul.chaineCalcul(null);
	}
  
  
}