/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package outils;

import java.awt.Color;

import objets.CCercleOA;
import objets.CDroitePerpendiculaire;
import objets.CElementGenerique;
import objets.CImplementationProto;
import objets.CIntDroiteCercle;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CPointImage;
import objets.CPointLieBipoint;
import objets.CPrototype;
import objets.CSegment;
import objets.CTranslation;
import types.StyleTrait;
import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilCarreDirect extends OutilPar2pts {

  /** Creates a new instance of OutilCarreDirect */
  public OutilCarreDirect(MtgFrame frameInit) {
    super(frameInit);
  }
  
  @Override
  public void creeObjet() {
		MtgInternalFrame cadre = frame.cadreActif();
		CListeObjets liste = cadre.listePr;
		CPrototype proto = frame.chargeConstructionDansDossierMtgFrame("CarreDirect.mgc");
		((CElementGenerique)proto.get(0)).elementAssocie = point1;
		((CElementGenerique)proto.get(1)).elementAssocie = point2;
		CImplementationProto impProto = new CImplementationProto(liste, proto);
		impProto.implemente(liste, proto, cadre);
		proto.retireTout(); // Ajout version 4.6.2
		impProto.nomProto = getString("carreDirect");  				
		CPointAncetre pt1 = (CPointAncetre) liste.get(liste.size() - 3);
		pt1.donneCouleur(frame.couleurActive());
		pt1.donneMotif(frame.stylePointActif);
		pt1.donneNom(frame.genereNomPourPoint()); // Ajout version 4.7.0.1
		CPointAncetre pt2 = (CPointAncetre) liste.get(liste.size() - 1);
		pt2.donneCouleur(frame.couleurActive());
		pt2.donneMotif(frame.stylePointActif);
		pt2.donneNom(frame.genereNomPourPoint()); // Ajout version 4.7.0.1
		CSegment seg1 = new CSegment(liste, null, false, frame.couleurActive(), false,
				frame.styleTraitActif(), point1, point2);
		if (!liste.existeDeja(seg1)) liste.add(seg1);
		CSegment seg2 = new CSegment(liste, null, false, frame.couleurActive(), false,
				frame.styleTraitActif(), point1, pt1);
		if (!liste.existeDeja(seg2)) liste.add(seg2);
		CSegment seg3 = new CSegment(liste, null, false, frame.couleurActive(), false,
				frame.styleTraitActif(), pt1, pt2);
		if (!liste.existeDeja(seg3)) liste.add(seg3);
		CSegment seg4 = new CSegment(liste, null, false, frame.couleurActive(), false,
				frame.styleTraitActif(), pt2, point2);
		if (!liste.existeDeja(seg4)) liste.add(seg4);
    frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil136"));
    annuleClignotement();
    cadre.repaintFigureAvecCalcul();    
    select();
  }
  
  @Override
  public void ajouteObjetsVisuels() {
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
		CListeObjets liste = frame.listeObjetsVisuels;
  	CSegment seg1 = new CSegment(liste, null, true, frame.couleurActive(), false,
  			frame.styleTraitActif(), point1, pane.pointSouris);
  	frame.ajouteObjetVisuel(seg1);
  	CDroitePerpendiculaire perp = new CDroitePerpendiculaire(liste, null, false,
  			Color.black, StyleTrait.traitFin, point1, seg1, frame.pref_indiceTaillePoliceNom);
  	perp.cache();
  	frame.ajouteObjetVisuel(perp);
  	CCercleOA cerc = new CCercleOA(liste, null, false, Color.black, true,
  			StyleTrait.traitFin, point1, pane.pointSouris);
  	frame.ajouteObjetVisuel(cerc);
  	CIntDroiteCercle inter = new CIntDroiteCercle(liste, null, false, perp, cerc);
  	frame.ajouteObjetVisuel(inter);
  	// On ne se sert que d'un des deux points d'intersection qui est dans le bon sens.
    CPointLieBipoint ptl = new CPointLieBipoint(liste, null, false, frame.couleurActive(),
        false, 0d, 0d, false, "", 2, frame.stylePointActif(), inter, 2, false);
		frame.ajouteObjetVisuel(ptl);
		CTranslation trans = new CTranslation(liste, null, false, point1, pane.pointSouris);
		frame.ajouteObjetVisuel(trans);
		CPointImage ptim = new CPointImage(liste, null, false, frame.couleurActive(), frame.stylePointActif(),
				frame.pref_indiceTaillePoliceNom, ptl, trans);
		frame.ajouteObjetVisuel(ptim);
  	CSegment seg2 = new CSegment(liste, null, true, frame.couleurActive(), false,
  			frame.styleTraitActif(), point1, ptl);
  	frame.ajouteObjetVisuel(seg2);
  	CSegment seg3 = new CSegment(liste, null, true, frame.couleurActive(), false,
  			frame.styleTraitActif(), ptl, ptim);
  	frame.ajouteObjetVisuel(seg3);
  	CSegment seg4 = new CSegment(liste, null, true, frame.couleurActive(), false,
  			frame.styleTraitActif(), ptim, pane.pointSouris);
  	frame.ajouteObjetVisuel(seg4);
  }
  
  public String chaineIndication (int indEtape) {
    switch(indEtape) {
      case 1 : return getString("carreDirect1");
      case 2 : return getString("carreDirect2");
      default: return "";
    }
  }
  
}
