/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import dialogs.ChoixObjetsNonGraphiquesPourImplementationProtoDlg;
import dialogs.ChoixPrototypeDansRepertoireDlg;
import dialogs.RenommerCalculDlg;
import entreesSorties.MathGraphDataInputStream;
import mathgraph32.Global;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CElementBase;
import objets.CElementGenerique;
import objets.CElementGraphique;
import objets.CImplementationProto;
import objets.CListeObjets;
import objets.CLongueur;
import objets.CPrototype;
import types.NatureCalcul;

public class OutilConstructionImplementationFichier extends Outil {
	public CPrototype proto ; // Le prototype  implmenter
	int nombreObjetsGraphiquesADesigner;
	int indiceElementSourceGraphique;
	int nombreObjetsCalculADetruire; // Ajout version 3.4.0.3 : il faut que, dans le cas o une bote de dialogue
	// de choix d'objets sources graphiques a t utilise et des calculs y ont t crs, qu'ils soient dtruits dans deselect()
	// lorsque la construction ne va pas  son terme.

	public OutilConstructionImplementationFichier(MtgFrame frameInit) {
    super(frameInit, true);
  }
	
	@Override
	public void deselect() {
		super.deselect();
		if (nombreObjetsCalculADetruire != 0) 
			frame.detruitNDerniersElements(nombreObjetsCalculADetruire);
			nombreObjetsCalculADetruire = 0;
	}
    	
  @Override
	public void select() {
    super.select();
    nombreObjetsCalculADetruire = 0;
    ChoixPrototypeDansRepertoireDlg dlg = dialogueUtilise();
		dlg.setLocationRelativeTo(null);
		dlg.setVisible(true);
		if (dlg.retourOK()) {
			File file = dlg.fc.getSelectedFile();
			String ch = file.getAbsolutePath();
			if (!ch.toLowerCase().endsWith(Global.GLOBAL_COMMA_MGC)) {
				ch = ch + Global.GLOBAL_COMMA_MGC;
				file = new File(ch);
			}			
			if (file != null) {
				FileInputStream fis;
				try {
					fis = new FileInputStream(file);
					MathGraphDataInputStream inps = new MathGraphDataInputStream(fis);
					proto = new CPrototype();
					try {
						proto.read(inps);
						inps.close();
					} catch (IOException ioerr) {
						frame.dialogueAvertissement(getString("ch35"));
						frame.activeOutilCapture();
						return;
					}
				} catch (FileNotFoundException e1) {
					frame.dialogueAvertissement(getString("ch35"));
					frame.activeOutilCapture();
					return;			
				}
			}
	    CListeObjets liste = frame.cadreActif().listePr;
	    // Il faut regarder si ce prototype peut tre implment dans la figure
	    String chErr = proto.implementationPossible(liste);
	    if (chErr.length() != 0) {
	    	frame.dialogueAvertissement(getString("ch142") + "\n" + chErr);
	    	frame.activeOutilCapture();
	    	return;
	    }
	    if ((liste.pointeurLongueurUnite == null) && proto.utiliseLongueurUnite()) {
	    	if (proto.premiereLongueur() == null) {
		    	frame.dialogueAvertissement(getString("ch138"));
		    	frame.activeOutilCapture();
		    	return;
	    	}
	    }

	    // Il peut arriver que la construction n'utilise aucun objet source
	    if (proto.nbObjSources == 0) {
	    	creeObjet();
	    	frame.activeOutilCapture();
	    	return;
	    }
	    if (proto.natureSources != CPrototype.ConstructionEltsGraphiquesSeuls) {
	    	// Modification version 3.5 : Si la macro ne possde qu'un seul objet source et que c'est un repre
	    	// et qu'il n'y a qu'un repre dans la figure (qui ne soit pas objet final, on l'affecte automatiquement)
	    	if ((proto.nombreSourcesTypeCalcul() == 1) && (proto.nombreObjetsSourcesCalcul(NatureCalcul.NRepere) == 1)
	    			&& liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere) == 1) {
	    		CElementGenerique gen = (CElementGenerique)proto.get(0);
	    		gen.elementAssocie = liste.premierRepere();	    		
	    	}
	    	else {
		    	// Choix des objets sources de type calcul
		    	ChoixObjetsNonGraphiquesPourImplementationProtoDlg dlg2 = new ChoixObjetsNonGraphiquesPourImplementationProtoDlg(frame, proto);
		  		dlg2.setLocationRelativeTo(null);
		    	dlg2.setVisible(true);
		    	if (dlg2.retourOK()) {
		    		nombreObjetsCalculADetruire = dlg2.nombreCalculsCrees;
		    	}
		    	else {
		      	frame.activeOutilCapture();
		      	return;    		
		    	}
	    	}
	    }
	    if (proto.natureSources == CPrototype.ConstructionEltsCalculsSeuls) {
	    	creeObjet();
	    	return;
	    }
	    else {
	  		frame.outilPointageActif = frame.outilPointageCre;
	  		indiceElementSourceGraphique = proto.nombreSourcesTypeCalcul();
	  		CElementGenerique gen = (CElementGenerique)proto.get(indiceElementSourceGraphique);
	  		long nat = gen.natureGraphique;
	  		// Diffrence par rapport au C++ : on ne peut plus dsigner un repre en cliquant sur ses axes
	  		/* Supprim version 3.4.0.3. N'a plus d'utilit car les repres sont des souces numriques
	  		while ((nat == NatureCalcul.NRepere) && (indiceElementSourceGraphique < proto.nbObjSources)) {
	  			ChoixRepereDlg repdlg = new ChoixRepereDlg(frame);
	  			repdlg.setLocationRelativeTo(frame);
	  			repdlg.setVisible(true);
	  			if (repdlg.retourOK()) {
	  				gen.elementAssocie = repdlg.repereChoisi;
	  				indiceElementSourceGraphique++;
	  				if (indiceElementSourceGraphique < proto.nbObjSources) {
		  				gen = (CElementGenerique)proto.get(indiceElementSourceGraphique);;
		  				nat = gen.natureCalcul;
	  				}
	  			}
	  			else {
	        	frame.activeOutilCapture();
	        	return;    		  				
	  			}
	  		}
	  		*/
	  		if (indiceElementSourceGraphique == proto.nbObjSources) {
	  			creeObjet();
	  			return;
	  		}
	  		frame.outilPointageActif.designationDe(CElementBase.natureGraphiquePourImplementationProto(nat));
	  		frame.indication(proto.chaineIndicationImplementationProto(indiceElementSourceGraphique + 1 , nat)
	  				+ ' ' + getString("ch158"));
	    }			
		}
		else {
			frame.activeOutilCapture();
		}
	}
	
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
		CElementGenerique gen = (CElementGenerique)proto.get(indiceElementSourceGraphique);
		gen.elementAssocie = elg;
		indiceElementSourceGraphique++;
		if (indiceElementSourceGraphique == proto.nbObjSources) creeObjet();
		else {
			CElementGenerique gen2 = (CElementGenerique)proto.get(indiceElementSourceGraphique);			
  		long nat = gen2.natureGraphique;
  		/* Supprim version 3.4.0.3. N'a plus d'utilit car les repres sont des souces numriques
  		while ((nat == NatureCalcul.NRepere) && (indiceElementSourceGraphique < proto.nbObjSources)) {
  			ChoixRepereDlg repdlg = new ChoixRepereDlg(frame);
  			repdlg.setLocationRelativeTo(frame);
  			repdlg.setVisible(true);
  			if (repdlg.retourOK()) {
  				gen2.elementAssocie = repdlg.repereChoisi;
  				indiceElementSourceGraphique++;
  				if (indiceElementSourceGraphique < proto.nbObjSources) {
	  				CElementGenerique gen3 = (CElementGenerique)proto.get(indiceElementSourceGraphique);;
	  				nat = gen3.natureCalcul;
  				}
  			}
  			else {
        	frame.activeOutilCapture();
        	return;    		  				
  			}
  		}
  		if (indiceElementSourceGraphique == proto.nbObjSources) {
  			creeObjet();
  			return;
  		}
  		*/
  		frame.outilPointageActif.designationDe(CElementBase.natureGraphiquePourImplementationProto(nat));
  		frame.indication(proto.chaineIndicationImplementationProto(indiceElementSourceGraphique + 1, nat)
  				+ ' ' + getString("ch158"));
			frame.outilActif.ajouteClignotementDe(elg);
			frame.outilActif.resetClignotement();
		}
	}
	
  @Override
	public void creeObjet() {
		MtgInternalFrame cadre = frame.cadreActif();
		CListeObjets liste = frame.cadreActif().listePr;
		CImplementationProto impProto = new CImplementationProto(liste, proto);
		CLongueur ancienneLongueurUnite = liste.pointeurLongueurUnite;
    CLongueur unite = null;
    if ((liste.pointeurLongueurUnite == null) && proto.utiliseLongueurUnite()) {
    	unite = proto.premiereLongueur();
    	if (unite == null) {
	    	frame.dialogueAvertissement(getString("ch138"));
	    	frame.activeOutilCapture();
	    	return;
    	}
    }
    impProto.implemente(liste, proto, cadre);
		int indprot = liste.indexOf(impProto);
    if (unite != null) {
    	// Si on a accept l'implmentation alors que la figure ne contient pas de longueur unit c'est que
    	// le premier objet utilisant une longueur dans le prototype est une longueur et est un objet final
    	// Ce sera en gnral une longueur unit
    	liste.pointeurLongueurUnite = liste.premiereLongueur();
    }
		liste.reconstruitChainesCalcul(indprot + 1, indprot + proto.size() - proto.nbObjSources);
    // On regarde si on moins un des objets finaux crs existe
    liste.positionne(false, cadre.dimf);
    boolean auMoinsUnExiste = false;
    // Optimis version 3.4.0.1
    // for (int i= liste.indexOf(impProto)+1; i < liste.size(); i++) {
    for (int i= liste.indexOf(impProto)+1; (i < liste.size()) && !auMoinsUnExiste; i++) {
    	CElementBase elb = liste.get(i);
    	auMoinsUnExiste = auMoinsUnExiste || elb.existe;
    }
    
    
    // Si aucun lment final existe on prvient l'utilisateur
		if (!auMoinsUnExiste) {
			if (!frame.dialogueConfirmation_YES_NO(getString("ch135"))) {
				liste.pointeurLongueurUnite = ancienneLongueurUnite;
				// Il faut retirer de la liste les lments qui ont t rajouts en cas de refus
				// Il y en a autant que d'objets finaux et intermdiaires plus l'implmentation de prototype elle-mme
				frame.detruitNDerniersElements(proto.size() - proto.nbObjSources + 1);
      	frame.activeOutilCapture();
      	return;    						
			}
		}
		// Si des objets finaux sont de type calcul nomm on propose de les renommer
		if (impProto.nombreObjetsFinauxTypeCalculNommes() != 0) {
			if (frame.dialogueConfirmation_YES_NO(getString("ch143"))) {
				int ind = liste.indexOf(impProto) + 1;
				RenommerCalculDlg dlg = new RenommerCalculDlg(frame, ind);
				dlg.setLocationRelativeTo(null);
				dlg.setVisible(true);
			}
		}
		
		// On sauvegarde l'tat de la figure
		// Ligne suivante supprime versio 4.2 (inutile appel par CImplementationProto.implemente)
	  // cadre.listePr.metAJour(); // Pour les lieux de points ou d'objets
	  cadre.repaintFigureAvecCalcul();
		cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil115"));
		// Version 3.1.5 : Si des objets finaux sont des variables, on met ventuellement  jour leur dialogue de modification
		boolean auMoinsUneVariableObjetFinal = false;
    for (int i= liste.indexOf(impProto)+1; (i < liste.size()) && !auMoinsUneVariableObjetFinal; i++) {
    	CElementBase elb = liste.get(i);
    	if (elb.estElementFinal && (elb.getNatureCalcul() == NatureCalcul.NVariable)) auMoinsUneVariableObjetFinal = true;
    }
    if (auMoinsUneVariableObjetFinal) {
			// cadre.supprimeDialogueAffichageVariables();    	
    	// cadre.creeDialogueAffichageVariables();
    	cadre.creePaneAffichageVariables();
  		// frame.mtgDesktopPane.getDesktopManager().activateFrame(cadre);		
    	// frame.requestFocus();
    }
    // Si tout s'est bien pass il ne faut pas que deselect dtruise les ventuels calculs crs dans
    // la bote de dialogue de choix d'objets sources graphiques.
    nombreObjetsCalculADetruire = 0;
	  frame.activeOutilCapture();
	}
  
  /**
   * Redfini pour OutilConstructionImplementationPredefinie
   * @return
   */
  protected ChoixPrototypeDansRepertoireDlg dialogueUtilise() {
  	return new ChoixPrototypeDansRepertoireDlg(frame);
  }
}
