/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import java.awt.geom.Point2D;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CCercle;
import objets.CDroiteAncetre;
import objets.CElementGraphique;
import objets.CLieuDePoints;
import objets.CLieuParVariable;
import objets.CLignePolygonaleAncetre;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CPointBase;
import objets.CPointLie;
import objets.CPointLieCercle;
import objets.CPointLieDroite;
import objets.CPointLieLieuParPtLie;
import objets.CPointLieLieuParVar;
import objets.CPointLieLigne;
import objets.CPointLiePoint;
import types.Doub;
import types.NatureObjet;

public class OutilCreationLiaison extends Outil {
	private CPointBase ptBase;
	private CElementGraphique objet;
  private int xclic;
  private int yclic;
	public OutilCreationLiaison(MtgFrame frameInit) {
		super(frameInit, true);
	}

  @Override
	public void select() {
		super.select();
		ptBase = null;
		objet = null;
		// Spcial version applet
		// CListeObjets liste = frame.cadreActif().listePr;
		CListeObjets liste = frame.listePourConstruction();
		frame.listeClignotante.ajouteObjetsParNature(liste, NatureObjet.NPointBase);		
		frame.outilPointageActif = frame.outilPointageObjetClignotant;
		frame.outilPointageActif.designationDe(NatureObjet.NPointBase);
		frame.indication(getString("creationLiaison1"));
		resetClignotement();		
	}

  @Override
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (ptBase == null) {
  		// Ajout version 3.2.2 : on refuse si le point libre est un objet final de construction et
  		// on conseille de fusionner d'abord les constructions de la figure.
  		if (elg.estElementFinal) {
  			frame.dialogueAvertissement(getString("ch174") + "\n" + getString("ch175"));
  			return;    			
  		}
      ptBase = (CPointBase) elg;
      annuleClignotement();
      frame.listeClignotante.add(ptBase);
      resetClignotement();
      frame.outilPointageActif = frame.outilPointageCre;
      long nat = NatureObjet.NTouteDroite + NatureObjet.NCercle + NatureObjet.NArc + NatureObjet.NToutPoint
      	+ NatureObjet.NPolygone + NatureObjet.NLigneBrisee + NatureObjet.NLieu - NatureObjet.NPointBase;
      frame.outilPointageActif.designationDe(nat);
      frame.outilPointageActif.reset();
  		frame.indication(getString("creationLiaison2"));
    }
    else {
    	if (objet == null) {
    		// Il faut regarder si l'objet auquel on veut lier le point libre dpend de ce point libre
    		if (elg.appartientABlocDependantPourReclassement(ptBase)) {
    			frame.dialogueAvertissement(getString("ch122"));
    			ptBase = null;
    			annuleClignotement();
    			frame.activeOutilCapture();
    			return;
    		}
        xclic = pointClique.x;
        yclic = pointClique.y;
        objet = elg;
    		annuleClignotement();
    		creeObjet();
    	}
    }
	}
	
  @Override
  public void creeObjet() {
    Doub pointabs = new Doub();
    Point2D.Double pointres = new Point2D.Double();
    CPointLie ptLie = null;
    CPointLiePoint ptLiePt = null;
    CPointAncetre nouveauPoint;
    CListeObjets liste = frame.cadreActif().listePr;    
    long nat = objet.getNature();
    if ((nat & NatureObjet.NTouteDroite) != 0) {
    	ptLie = new CPointLieDroite(liste, null, false, ptBase.couleur,
        ptBase.nomMasque, ptBase.dec_x, ptBase.dec_y, ptBase.masque, ptBase.rendNom(),
        ptBase.indiceTailleNom, ptBase.motif, ptBase.pointLibre, 0,
        (CDroiteAncetre) objet, ptBase.marquePourTrace);
    }
    else {
      if ((nat & NatureObjet.NToutCercle) != 0) {
      	ptLie = new CPointLieCercle(liste, null, false, ptBase.couleur,
      			ptBase.nomMasque, ptBase.dec_x, ptBase.dec_y, ptBase.masque, ptBase.rendNom(),
      			 ptBase.indiceTailleNom, ptBase.motif, true, 0,
        (CCercle) objet, ptBase.marquePourTrace);       
      }
      else {
        if (( nat == NatureObjet.NPolygone) || (nat ==NatureObjet.NLigneBrisee)) {
        	ptLie = new CPointLieLigne(liste, null, false, ptBase.couleur,
        			ptBase.nomMasque, ptBase.dec_x, ptBase.dec_y, ptBase.masque, ptBase.rendNom(),
        			ptBase.indiceTailleNom, ptBase.motif, true, 0,
            (CLignePolygonaleAncetre) objet, false);              
        }
        else {
        	if (( nat & NatureObjet.NToutPoint) != 0) {
        	ptLiePt = new CPointLiePoint(liste, null, false, ptBase.couleur,
        			ptBase.nomMasque, ptBase.dec_x, ptBase.dec_y, ptBase.masque, ptBase.rendNom(),
        			ptBase.indiceTailleNom, ptBase.motif, (CPointAncetre) objet, ptBase.marquePourTrace);     
        	}
        	else {
        		if (nat == NatureObjet.NLieu) {
        			if (objet.getClass() == CLieuDePoints.class) {
        				ptLie = new CPointLieLieuParPtLie(liste, null, false, ptBase.couleur,
        						ptBase.nomMasque, ptBase.dec_x, ptBase.dec_y, ptBase.masque, ptBase.rendNom(),
        						ptBase.indiceTailleNom, ptBase.motif, true, (CLieuDePoints) objet, 0, ptBase.marquePourTrace);
        			}
        			else {
        				// Cas d'un point li  un lieu de points gnr par une variable
        				ptLie = new CPointLieLieuParVar(liste, null, false, ptBase.couleur,
        						ptBase.nomMasque, ptBase.dec_x, ptBase.dec_y, ptBase.masque, ptBase.rendNom(),
        						ptBase.indiceTailleNom, ptBase.motif, true, (CLieuParVariable) objet, 0, ptBase.marquePourTrace);        				
        			}
        		}
        	}
        }
      }
    }
    if (ptLie != null) {
	    ptLie.testDeplacement(frame.cadreActif().dimf, xclic, yclic, pointres, pointabs);
	    ptLie.donneAbscisse(pointabs.getValue());
	    nouveauPoint = ptLie;
    }
    else nouveauPoint = ptLiePt;
		int indicePointRemplace = liste.indexOf(ptBase);
		int indiceObjetCible;
		if (objet.estElementFinal) indiceObjetCible = liste.indexOf(objet.impProto.dernierObjetFinal());
		else indiceObjetCible = liste.indexOf(objet);
		frame.remplaceElement(ptBase, nouveauPoint);
		liste.remplacePoint(ptBase, nouveauPoint);
		
		// Si l'objet auquel on doit lier le point a t cr aprs le
		// point, il faut dcaler le point ainsi que tous les objets en 
		// dpendant et ayant t crs entre ce point et l'objet
		// immdiatement aprs cet objet}
		if (indicePointRemplace < indiceObjetCible)
		{
			liste.decaleDependants(indicePointRemplace, indiceObjetCible);
		}
		// if (frame.cadreActif().isConstruction) frame.etablitListeObjetsConstruits(); // Spcial version applet
		frame.etablitListeObjetsConstruits(); // Modifi version 4.7
		MtgInternalFrame cadre = frame.cadreActif();
		
    cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil97"));
    cadre.repaintFigureAvecCalcul();
    frame.activeOutilCapture();
    cadre.videListesPourConstructions();  // On remet  jour une construction ventuellement commence
  }

}
