/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package outils;

import java.awt.Point;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CPointBase;
import objets.CPointInterieurPolygone;
import objets.CPolygone;
import objets.CSurfacePolygone;
import types.Bool;
import types.Couleur;
import types.Doub;
import types.Int;
import types.NatureObjet;
import types.StyleRemplissage;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class OutilCreationLiaisonInterieurPolygone extends Outil {
	private CPointBase ptBase;
	private CPolygone polygone1;
	private CPointInterieurPolygone pointInterieurPolygone;
	
	public OutilCreationLiaisonInterieurPolygone(MtgFrame frameInit) {
		super(frameInit, true);
	}

  @Override
	public void select() {
		super.select();
		ptBase = null;
		polygone1 = null;
		// Spcial version applet
		// CListeObjets liste = frame.cadreActif().listePr;
		CListeObjets liste = frame.listePourConstruction();
		frame.listeClignotante.ajouteObjetsParNature(liste, NatureObjet.NPointBase);		
		frame.outilPointageActif = frame.outilPointageObjetClignotant;
		frame.outilPointageActif.designationDe(NatureObjet.NPointBase);
		frame.indication(getString("creationLiaison1"));
		resetClignotement();		
  }
  
  @Override
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
  	MtgInternalFrame cadre = frame.cadreActif();
    if (ptBase == null) {
      ptBase = (CPointBase) elg;
      annuleClignotement();
      ajouteClignotementDe(ptBase);
      resetClignotement();
      frame.outilPointageActif = frame.outilPointageCre;
      frame.outilPointageActif.designationDe(NatureObjet.NPolygone);
      frame.outilPointageActif.reset();
  		frame.indication(getString("creationLiaison2"));
    }
    else {
    	if (polygone1 == null) {
    		// Il faut regarder si l'objet auquel on veut lier le point libre dpend de ce point libre
    		if (elg.appartientABlocDependantPourReclassement(ptBase)) {
    			frame.dialogueAvertissement(getString("ch122"));
    			ptBase = null;
    			annuleClignotement();
    			frame.activeOutilCapture();
    			return;
    		}
        polygone1 = (CPolygone) elg;
        ajouteClignotementDe(polygone1);
        // L'utilisateur doit maintenant cliquer  l'intrieur du polygone
        frame.outilPointageActif = frame.outilPointageClic;
        frame.indication(getString("creationLiaisonPolygone1"));
        resetClignotement();
        cadre.videDesignation();
        ajouteObjetsVisuels();
    	}
    	else {
        int x = pointClique.x;
        int y = pointClique.y;
        boolean bpointInt = false;
        // On regarde si le point cliqu est bien intrieur au polygone
        if (polygone1.pointInterieur(x, y)) {
        	CListeObjets liste = cadre.listePr;
        	Bool estSurCote = new Bool(false);
        	Int indicePremierSommet = new Int();
        	Doub alpha = new Doub();
        	Doub beta = new Doub();
        	Doub gamma = new Doub();
        	bpointInt = polygone1.determinePositionInterieure(x, y, indicePremierSommet, estSurCote, alpha, beta, gamma);
        	if (bpointInt) {
        		pointInterieurPolygone = new CPointInterieurPolygone(liste, null, false, ptBase.couleur,
        				ptBase.nomMasque, ptBase.dec_x, ptBase.dec_y, ptBase.masque, ptBase.rendNom(), ptBase.indiceTailleNom, ptBase.motif, true, polygone1,
        			estSurCote.getValue(), indicePremierSommet.getValue(), alpha.getValue(), beta.getValue(), gamma.getValue(), false);
        		pointInterieurPolygone.positionne(false, cadre.dimf);
  	      	if (pointInterieurPolygone.existe) creeObjet();
  	      	else bpointInt = false;
  	      }
        }
        if (!bpointInt) frame.dialogueAvertissement(getString("ch169"));
    	}
    }
  }
  
  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = frame.listeObjetsVisuels;
  	CSurfacePolygone surf = new CSurfacePolygone(liste, null, false, Couleur.TraduitCouleur(7), false,
  			StyleRemplissage.QuadrillagePleinTransp, polygone1);
  	frame.ajouteObjetVisuel(surf);
  }  
  
  
  @Override
  public void creeObjet() {
    CListeObjets liste = frame.cadreActif().listePr;      	
		int indicePointRemplace = liste.indexOf(ptBase);
		int indiceObjetCible;
		if (polygone1.estElementFinal) indiceObjetCible = liste.indexOf(polygone1.impProto.dernierObjetFinal());
		else indiceObjetCible = liste.indexOf(polygone1);
		frame.remplaceElement(ptBase, pointInterieurPolygone);
		liste.remplacePoint(ptBase, pointInterieurPolygone);
		// Si l'objet auquel on doit lier le point a t cr aprs le
		// point, il faut dcaler le point ainsi que tous les objets en 
		// dpendant et ayant t crs entre ce point et l'objet
		// immdiatement aprs cet objet}
		if (indicePointRemplace < indiceObjetCible) liste.decaleDependants(indicePointRemplace, indiceObjetCible);
		// if (frame.cadreActif().isConstruction) frame.etablitListeObjetsConstruits(); // Spcial version applet
		frame.etablitListeObjetsConstruits(); // Modifi version 4.7
		MtgInternalFrame cadre = frame.cadreActif();
		
    cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil97"));
    cadre.repaintFigureAvecCalcul();
    frame.activeOutilCapture();
    cadre.videListesPourConstructions();  // On remet  jour une construction ventuellement commence  	

  }
}
