/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;

import mathgraph32.MtgFrame;
import objets.CElementGraphique;
import objets.CInversion;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CPointImage;
import objets.CSegment;
import objets.CTransformation;
import objets.CValeurAssocieeAVariable;
import types.Couleur;
import types.Int;
import types.MotifPoint;
import types.NatureObjet;
import dialogs.RapportInversionDlg;

public class OutilInversion extends Outil {
  CPointAncetre point;
  CElementGraphique objet;
  private int nombreImagesCrees;
  private CInversion ptInv;
  
  /** Creates a new instance of OutilRot */
  public OutilInversion(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  @Override
  public void select() {
    super.select();
    point = null;
    objet = null;
    nombreImagesCrees = 0;
    frame.outilPointageCre.designationDe(NatureObjet.NToutPoint);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(getString("inversion1"));
  }
  
  /**
   * Il est ncessaire de regarder quand l'outil est dselectionn si on a vraiment cr des images
   * par une transformation. Sinon il faut supprimer le dernier objet qui est une homothtie
   */
  @Override
  public void deselect() {
    CListeObjets liste = frame.cadreActif().listePr;
    if (ptInv == liste.get(liste.size() - 1)){
    	frame.detruitDernierElement();
    }
    super.deselect();
  }

  @Override
  public void creeObjet() {
    CListeObjets liste = frame.cadreActif().listePr;
    if (nombreImagesCrees == 0) {
    	CTransformation inv = liste.transformationConfondueAvec(ptInv);
    	if (inv != null) {
    		ptInv = (CInversion) inv;
     	}
    	else frame.ajouteElement(ptInv);
    }
    Int nbImagesCrees = new Int();
    CElementGraphique elg = imageParTransformation(liste,  objet, ptInv, nbImagesCrees);
    if (frame.ajouteElementsImageAvecVerification(elg, nbImagesCrees.getValue(), ptInv)) {
    	frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil22"));
    	nombreImagesCrees++;
  		frame.cadreActif().repaintFigureAvecCalcul();    	
    }
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    CListeObjets liste = frame.cadreActif().listePr;
    if (point == null) {
      frame.indication(getString("inversion2"));
      point = (CPointAncetre) elg;
      excluDeDesignation(elg);
      ajouteClignotementDe(elg);
      resetClignotement();
      CValeurAssocieeAVariable rap = new CValeurAssocieeAVariable(liste, 0);
      ptInv = new CInversion(liste, null, false, point, rap);
      // On cre un dialogue modal permettant d'entrer l'angle de la rotation
      RapportInversionDlg rapportDlg = new RapportInversionDlg(frame, ptInv, false);
      // On redessine la figure pour liminer la dsignation ce point affiche pour le centre
      frame.cadreActif().videDesignation();
      frame.cadreActif().repaintFigureSansCalcul();
  		rapportDlg.setLocationRelativeTo(null);
      rapportDlg.setVisible(true);
      // On regarde si la bote de dialogue a bien t valide par OK
      if (rapportDlg.retourOK()) {
        ajouteObjetsVisuels();
        resetClignotement();
        frame.indication(getString("inversion3"));
      }
      else {
        annuleClignotement();
        select(); 
      }      
    }
    else {
      objet = elg;
      creeObjet();
    }
  }      
  /* Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = frame.listeObjetsVisuels;
    frame.ajouteObjetVisuel(ptInv);
    CPointImage ptim = new CPointImage(liste, null, false, Couleur.TraduitCouleur(1),
      MotifPoint.petitRond, 2, frame.cadreActif().paneFigure.pointSouris, ptInv);
    frame.ajouteObjetVisuel(ptim);
  	// On cre un segment en pointills joignant le centre au point souris
  	CSegment pts1 = new CSegment(liste, null, false, point, ptim);
    frame.ajouteObjetVisuel(pts1);
  }
}
