/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import java.util.Vector;

import objets.CElementGraphique;
import objets.CLigneBrisee;
import objets.CListeObjets;
import objets.CNoeudPointeurSurPoint;
import objets.CPointAncetre;
import objets.CPolygone;
import objets.CSegment;
import types.NatureObjet;

import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;

public class OutilLigneBrisee extends Outil {
  CPointAncetre premierPoint;
  Vector<CNoeudPointeurSurPoint> listePoints;
  
	public OutilLigneBrisee(MtgFrame frameInit) {
		super(frameInit, false); // False car pas de clignotement ncessaire
	}
	
  @Override
  public void select() {
    super.select();
    frame.outilPointageCre.designationDe(NatureObjet.NToutPoint);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(getString("ligneBrisee1"));
    listePoints = new Vector<CNoeudPointeurSurPoint> ();
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    CPointAncetre point = (CPointAncetre) elg;
    ajouteClignotementDe(elg);
  	if (listePoints.size() == 0) {
  		premierPoint = point;
  		CNoeudPointeurSurPoint n = new CNoeudPointeurSurPoint(point.listeProprietaire, point);
  		listePoints.add(n);
  		ajouteObjetsVisuels();
  		// Pour que l'objet soit bien visible, on dplace le pointeur souris
  		// vers la droite
  		excluDeDesignation(point);
  	}
  	else
  	if (point == premierPoint) {
  		if (listePoints.size() <= 2) {
  			frame.dialogueAvertissement(getString("ch42"));
  			select();
  		}
  		else {
    		frame.listeExclusionDesignation.retireTout();
    		excluDeDesignation(point);
  			CNoeudPointeurSurPoint n = new CNoeudPointeurSurPoint(point.listeProprietaire, point);
  			listePoints.add(n);
  			ajouteObjetsVisuels();
  		}
  	}
  	else
  	{
  		frame.listeExclusionDesignation.retireTout();
  		excluDeDesignation(point);
  		CNoeudPointeurSurPoint n = new CNoeudPointeurSurPoint(point.listeProprietaire, point);
  		listePoints.add(n);
  		ajouteObjetsVisuels();
  	}    
  }
  
  @Override
  public void creeObjetApresClicDroit() {
		if (listePoints.size() <= 2) {
			frame.dialogueAvertissement(getString("ch41"));
			select();
		}
		else {
			CNoeudPointeurSurPoint n1 = listePoints.firstElement();
			CNoeudPointeurSurPoint n2 = listePoints.lastElement();
			if (n1.pointeurSurPoint == n2.pointeurSurPoint ) {
				// Si le point de dbut et de fin sont confondus on cre un polygone
				frame.dialogueAvertissement(getString("ch43"));
				CPolygone poly = new CPolygone(frame.cadreActif().listePr, listePoints, frame.couleurActive(), frame.styleTraitActif());
				if (frame.ajouteElementAvecVerification(poly)) {
				  frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil36"));
				  frame.cadreActif().repaintFigureAvecCalcul();
			    annuleObjetsVisuels(); // Pour effacer les objets visuels			
					select();
				}
			}
			else {
				CLigneBrisee poly = new CLigneBrisee(frame.cadreActif().listePr, listePoints, frame.couleurActive(), frame.styleTraitActif());
				if (frame.ajouteElementAvecVerification(poly)) {
				  frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil37"));
				  frame.cadreActif().repaintFigureAvecCalcul();
				}
		    annuleObjetsVisuels(); // Pour effacer les objets visuels			
				select();
			}
		}
  }
  
  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = frame.listeObjetsVisuels;
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
  	// Si ce n'est pas le premier point, on transforme le dernier segment en modifiant
  	// son extrmit qui est pointe maintenant par Prelg;
  	CPointAncetre dernierPoint = listePoints.lastElement().pointeurSurPoint;
  	if (listePoints.size() >= 2)
  	{
  		CSegment pts2 = (CSegment) frame.listeObjetsVisuels.get(frame.listeObjetsVisuels.size()-1);
  		pts2.point2 = dernierPoint;
  	}
  	// On cre un segment joignant le dernier point entr au point souris
  	CSegment pts1 = new CSegment(liste, null, false,frame.couleurActive(), false,frame.styleTraitActif(),
  			dernierPoint, pane.pointSouris);
  	liste.add(pts1);;
  }
}
