/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Cursor;
import java.awt.Point;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CAffichageLieAPoint;
import objets.CCommentaire;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CMacro;
import objets.CMacroBoucleAvecTrace;
import objets.CMacroTraceAutoParVariable;
import objets.CSousListeObjets;
import types.NatureCalcul;
import types.NatureObjet;
import types.StyleEncadrement;
import dialogs.MacroBoucleAvecTraceDlg;

public class OutilMacroBoucleAvecTrace extends Outil {
  private boolean dialogueRempli;
  private CMacroBoucleAvecTrace macro;
  private CSousListeObjets sousListe;
	public OutilMacroBoucleAvecTrace(MtgFrame frameInit) {
		super(frameInit, true);
	}

	@Override
	public Cursor curseurUtilise() {
		if (dialogueRempli) return MtgFrame.crossHairCursor;
		else return MtgFrame.cornerCursor;
	}

  @Override
	public void select() {
		super.select();
    dialogueRempli = false;
		frame.outilPointageActif = frame.outilPointageClic;
		frame.indication(getString("macro1"));
	}

  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    int x = pointClique.x;
    int y = pointClique.y;
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets liste = cadre.listePr;
    if (!dialogueRempli) {
	    // Avant l'affichage du dialogue on cre un message qui sera ajout  la liste des objets clignotants
	    // pour matrialiser l'emplacement de l'affichage
    	CCommentaire comClignotant = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 0d, 0d, false, null, 2,
	    		StyleEncadrement.Sans, false, frame.cadreActif().couleurFond, getString("ch80"),
					CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
      comClignotant.positionne(false, frame.cadreActif().dimf);
	    ajouteClignotementDe(comClignotant);
	    resetClignotement();
 	    macro = new CMacroBoucleAvecTrace(liste, null, false, frame.couleurActive(), 
	    		x, y, 0d, 0d, false, null, 2, StyleEncadrement.Effet3D, false, cadre.couleurFondAffichage, "", "",
	    		null, 10, true, null, null, null,
	    		CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
	    frame.indication("");
	    MacroBoucleAvecTraceDlg dlg = new MacroBoucleAvecTraceDlg(frame, macro, false);
			dlg.setLocationRelativeTo(null);
	    dlg.setVisible(true);
	    if (dlg.retourOK()) {
	    	dialogueRempli = true;
	    	frame.indication(getString("macroAnimationAvecTrace1"));
	    	sousListe = new CSousListeObjets();
	    	frame.outilPointageActif = frame.outilPointageCre;
	    	frame.outilPointageActif.designationDe(NatureObjet.NToutObjet);
	    	frame.outilPointageActif.reset();
	      resetClignotement();      
	    }
	    else {
	    	annuleClignotement();
	    	frame.activeOutilCapture();
	    }
    }
    else {
    	frame.listeClignotante.add(elg);
    	frame.listeExclusionDesignation.add(elg);
    	sousListe.add(elg);
    }
  }
  
  @Override
	public void creeObjetApresClicDroit() {
		if (dialogueRempli) {
			// On compte le nombre d'objets cliqus
			int nb = sousListe.size();
			// Si aucun objet n'a t choisi, la cration n'est possible que si la macro de fin de boucle
			// est une macro de trace automatique par vriable
			CMacro macroFin = macro.macroFinBoucle;
			if (nb == 0) {
				if ((macroFin == null) || ((macroFin != null) && (macroFin.getClass() != CMacroTraceAutoParVariable.class))) {
					frame.dialogueAvertissement(getString("ch131"));
		    	annuleClignotement();
		    	frame.activeOutilCapture();
		    	return;
				}
			}
    	macro.listeAssociee = sousListe;
    	annuleClignotement();
			if (frame.ajouteElementAvecVerification(macro)) {
			  frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil109"));
			  frame.cadreActif().repaintFigureAvecCalcul();
			}
			frame.activeOutilCapture();
		}
	}
  
  @Override
	public boolean activationValide() {
		return (frame.cadreActif().listePr.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NVariable) != 0);
	}
	
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDeVariable");
	}	  	

}
