/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Color;
import objets.*;
import types.MotifPoint;
import types.StyleEncadrement;
import types.StyleFleche;
import types.StyleTrait;
import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

/**
 *
 * @author Yves
 */
public class OutilMesLong extends OutilPar2pts{

  /** Constructor */
  public OutilMesLong(MtgFrame frameInit) {
    super(frameInit);
  }

  /**
   * Les outils descendant de OutilPar2Pts et necessitant que les points dsigns
   * soeint nomms s'ils n'ont pas de noms devront redfinir cette fonction en renvoyant true
   * Cela servira pour l'outil de mesure de longueur
   * @return true si les points dsigns doivent tre nomms
   */
  @Override
  boolean nomsPointsIndisp() {
    return true;
  }
  
  @Override
  public void creeObjet() {
		MtgInternalFrame cadre = frame.cadreActif();
		CListeObjets liste = cadre.listePr;
  	if (frame.pref_affichageMesuresNiveau[frame.pref_niveau]) {
  		int nbObjetsInitiaux = liste.size();
			CSegment seg = new CSegment(liste, null, false, frame.couleurActive(), false,
					frame.styleTraitActif(), point1, point2);
			if (!liste.existeDeja(seg)) liste.add(seg);
  		CPrototype proto = frame.chargeConstructionDansDossierMtgFrame("MesureLongueur.mgc");
  		((CElementGenerique)proto.get(0)).elementAssocie = point1;
  		((CElementGenerique)proto.get(1)).elementAssocie = point2;
  		CImplementationProto impProto = new CImplementationProto(liste, proto);
  		impProto.implemente(liste, proto, cadre);
  		impProto.nomProto = getString("mesureLongueur");  		
  		CLongueur lon = (CLongueur) liste.get(liste.size() - 2);
  		// On regarde si une telle longueur a dj t mesure
  		if (liste.existeDeja(lon, nbObjetsInitiaux - 1)) {	
				frame.dialogueAvertissement(getString("ch7"));
				frame.detruitNDerniersElements(liste.size() - nbObjetsInitiaux);
  		} else {
  			objetCree = true;
	  		CValeurAffichee affval = (CValeurAffichee) liste.get(liste.size() - 1);
	  		// affval.enTete = point1.rendNom() + point2.rendNom() + " = ";
	  		affval.couleurFondAffichage = cadre.couleurFond;
	  		// affval.effacementFond = true; Modifi version 4.6.3 false dans la construction
	  		affval.donneCouleur(frame.couleurActive());
	  		affval.indiceTaillePolice = 3; // Rajout version 4.6.3
	      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil23"));
	      cadre.repaintFigureAvecCalcul();
  		}
  	} else {
	    CLongueur lon = new CLongueur(liste, null, false , point1, point2);
	    if (frame.ajouteElementAvecVerification(lon)) {
	      objetCree = true; // Necessaire  cause du fait que point1 et point2 ont t ventuellement
	      // nomms et que si on ne cre pas l'objet on remet alors leur nom  une chaine vide
	      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil23"));
	      frame.dialogueInformation(getString("ch25") + " " + lon.nom() + " " +  getString("ch26"));
	    }
  	}
    annuleClignotement();
    select();
  }
  
  /** Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets list = frame.listeObjetsVisuels;
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
    CMathGrafDoc doc = frame.cadreActif().document;
    // On cre deux vecteurs provisoires pour avoir une double flche
    CVecteur v1 = new CVecteur(list, null, false,
            frame.couleurActive(), false, StyleTrait.traitFin,
            point1, pane.pointSouris, StyleFleche.FlecheLonguePleine);
    frame.ajouteObjetVisuel(v1);
    CVecteur v2 = new CVecteur(list, null, false,
            frame.couleurActive(), false, StyleTrait.traitFin,
            pane.pointSouris, point1, StyleFleche.FlecheLonguePleine);
    frame.ajouteObjetVisuel(v2);
    // On cre le milieu pour lui lier un point d'interrogation     
    CMilieu mil = new CMilieu(list, null, false, Color.BLACK,
            MotifPoint.croix, pane.pointSouris, point1);
    mil.cache();
    frame.ajouteObjetVisuel(mil);
    // Si on est en niveau lmentaire, on cre un affichage de longueur li au milieu
  	if (frame.pref_affichageMesuresNiveau[frame.pref_niveau]) {
    	// Attention : la liste visuelle doit avoir une longueur unit
    	frame.listeObjetsVisuels.pointeurLongueurUnite = frame.cadreActif().listePr.pointeurLongueurUnite;
    	CLongueur lon = new CLongueur(list, null, false, point1, pane.pointSouris);
    	frame.ajouteObjetVisuel(lon);
    	// Modification version 4.6.3
    	/*
    	CValeurAffichee affval = new CValeurAffichee(list, null, false, frame.couleurActive(),
    			0d, 0d, 0d, 0d, false, mil, 3, StyleEncadrement.Sans, true, doc.couleurFond, lon, "$", "$ ?", 1,
    			CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);
    	frame.ajouteObjetVisuel(affval);
    	*/
    	CMediatrice med = new CMediatrice(list, null, false, Color.black, true, 0, 0, true, "", 2,
    			StyleTrait.traitFin, 0.9, point1, pane.pointSouris);
    	frame.ajouteObjetVisuel(med);
    	CCercleOR cerc = new CCercleOR(	list, null, false, Color.black, true, StyleTrait.traitFin, mil,
    			new CValeurAssocieeAVariable(list, 16), true); // Rayon en pixels
    	frame.ajouteObjetVisuel(cerc);
    	CIntDroiteCercle intdc = new CIntDroiteCercle(list, null, false, med, cerc);
    	frame.ajouteObjetVisuel(intdc);
    	CPointLieBipoint ptplb = new CPointLieBipoint(list, null, false, Color.black,
    			false, 0, 0, true, "", 2, MotifPoint.petitRond, intdc, 1, false);
    	frame.ajouteObjetVisuel(ptplb);
    	CValeurAffichee affval = new CValeurAffichee(list, null, false, frame.couleurActive(),
    			0d, 0d, 0d, 0d, false, ptplb, 3, StyleEncadrement.Sans, false, doc.couleurFond, lon, "$", "$ ?", 1,
    			CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);
    	frame.ajouteObjetVisuel(affval);  	
    	
    } else {
	    // On cre un commentaire contenant comme texte un point d'interrogation
	    CCommentaire com = new CCommentaire(list, null, false, Color.BLACK, 0d, 0d,
        0d, +0d, false, mil, 5, StyleEncadrement.Sans, true, doc.couleurFond, "#G?",
        CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);
	    frame.ajouteObjetVisuel(com);
    }
  }
  
  /**
   * Fonction renvoyant la chane  afficher  l'tape i (i >= 1) comme indication
   * Doit tre redfinie par les descendants devant crire plusieurs indications
   * @param i : l'indice
   * @return
   */          
  @Override
  public String chaineIndication (int indEtape) {
    switch(indEtape) {
      case 1 : return getString("mesLong1");
      case 2 : return getString("mesLong2");
      default: return "";
    }
  } 
  
  @Override
  public boolean activationValide() {
    return (frame.cadreActif().listePr.pointeurLongueurUnite != null);
  }
  
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDeLongueurUnite");
	}	
  
}