/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Color;
import java.awt.Point;

import objets.CAffichageLieAPoint;
import objets.CBarycentre;
import objets.CCommentaire;
import objets.CElementBase;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CNoeudPondere;
import objets.CPointAncetre;
import objets.CValeurAssocieeAVariable;
import types.NatureObjet;
import types.StyleEncadrement;

import dialogs.CoefficientBarycentreDlg;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

/** L'outil de modification d'un barycentre est particulier car il n'est pas slecionnable par la
 * barre d'outils ou par menu
 * @author Yves
 *
 */
public class OutilModifierBarycentre extends Outil {
	private CBarycentre barycentre;
	private CPointAncetre pointDesigne;

	/** Creates a new instance of OutilModifierBarycentre */
	public OutilModifierBarycentre(MtgFrame frameInit) {
		super(frameInit, true);
	}

	public void select(CBarycentre barycentreInit) {
		super.select();
		barycentre = barycentreInit;
		ajouteClignotementDe(barycentre);
		excluDeDesignation(barycentre);
		frame.indication(getString("modificationBarycentre1"));
		// Pour chacun des points pondrs, on cr un commentaire associ qui affichera le coefficient
		// Tous ces commentaires seront rajouts  la liste des objets clignotants
		int nbPoints = barycentre.col.size();
		for (int i = 0; i < nbPoints; i++) {
			CNoeudPondere noeud  = barycentre.col.get(i);
			CPointAncetre point = noeud.pointAssocie;
			ajouteClignotementDe(point);
    	CCommentaire com = new CCommentaire(barycentre.listeProprietaire, null, false, Color.blue, 0d, 0d, 0d, 10d, false, point, 2,
    			StyleEncadrement.Simple, true,  new Color(230,230,246), noeud.poids.chaineInfo(),
    			CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalHaut);
    	com.positionne(false, frame.cadreActif().dimf);
    	ajouteClignotementDe(com);
    	excluDeDesignation(com);
		}
		frame.outilPointageActif = frame.outilPointageObjetClignotant;
		frame.outilPointageActif.designationDe(NatureObjet.NToutPoint);
		frame.outilPointageActif.reset();
	}
	
	@Override
	public void deselect() {
		barycentre = null;
		System.gc(); // Pour dtruire les commentaires clignotants
		super.deselect();
	}
	
	@Override
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
		MtgInternalFrame cadre = frame.cadreActif();
		CValeurAssocieeAVariable coef = null;
		pointDesigne = (CPointAncetre) elg;
    cadre.videDesignation();
    // On recherche le coefficient du point qui a t cliqu
    for (int i = 0; i < barycentre.col.size(); i++) {
    	CNoeudPondere noeud = barycentre.col.get(i);
    	if (noeud.pointAssocie == pointDesigne) {
    		coef = noeud.poids;
    		break;
    	}
    }
    // On ouvre une bote de dialogue pour changer ce coefficient
    CoefficientBarycentreDlg dlg = new CoefficientBarycentreDlg(frame, barycentre, coef, true);
		dlg.setLocationRelativeTo(null);
    dlg.setVisible(true);
    if (dlg.retourOK()) {
    	CListeObjets liste = frame.listeClignotante;
    	// Si un coefficient a t chang on doit modifier le commentaire clignotant
    	for (int j = 0; j < liste.size(); j++) {
    		CElementBase elb =  liste.get(j);
    		if (elb.getNature() == NatureObjet.NCommentaire) {
    			CCommentaire com = (CCommentaire) elb;
    			if (com.pointLie == pointDesigne) {
    				// com.efface(cadre.gbuffer, true, doc.couleurFond, doc.transparencePourSurfaces);
    				com.chaineCommentaire = coef.chaineInfo();
    				com.positionne(false, frame.cadreActif().dimf);
    				break;
    			}
    		}
    	}
		  frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil66"));
		  frame.cadreActif().repaintFigureAvecCalcul();    	
    	resetClignotement();
    	
    }   
	}

  @Override
  public void creeObjetApresClicDroit() {
  	frame.activeOutilCapture();
  }
	
}
