/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;

import mathgraph32.MtgFrame;
import objets.CDroite;
import objets.CElementGraphique;
import types.NatureObjet;

/**
 * Outil anctre de tous les outils devant attendre le clic sur une droite
 * Si la droite doit tre nomme, la fonction nomsPointsIndisp doit tre redfinie
 * @author Yves
 *
 */

public class OutilParDte extends Outil {
  CDroite droite1;
  // objetCre servira dans le cas ou les points dsigns doivent tre renomms
  // Si l'objet n'a pas t cr, il faudra raffecter  point1 et point2 leurs
  // anciens noms au cas o ceux-ci auraient t changs
  protected boolean objetCree;
  private boolean droite1Nommee;
  /** Creates a new instance of OutilPar2pts */
  public OutilParDte(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  /**
   * Les outils descendant de OutilPar2Pts et necessitant que les points dsigns
   * soient nomms s'ils n'ont pas de noms devront redfinir cette fonction en renvoyant true
   * Cela servira pour l'outil de mesure de longueur
   * @return true si les points dsigns doivent tre nomms
   */
  boolean nomsPointsIndisp() {
    return false;
  }
  
  @Override
  public void select() {
    super.select();
    droite1 = null;
    droite1Nommee = false;
    frame.outilPointageCre.designationDe(NatureObjet.NDroite);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(chaineIndication(1));
    objetCree = false;
  }
  
  @Override
  public void deselect() {
    if (nomsPointsIndisp() && !objetCree) {
      if ((droite1 != null) && droite1Nommee) droite1.donneNom("");
     }
    droite1 = null;
    droite1Nommee = false;
    super.deselect();
  }  
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (droite1 == null) {
      droite1 = (CDroite) elg;
      if (nomsPointsIndisp()) {
        if (droite1.rendNom().length() == 0) {
          droite1.donneNom(frame.cadreActif().listePr.genereNomPourDroite());
          droite1Nommee =true;
        }
      }
      creeObjet();
      annuleClignotement();
    }
  }
}
