/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import java.util.ListIterator;

import dialogs.ChoixRepereDlg;

import mathgraph32.MtgFrame;
import objets.CElementBase;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CRepere;
import types.NatureCalcul;
import types.NatureObjet;

	/**
	 * Outil anctre de tous les outils devant attendre le clic sur un point qui devra tre nomm
	 * puis le choix d'un repre sauf s'il n'y a qu'un seul repre de cr
	 * IL faut redfinir chaineIndication pour les descendants
	 * @author Yves
	 *
	 */
public class OutilParPointRepere extends Outil {
  CPointAncetre point1;
  CRepere repere1;
  // objetCre servira dans le cas ou les points dsigns doivent tre renomms
  // Si l'objet n'a pas t cr, il faudra raffecter  point1 et point2 leurs
  // anciens noms au cas o ceux-ci auraient t changs
  protected boolean objetCree;
  private boolean point1Nomme;
  
  public OutilParPointRepere(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  @Override
  public void select() {
    super.select();
    point1 = null;
    frame.outilPointageCre.designationDe(NatureObjet.NToutPoint);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(chaineIndication(1));
    objetCree = false;
    point1Nomme = false; // Correction bug version 4.5
  }
  
  /** Appel lorsque l'outil est dslectionn.
   * Se charge de redonner un nom vide aux points s'ils avaient t nomms par l'outil
   */
  @Override
  public void deselect() {
    if (!objetCree) {
      if ((point1 != null) && point1Nomme) point1.donneNom("");
    }
    super.deselect();
  }
 
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (point1 == null) {
      point1 = (CPointAncetre) elg;
      if (point1.rendNom().length() == 0) {
        point1.donneNom(frame.cadreActif().listePr.genereNomPourPoint());
        point1Nomme =true;
      }
      ajouteClignotementDe(elg);
      resetClignotement();
      frame.indication(chaineIndication(2));
      // Si un seul repre a t cr on le fait pointer par repere1
      CListeObjets liste = frame.cadreActif().listePr;
      int nbrep = liste.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere);
      repere1 = null;
      if (nbrep == 1) {
      	ListIterator<CElementBase> li = liste.listIterator();
      	while (li.hasNext() && (repere1 == null)) {
      		CElementBase elb = li.next();
      		if (!elb.estElementIntermediaire() && (elb.getNatureCalcul() == NatureCalcul.NRepere)) {
      			repere1 = (CRepere) elb;
      		}
      	}
      }
      else { //  Sinon on ouvre une bote de dialogue de choix de repre
      	frame.cadreActif().videDesignation();
      	ChoixRepereDlg dlg = new ChoixRepereDlg(frame);
    		dlg.setLocationRelativeTo(null);
      	dlg.setVisible(true);
      	if (!dlg.retourOK()) frame.activeOutilCapture();
      	repere1 = dlg.repereChoisi;
      }
      if (repere1 != null) creeObjet();
      else annuleClignotement();
    }
  }  
}
