/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package outils;

import java.util.Vector;

import dialogs.PolygoneRegulierDlg;
import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CListeObjets;
import objets.CNoeudPointeurSurPoint;
import objets.CPointAncetre;
import objets.CPointImage;
import objets.CPolygone;
import objets.CRotation;
import objets.CValeurAngle;
import types.UniteAngle;

public class OutilPolygoneRegulier extends OutilPar2pts {
	int nombreSommets;
  /** Creates a new instance of OutilCarreDirect */
  public OutilPolygoneRegulier(MtgFrame frameInit) {
    super(frameInit);
  }
  
  @Override
  public void select() {
  	frame.indication("");
  	PolygoneRegulierDlg dlg = new PolygoneRegulierDlg(frame);
		dlg.setLocationRelativeTo(null);
  	dlg.setVisible(true);
		if (!dlg.retourOK()) {
			frame.activeOutilCapture();
			return;
		}
		nombreSommets = dlg.nombrePoints;
  	super.select();
  }
  
  @Override
  public void creeObjet() {
		MtgInternalFrame cadre = frame.cadreActif();
		CListeObjets liste = cadre.listePr;
		CValeurAngle val;
		double ang;
		if (liste.uniteAngle == UniteAngle.Radian) {
			ang = 2*Math.PI/nombreSommets;
			val = new CValeurAngle(liste, ang, "2*pi/"+Integer.toString(nombreSommets));
		}
		else {
			ang = 360d/(double)nombreSommets;
			val = new CValeurAngle(liste, ang, "360/"+Integer.toString(nombreSommets));
		}
		CPointAncetre point = point2;
		CRotation rot = new CRotation(liste, null, false, point1, val);
		frame.ajouteElement(rot);
		Vector<CNoeudPointeurSurPoint> vec = new Vector<CNoeudPointeurSurPoint>(nombreSommets);
		vec.add(new CNoeudPointeurSurPoint(liste, point2));
		for (int i = 1; i < nombreSommets; i++) {
			CPointImage ptim = new CPointImage(liste, null, false, frame.couleurActive(), false,
					0d, 0d, false, frame.genereNomPourPoint(), frame.pref_indiceTaillePoliceNom, frame.stylePointActif, point, rot, false);
			frame.ajouteElement(ptim);
			CNoeudPointeurSurPoint noeud = new CNoeudPointeurSurPoint(liste, ptim);
			vec.add(noeud);
			point = ptim;
		}
		vec.add(new CNoeudPointeurSurPoint(liste, point2));
		CPolygone poly = new CPolygone(liste, vec, frame.couleurActive(), frame.styleTraitActif());
		frame.ajouteElement(poly);
    frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil138"));
    annuleClignotement();
    cadre.repaintFigureAvecCalcul();    
    frame.activeOutilCapture();
  }
  
  @Override
  public void ajouteObjetsVisuels() {
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
		CListeObjets liste = frame.listeObjetsVisuels;
		liste.uniteAngle = frame.cadreActif().listePr.uniteAngle;
		liste.ajouteConstantePi();
		CValeurAngle val;
		double ang;
		if (liste.uniteAngle == UniteAngle.Radian) {
			ang = 2*Math.PI/nombreSommets;
			val = new CValeurAngle(liste, ang, "2*pi/"+Integer.toString(nombreSommets));
		}
		else {
			ang = 360d/(double)nombreSommets;
			val = new CValeurAngle(liste, ang, "360/"+Integer.toString(nombreSommets));
		}
		CPointAncetre point = pane.pointSouris;
		CRotation rot = new CRotation(liste, null, false, point1, val);
		frame.ajouteObjetVisuel(rot);
		Vector<CNoeudPointeurSurPoint> vec = new Vector<CNoeudPointeurSurPoint>(nombreSommets);
		vec.add(new CNoeudPointeurSurPoint(liste, pane.pointSouris));
		for (int i = 1; i < nombreSommets; i++) {
			CPointImage ptim = new CPointImage(liste, null, false, frame.couleurActive(), false,
					0d, 0d, false, "", frame.pref_indiceTaillePoliceNom, frame.stylePointActif, point, rot, false);
			frame.ajouteObjetVisuel(ptim);
			CNoeudPointeurSurPoint noeud = new CNoeudPointeurSurPoint(liste, ptim);
			vec.add(noeud);
			point = ptim;
		}
		vec.add(new CNoeudPointeurSurPoint(liste, pane.pointSouris));
		CPolygone poly = new CPolygone(liste, vec, frame.couleurActive(), frame.styleTraitActif());
		frame.ajouteObjetVisuel(poly);		
  }
  
  public String chaineIndication (int indEtape) {
    switch(indEtape) {
      case 1 : return getString("PolygoneRegulier1");
      case 2 : return getString("PolygoneRegulier2");
      default: return "";
    }
  }
  
}
