/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import java.util.ListIterator;
import java.util.Vector;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CElementBase;
import objets.CElementGraphique;
import objets.CMacroAvecListe;
import objets.CSousListeObjets;
import types.Bool;
import types.NatureObjet;
import dialogs.ChoixMacroAvecListeDlg;

public class OutilRetraitObjetsMacro extends Outil {
  private CSousListeObjets sousListe = null; // Pointera sur les ojets  retirer de la macro
  private Vector<Bool> masquages = null;
  private CMacroAvecListe macro; // La macro laquelle on ajoute des objest

  public OutilRetraitObjetsMacro(MtgFrame frameInit) {
    super(frameInit, true);
  }
    
  @Override
	public void select() {
    super.select();
    frame.indication(getString("retraitObjetsMacro1"));    
    ChoixMacroAvecListeDlg dlg = new ChoixMacroAvecListeDlg(frame);
		dlg.setLocationRelativeTo(null);
    dlg.setVisible(true);
    if (!dlg.retourOK()) {
    	frame.activeOutilCapture();
    	return;
    }
    macro = dlg.macroChoisie;
  	sousListe = new CSousListeObjets();
  	masquages = new Vector<Bool>();
    // on mmorise si les lments traits par la macro sont cachs ou non
  	// et on les rajoute  la liste clignotante
    ListIterator<CElementBase> li = macro.listeAssociee.listIterator();
    while (li.hasNext()) {
    	CElementGraphique elg = (CElementGraphique) li.next();
    	Bool b = new Bool(elg.masque);
    	masquages.add(b);
    	ajouteClignotementDe(elg);
    }
    // On rend tous les ojjets visibles mms s'ils sont masqus
    macro.listeAssociee.montreTout(true);
    frame.outilPointageActif = frame.outilPointageObjetClignotant;
    frame.outilPointageActif.designationDe(NatureObjet.NToutObjet);
    frame.outilPointageActif.reset();
    frame.indication(getString("retraitObjetsMacro2"));
  }
  
  @Override
  public void deselect() {
  	super.deselect();
		// On remet le masquage des lments  son tat initial
  	if (masquages != null) {
	  	if (masquages.size() != 0) {
	  		// On remet le masquage des lments  son tat initial
	  		ListIterator<CElementBase> li = macro.listeAssociee.listIterator();
	  		int i = 0;
	  		while (li.hasNext()) {
	  			CElementGraphique elg = (CElementGraphique) (li.next());
	  			elg.masque = masquages.get(i).getValue();
	  			i++;
	  		}
	  	}
  		masquages.removeAllElements();
  		sousListe.retireTout();
  		masquages = null;
  		sousListe = null;	  	
  	}
  }
	
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
  	if (sousListe.size() == macro.listeAssociee.size() - 1) return;
  	annuleClignotement();
  	frame.listeExclusionDesignation.add(elg);
    ListIterator<CElementBase> li = macro.listeAssociee.listIterator();
    while (li.hasNext()) {
    	CElementGraphique elg2 = (CElementGraphique) li.next();
    	if (!frame.listeExclusionDesignation.contains(elg2)) ajouteClignotementDe(elg2);
    }
  	resetClignotement();
  	sousListe.add(elg);
  }
	
  @Override
  public void creeObjetApresClicDroit() {
  	if (sousListe.size() != 0) {
   		MtgInternalFrame cadre = frame.cadreActif();
  		// On remet le masquage des lments  son tat initial
   		// Ncessaire avant de saubegarder l'tat de la figure mme si ce sera fait par deselect
  		ListIterator<CElementBase> li = macro.listeAssociee.listIterator();
  		int i = 0;
  		while (li.hasNext()) {
  			CElementGraphique elg = (CElementGraphique) (li.next());
  			elg.masque = masquages.get(i).getValue();
  			i++;
  		}
  		li = sousListe.listIterator();
  		i = 0;
  		while (li.hasNext()) {
  			CElementBase elb = li.next();
  			macro.listeAssociee.removeAndKeepComposant(elb);
  		}
  		annuleClignotement();
		  cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil118"));
  	}
  	masquages.removeAllElements();
  	sousListe.retireTout();
  	frame.activeOutilCapture();
  }
  
  @Override
  public boolean activationValide() {
  	// Modifi version Applet
    // return frame.cadreActif().listePr.nombreMacrosAvecListe()!= 0;
    return frame.listePourConstruction().nombreMacrosAvecListe()!= 0;
  }
}
